//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#ifndef __QMOL_3D_OBJ
#define __QMOL_3D_OBJ

#include "TrackBall.h"
#include "QmolUtil.h"

#ifndef X
#define	X	0
#endif

#ifndef Y
#define	Y	1
#endif

#ifndef Z
#define	Z	2
#endif

// Look, a class called "DDD" -- get it?
class DDD: public VirtualTrackBall{
private:
	int beginRotationX, beginRotationY;
	int beginMoveX, beginMoveY;
	int beginScaleX;
	int num_rotation;

	float translationFrame[4][4];
	BOOL isDynamic;

public:

	// The current location of the center of mass of the
	// three dimensional object
	float R[3];

	DDD()
	{
		isDynamic = TRUE;
		num_rotation = 0;

		R[X] = R[Y] = R[Z] = 0.0f;
	};

	inline void SetDynamic(BOOL m_isDynamic)
	{
		isDynamic = m_isDynamic;
	};

	inline BOOL GetDynamic()
	{
		return isDynamic;
	};

	inline void ToggleDynamic()
	{
		isDynamic = !isDynamic;
	};

	inline BOOL HasRotated()
	{
		return (num_rotation != 0);
	};

	inline void Update()
	{
		if(!isDynamic)
			return;

		VirtualTrackBall::update();
	};

	inline void Update(float a, float b, float c, float d)
	{
		if(!isDynamic)
			return;
		
		VirtualTrackBall::trackball(a, b, c, d);
		VirtualTrackBall::update();
	};

	inline void Translate(float x, float y, float z)
	{
		if(!isDynamic)
			return;

		R[X] = x;
		R[Y] = y;
		R[Z] = z;
	};

	inline void Rotation(int x, int y, int width, int height)
	{
		if(!isDynamic)
			return;

		trackball((2.0f*beginRotationX - width)/width,
    			  (height - 2.0f*beginRotationY)/height,
    			  (2.0f*x - width)/width,
    			  (height - 2.0f*y)/height);
  	
		beginRotationX = x;
  		beginRotationY = y;

		num_rotation++;
	};

	inline void StartRotation(int x, int y)
	{
		if(!isDynamic)
			return;

		beginRotationX = x;
  		beginRotationY = y;

		num_rotation = 0;
	};

	inline void StartMove(int x, int y)
	{
		if(!isDynamic)
			return;

		beginMoveX = x;
		beginMoveY = y;

		VirtualTrackBall::build_rotmatrix(translationFrame);
		InvertMatrix(translationFrame);
	};

	inline void Move(int x, int y, float m_frac)
	{
		if(!isDynamic)
			return;

		// ** Note the minus signs to flip the Y axes ** 
		float offsetX = (3.0f*(x - beginMoveX))*m_frac;
		float offsetY = -(3.0f*(y - beginMoveY))*m_frac;
	
		beginMoveX = x;
		beginMoveY = y;

		// Compute the new translation vector here
		R[X] += translationFrame[X][X]*offsetX + translationFrame[Y][X]*offsetY;
		R[Y] += translationFrame[X][Y]*offsetX + translationFrame[Y][Y]*offsetY;
		R[Z] += translationFrame[X][Z]*offsetX + translationFrame[Y][Z]*offsetY;
	};

	inline void StartScale(int x)
	{
		if(!isDynamic)
			return;

		beginScaleX = x;

		VirtualTrackBall::build_rotmatrix(translationFrame);
		InvertMatrix(translationFrame);
	};

	inline void Scale(int x, float m_frac)
	{
		if(!isDynamic)
			return;

		float offsetZ = (3.0f*(x - beginScaleX))*m_frac;
	
		beginScaleX = x;

		// Compute the new translation vector here
		R[X] += translationFrame[Z][X]*offsetZ;
		R[Y] += translationFrame[Z][Y]*offsetZ;
		R[Z] += translationFrame[Z][Z]*offsetZ;
	};

	inline void BuildRotationMatrix(GLfloat m[][4])
	{
		VirtualTrackBall::build_rotmatrix(m);
	};
};


#endif // __QMOL_3D_OBJ