// MoleculeSelectDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Qmol.h"
#include "MoleculeSelectDialog.h"
#include "Excepts.h"
#include "QmolDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMoleculeSelectDialog dialog


CMoleculeSelectDialog::CMoleculeSelectDialog(CWnd* pParent)
	: CDialog(CMoleculeSelectDialog::IDD, pParent)
{
	pView = (CView*)pParent;

	//{{AFX_DATA_INIT(CMoleculeSelectDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CMoleculeSelectDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMoleculeSelectDialog)
	DDX_Control(pDX, IDC_MOL_LIST, m_molList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMoleculeSelectDialog, CDialog)
	//{{AFX_MSG_MAP(CMoleculeSelectDialog)
	ON_BN_CLICKED(IDC_DELETE_BUTTON, OnDeleteButton)
	ON_BN_CLICKED(IDC_DYNAMIC_BUTTON, OnDynamicButton)
	ON_BN_CLICKED(IDC_PRIMARY_BUTTON, OnPrimaryButton)
	ON_MESSAGE(WM_MOL_SELECT_UPDATE, UpdateSelection)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_SELECT_BUTTON, OnSelectButton)
	ON_BN_CLICKED(IDC_CLONE_BUTTON, OnCloneButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMoleculeSelectDialog message handlers

BOOL CMoleculeSelectDialog::Create()
{
	return CDialog::Create(CMoleculeSelectDialog::IDD);
}

BOOL CMoleculeSelectDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Initialize the list control
	// Initialize the sequence list  control
	// Build the columns in the parameter list control
	LVCOLUMN tmpCol;
	CDC *pDC;
	pDC = m_molList.GetDC();

	tmpCol.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	tmpCol.fmt = LVCFMT_LEFT;
	tmpCol.pszText = "Molecule";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(3.5*pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	tmpCol.iSubItem = 0;	// Ignored
	tmpCol.iImage = 0;		// Ignored
	tmpCol.iOrder = 0;		// Ignored

	m_molList.InsertColumn(0, &tmpCol);

	tmpCol.pszText = "Status";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(2.5f*pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	m_molList.InsertColumn(1, &tmpCol);

	tmpCol.pszText = "Selected";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(1.1f*pDC->GetTextExtent("X", tmpCol.cchTextMax).cx);

	m_molList.InsertColumn(2, &tmpCol);

	tmpCol.pszText = "Atoms";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);
	tmpCol.cx = (int)(1.1f*pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	m_molList.InsertColumn(3, &tmpCol);

	tmpCol.pszText = "Frames";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);
	tmpCol.cx = (int)(pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	m_molList.InsertColumn(4, &tmpCol);

	m_molList.ReleaseDC(pDC);

	// Make selection extend across an entire row
	m_molList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	UpdateSelection(0, 0);

	// Give the focus to the list control
	return FALSE;  // return TRUE unless you set the focus to a control
	               // EXCEPTION: OCX Property Pages should return FALSE
}

LRESULT CMoleculeSelectDialog::UpdateSelection(WPARAM wParam, LPARAM lParam)
{
	// Pack the list control with sequence data
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	// Remove any pre-existing items from the list control
	m_molList.DeleteAllItems();

	CString buffer;
	LVITEM tmpLVITEM;
		
	// Zero out the LVITEM structure
	memset(&tmpLVITEM, 0, sizeof(LVITEM));

	// Save the index of the primary molecule
	int primaryIndex = -1;

	for(int i = 0;i < pDoc->molList.Length();i++){

		if(pDoc->molList[i] == pDoc->activeMol){
			primaryIndex = i;
		}

		// Add the Atom name
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 0; // The item itself
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		buffer.Format("%d) %s", i+1, pDoc->molList[i]->coordName);
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_molList.InsertItem(&tmpLVITEM);
		
		// Add the status
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 1; // The molecule status
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		if(pDoc->molList[i]->GetDynamic()){
			if(pDoc->molList[i] == pDoc->activeMol){
				buffer.Format("Primary, Free");
			}
			else{
				buffer.Format("Free");
			}
		}
		else{
			if(pDoc->molList[i] == pDoc->activeMol){
				buffer.Format("Primary, Fixed");;
			}
			else{
				buffer.Format("Fixed");
			}
		}

		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_molList.SetItem(&tmpLVITEM);
		
		// Show the selection idicator
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 2; // The selection state
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		if(pDoc->molList[i]->isSelected)
			buffer.Format("X");
		else
			buffer.Format(" ");
		
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_molList.SetItem(&tmpLVITEM);

		// Add the number of atoms
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 3; // The number of atoms
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		buffer.Format("%d", pDoc->molList[i]->NumAtoms());
		
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_molList.SetItem(&tmpLVITEM);
		
		// Add the number of animation frames
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 4; // The number of animation frames
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		buffer.Format("%d", pDoc->molList[i]->steps);
		
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_molList.SetItem(&tmpLVITEM);
	}
	
	if((pDoc->molList.Length() > 0) && (primaryIndex != -1)){
		ListView_SetItemState(m_molList.GetSafeHwnd(),
			primaryIndex,
			LVIS_SELECTED,
			LVIS_SELECTED);
	}

	// Give the list control the focus
	m_molList.SetFocus();

	return 0L;
}

// Return the first selected molecule
int CMoleculeSelectDialog::GetSelectedMol()
{
	// Get the total number of selected molecules
	POSITION pos = m_molList.GetFirstSelectedItemPosition();	
	
	if(!pos)
		// No molecule has been selected
		return -1;

	return m_molList.GetNextSelectedItem(pos);
}

void CMoleculeSelectDialog::GetSelectedMol(list<int> &m_molSel)
{
	// Clean up the existing list if needed
	while(!m_molSel.IsEmpty())
		m_molSel.Pop();

	// Get the total number of selected molecules
	POSITION pos = m_molList.GetFirstSelectedItemPosition();	

	while(pos)
		m_molSel.Push(m_molList.GetNextSelectedItem(pos));
}

void CMoleculeSelectDialog::OnCancel() 
{
	if(pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		pView->PostMessage(WM_MOL_SELECT_DIALOG, IDCANCEL);
	}
}

void CMoleculeSelectDialog::OnClose() 
{
	if(pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		pView->PostMessage(WM_MOL_SELECT_DIALOG, IDCANCEL);
	}
}

void CMoleculeSelectDialog::OnDeleteButton() 
{
	if(pView != NULL){
		pView->PostMessage(WM_MOL_SELECT_DIALOG, WM_MOL_SELECT_DELETE);
	}
}

void CMoleculeSelectDialog::OnDynamicButton() 
{
	if(pView != NULL){
		pView->PostMessage(WM_MOL_SELECT_DIALOG, WM_MOL_SELECT_DYNAMIC);
	}
}

void CMoleculeSelectDialog::OnPrimaryButton() 
{
	if(pView != NULL){
		pView->PostMessage(WM_MOL_SELECT_DIALOG, WM_MOL_SELECT_ACTIVE);
	}
}

void CMoleculeSelectDialog::OnSelectButton() 
{
	if(pView != NULL){
		pView->PostMessage(WM_MOL_SELECT_DIALOG, WM_MOL_SELECT_SELECTION);
	}
}


void CMoleculeSelectDialog::OnCloneButton() 
{
	if(pView != NULL){
		pView->PostMessage(WM_MOL_SELECT_DIALOG, WM_MOL_SELECT_CLONE);
	}
}

BOOL CMoleculeSelectDialog::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// Don't process IDOK messages sent from the list control
	if(wParam == IDOK)
		return 0;

	return CDialog::OnCommand(wParam, lParam);
}
