//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#ifndef __QMOL_MOL_STATE
#define __QMOL_MOL_STATE

#include "AdjustTorsion.h"
#include "QmolDefine.h"

// Store the current display state of the molecule
class moleculeState{
	public:
	
	moleculeState()
	{
		// Set defaults
		coordFileType = QM_PDB;
		trajecFileType = QM_DCD;
		coordName = NULL;
		trajName = NULL;
		current_step = 0;
		solvent = TRUE;
		displayType = QM_RENDER_WIREFRAME;
		showWireFrame = FALSE;
		
		atom_labels = FALSE;
		monitors = TRUE;
		monitorLabels = TRUE;

		isSelected = FALSE;

		centerX = 0.0f;
		centerY = 0.0f;
		centerZ = 0.0f;

		offsetX = 0.0f;
		offsetY = 0.0f;
		offsetZ = 0.0f;

		trajProcess = QM_CENTER_ALL;
		atom_radius_method = QM_RADIUS_DEFAULT;
	};
	
	~moleculeState()
	{
		if(coordName)
			delete [] coordName;
			
		if(trajName)
			delete [] trajName;
	};
	
	moleculeState& operator=(moleculeState &copy)
	{
		coordFileType = copy.coordFileType;
		trajecFileType = copy.trajecFileType;
		atom_radius_method = copy.atom_radius_method;
		displayType = copy.displayType;
		showWireFrame = copy.showWireFrame;
		isSelected = copy.isSelected;
		solvent = copy.solvent;
		atom_labels = copy.atom_labels;
		monitors = copy.monitors;
		monitorLabels = copy.monitorLabels;
		current_step = copy.current_step; 
		centerX = copy.centerX;
		centerY = copy.centerY;
		centerZ = copy.centerZ;
		offsetX = copy.offsetX;
		offsetY = copy.offsetY;
		offsetZ = copy.offsetZ;
		trajProcess = copy.trajProcess;

		SetCoordName(copy.coordName);
		SetTrajName(copy.trajName);

		return (*this);
	};

	void SetCoordName(const char *name)
	{
		if(coordName)
			delete [] coordName;
		
		if(name){
			coordName = new char[strlen(name) + 1];
			strcpy(coordName, name);
		}
		else{
			coordName = NULL;
		}
	};
	
	void SetTrajName(char *name)
	{
		if(trajName)
			delete [] trajName;
		
		if(name){
			trajName = new char[strlen(name) + 1];
			strcpy(trajName, name);
		}
		else{
			trajName = NULL;
		}
	};
	
	inline void ToggleSelection()
	{
		isSelected = !isSelected;
	};

	// File types (pdb, xyz, etc)
	int coordFileType, trajecFileType;
	
	RADIUS_FORMAT atom_radius_method;

	// Should we display atomic Van der Waals radii?
	RENDER_FORMAT displayType;
	
	// Show we superimpose a wireframe structure over the selected ribbon
	// diagram?
	BOOL showWireFrame;

	// Is the molecule currently selected?
	BOOL isSelected;

	// Display solvent and/or any ligand molecules
	int solvent;
	
	// Display atom labels?
	int atom_labels;
	
	// Display atom monitors?
	int monitors;

	// Display atom monitor labels?
	int monitorLabels;

	// The current structure from the trajectory
	int current_step; 

	// The name of the coordinate file
	char *coordName;
		
	// The name of the trajectory file
	char *trajName;
	
	// The center of the molecule.
	float centerX, centerY, centerZ;
	
	// The offset due to user translation of the molecule
	float offsetX, offsetY, offsetZ;

	// How should the trajectory be processed during/after reading?
	int trajProcess;
};

#endif // __QMOL_MOL_STATE