//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html).
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//

#include "stdafx.h"
#include "QmolDoc.h"
#include "OpenCoordDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COpenCoordDialog dialog


COpenCoordDialog::COpenCoordDialog(CWnd* pParent /*=NULL*/)
	: CDialog(COpenCoordDialog::IDD, pParent)
{
	// The default filename
	filename = _T("");

	// The default file type
	filetype = QM_PDB;

	//{{AFX_DATA_INIT(COpenCoordDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

BOOL COpenCoordDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_file_type.SetCurSel(filetype);

	return TRUE;
}

void COpenCoordDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COpenCoordDialog)
	DDX_Control(pDX, IDC_FILE_NAME, m_file_name);
	DDX_Control(pDX, IDC_OPEN_COORD_COMBO, m_file_type);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COpenCoordDialog, CDialog)
	//{{AFX_MSG_MAP(COpenCoordDialog)
	ON_BN_CLICKED(IDC_OPEN_COORD_BROWSE, OnOpenCoordBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenCoordDialog message handlers

void COpenCoordDialog::OnOpenCoordBrowse() 
{
	// Get the current file type (to set the filter in the
	// open file dialog).
	filetype = m_file_type.GetCurSel();

	// Open a file dialog here to load a file name
	CFileDialog fDlg(TRUE);
	
	switch(filetype){
		case QM_PDB:
			fDlg.m_ofn.lpstrFilter = 
				"PDB files\0*.pdb\0All files\0*.*\0\0";
			break;
		case QM_XYZ:
			fDlg.m_ofn.lpstrFilter = 
				"XYZ files\0*.xyz\0All files\0*.*\0\0";
			break;
		case QM_CRD:
			fDlg.m_ofn.lpstrFilter = 
				"CRD files\0*.crd\0All files\0*.*\0\0";
			break;
	};

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		// Load the selected file name into the filename
		// text box.
		filename = fDlg.GetPathName();
		m_file_name.SetWindowText(filename);
	}
}

void COpenCoordDialog::OnCancel() 
{	
	CDialog::OnCancel();
}

void COpenCoordDialog::OnOK() 
{
	// Save the file name and type
	m_file_name.GetWindowText(filename);
	filetype = m_file_type.GetCurSel();

	CDialog::OnOK();
}

CString COpenCoordDialog::GetFileName()
{
	return filename;
}

int COpenCoordDialog::GetFileType()
{
	return filetype;
}
