//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 

#ifndef __QMOL_MOVIE
#define __QMOL_MOVIE

#include "vfw.h"

// All of the info that is needed for creating an AVI movie
class QmolMovie{
public:
	QmolMovie()
	{
		valid = FALSE;

		pView = NULL;

		hRC = NULL;
		hMemRC = NULL;

		hDC = NULL;
		hMemDC = NULL;

		hDib = NULL;
		hOldDib = NULL;
		
		pBitmapBits = NULL;
		bitmapSize = 0;

		filename = _T("");

		frames_written = 0;
		width = 0;
		height = 0;
		fps = 0;
		bpp = 0;

		aviFile = NULL;
		aviStream = NULL;
		aviStreamCompressed = NULL;
		
		// Fill in the header for the video stream
		memset(&aviInfo, 0, sizeof(aviInfo));
	};
	
	~QmolMovie()
	{
		Close();
	}
	
	inline BOOL IsValid()
	{
		return valid;
	};

	// Return FALSE if the user cancels the compression dialog,
	// TRUE otherwise.
	BOOL Init(CString &m_filename, int m_width, int m_height,
		int m_fps, int m_bpp, CView *m_pView, BOOL m_update_surf);
	
	void Render();

	void Close();
private:
	// Has the class been properly initialized?
	BOOL valid;

	// A pointer to the view
	CView *pView;

	// The name of the output file
	CString filename;

	HGLRC hRC;
	HGLRC hMemRC;

	HDC hDC;
	HDC hMemDC;

	HBITMAP hDib;
	HGDIOBJ hOldDib;
	
	LPVOID pBitmapBits;
	int bitmapSize;

	// The number of frames written so far
	int frames_written;

	// The width an height of the output
	int width;
	int height;

	// Update the surface every step?
	BOOL update_surf;

	// The frame rate
	int fps;

	// The bits-per-pixel
	int bpp;

	// AVI stream info
	PAVIFILE aviFile;
	PAVISTREAM aviStream;
	PAVISTREAM aviStreamCompressed;
	AVISTREAMINFO aviInfo;

	// Information for the compression engine
	BITMAPINFO bmpInfoInput;
	BITMAPINFO bmpInfoOutput;
	
	COMPVARS compressOpts;
};

#endif // __QMOL_MOVIE