// SurfDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Qmol.h"
#include "SurfDialog.h"
#include "QmolView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSurfDialog

IMPLEMENT_DYNAMIC(CSurfDialog, CPropertySheet)

CSurfDialog::CSurfDialog(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
	m_pWnd = pParentWnd;

	// Add the property pages
	optPage.Construct(IDD_SURF_OPT_SHEET);
	clrPage.Construct(IDD_SURF_CLR_SHEET);

	optPage.m_pWnd = pParentWnd;
	clrPage.m_pWnd = pParentWnd;

	AddPage(&optPage);
	AddPage(&clrPage);
}

CSurfDialog::CSurfDialog(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	m_pWnd = pParentWnd;

	// Add the property pages
	optPage.Construct(IDD_SURF_OPT_SHEET);
	clrPage.Construct(IDD_SURF_CLR_SHEET);

	optPage.m_pWnd = pParentWnd;
	clrPage.m_pWnd = pParentWnd;

	AddPage(&optPage);
	AddPage(&clrPage);
}

CSurfDialog::~CSurfDialog()
{
}


BEGIN_MESSAGE_MAP(CSurfDialog, CPropertySheet)
	//{{AFX_MSG_MAP(CSurfDialog)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_SURF_CANCEL, OnCancel)
	ON_COMMAND(ID_SURF_OK, OnOk)
	ON_COMMAND(ID_SURF_APPLY, OnApply)
	ON_MESSAGE(WM_SURF_OPT, OnApply)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSurfDialog message handlers
BOOL CSurfDialog::OnInitDialog()
{
	CPropertySheet::OnInitDialog();

	RECT rc;

	GetWindowRect (&rc);
	// Increase the height of the CPropertySheet by 30
	rc.bottom += 30;

	// Resize the CPropertySheet
	MoveWindow (rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top);
	// Convert to client coordinates
	ScreenToClient (&rc);

	int space = 20;
	int width = (rc.right - rc.left - 4*space)/3;

	m_ApplyButton.Create ("Apply", WS_CHILD | WS_VISIBLE | WS_TABSTOP,
		CRect (space, rc.bottom-30, width + space, rc.bottom-5),
		this, ID_SURF_APPLY);
	
	m_OkButton.Create ("OK", WS_CHILD | WS_VISIBLE | WS_TABSTOP,
		CRect (2*space + width, rc.bottom-30, 2*space + 2*width, rc.bottom-5),
		this, ID_SURF_OK);

	m_CancelButton.Create ("Cancel", WS_CHILD | WS_VISIBLE | WS_TABSTOP,
		CRect (3*space + 2*width, rc.bottom-30, 3*space + 3*width, rc.bottom-5),
		this, ID_SURF_CANCEL);

	// Set the default font (from the property sheets default font).
	m_ApplyButton.SetFont(GetFont());
	m_OkButton.SetFont(GetFont());
	m_CancelButton.SetFont(GetFont());

	return TRUE;
}

void CSurfDialog::OnOk()
{
	// Update the data on the render and color pages
	if(optPage.GetSafeHwnd())
		if(!optPage.UpdateData())
			return;

	if(clrPage.GetSafeHwnd())
		if(!clrPage.UpdateData())
			return;

	// Make sure that the color dialog window does not remain open
	if(clrPage.m_flexClrDlg->GetSafeHwnd() != 0){
		clrPage.m_flexClrDlg->DestroyWindow();
	}

	if(m_pWnd){
		m_pWnd->PostMessage(WM_SURF_OPT, ID_SURF_OK);
	}
}

void CSurfDialog::OnCancel()
{
	// Make sure that the color dialog window does not remain open
	if(clrPage.m_flexClrDlg->GetSafeHwnd() != 0){
		clrPage.m_flexClrDlg->DestroyWindow();
	}

	if(m_pWnd){
		m_pWnd->PostMessage(WM_SURF_OPT, ID_SURF_CANCEL);
	}
}

void CSurfDialog::OnApply()
{
	// Update the data on the render and color pages
	if(optPage.GetSafeHwnd())
		if(!optPage.UpdateData())
			return;

	if(clrPage.GetSafeHwnd())
		if(!clrPage.UpdateData())
			return;

	if(m_pWnd){
		m_pWnd->PostMessage(WM_SURF_OPT, ID_SURF_APPLY);
	}
}


int CSurfDialog::GetColorStyle()
{
	// Currently CSurfColorProperty::COLOR_BY_ATOM or
	// CSurfColorProperty::COLOR_USER

	return clrPage.color_style;
}

void CSurfDialog::SetColorStyle(int m_style)
{
	// Currently CSurfColorProperty::COLOR_BY_ATOM or
	// CSurfColorProperty::COLOR_USER

	if(m_style == CSurfColorProperty::COLOR_BY_ATOM){
		clrPage.color_style = CSurfColorProperty::COLOR_BY_ATOM;
	}
	else{
		clrPage.color_style = CSurfColorProperty::COLOR_USER;
	}
}

int CSurfDialog::GetAlpha()
{
	if(clrPage.m_enable_alpha){
		return clrPage.m_alpha;
	}

	// By default (unless the user has enabled transparency)
	// return an opaque alpha level.
	return 255;
}

void CSurfDialog::SetAlpha(int m_alpha)
{
	clrPage.m_alpha = m_alpha;
}

void CSurfDialog::GetRGBA(int &r, int &g, int &b, int &a)
{
	r = clrPage.m_r;
	g = clrPage.m_g;
	b = clrPage.m_b;
	a = clrPage.m_a;
}

void CSurfDialog::SetRGBA(int r, int g, int b, int a)
{
	clrPage.m_r = r;
	clrPage.m_g = g;
	clrPage.m_b = b;
	clrPage.m_a = a;
}

int CSurfDialog::GetSurfQuality()
{
	return optPage.m_res;
}

void CSurfDialog::SetSurfQuality(int m_quality)
{
	optPage.m_res = m_quality;
}

int CSurfDialog::GetSurfType()
{
	// Only one possiblity
	//if(optPage.m_surf_type == CSurfOptProperty::VDW)
	return MolSurface::HEURISTIC;

}

void CSurfDialog::SetSurfType(int m_type)
{
	//if(m_type == MolSurface::HEURISTIC){
		optPage.m_surf_type = CSurfOptProperty::VDW;
	//}
}

void CSurfDialog::UseSolventAtoms(BOOL m_solvent)
{
	optPage.m_use_solvent = m_solvent;
}

BOOL CSurfDialog::UseSolventAtoms()
{
	return optPage.m_use_solvent;
}

float CSurfDialog::GetProbeRadius()
{
	return optPage.m_probe_radius;
}

void CSurfDialog::SetProbeRadius(float m_radius)
{
	optPage.m_probe_radius = m_radius;
}

void CSurfDialog::SetColorFocus()
{
	SetActivePage(&clrPage);
}

void CSurfDialog::SetRenderFocus()
{
	SetActivePage(&optPage);
}


