#ifndef __ARRAY
#define __ARRAY

#include "Excepts.h"

#ifndef _DEBUG
#define ARRAY_NO_BOUND
#endif

template <class A>
class array{
	private:
		A *top;
		int len;
	public:
		array()
		{
			top = NULL;
			len = 0;
		};

		array(int n)
		{
			len = n;

			if(len > 0)
				top = new A [n];
			else
				top = NULL;
		};
	
		array(const array<A> &copy)
		{
			top = NULL;

			(*this) = copy;
		};
		
		~array()
		{
			if(top != NULL){
				delete [] top;
				top = NULL;
			}
		};

		inline A& operator[](int i)
		{
			#ifndef ARRAY_NO_BOUND
			if((i < 0) || (i >= len))
				throw QmolError("Index out of bounds in array!");
			#endif // ARRAY_NO_BOUND

			return top[i];
		};

		inline A* Pointer()
		{
			return top;
		};

		array<A>& operator=(const array<A> &copy)
		{
			if(top){
				delete [] top;
			}

			len = copy.len;

			if(len > 0)
				top = new A [copy.len];
			else
				top = NULL;

			for(int i = 0;i < len;i++)
				top[i] = copy.top[i];

			return (*this);
		}

		// Initialize all elements of the array to the same
		// value. This function is needed for backwards compatibility
		// with the vector class that was used previously.
		array<A>& operator=(const A &init)
		{
			for(int i = 0;i < len;i++)
				top[i] = init;

			return (*this);
		}

		inline int Length() const 
		{
			return len;
		};

		inline void Alloc(int n)
		{
			if(top)
				delete [] top;
			
			len = n;

			if(len > 0){
				top = new A [n];

				if(top == NULL)
					throw QmolError("Out of memory");
			}
			else{
				top = NULL;
			}
		};
};

#endif // __ARRAY