//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include <afxpriv.h>
#include <float.h>
#include <math.h>
#include "Qmol.h"

#include "QmolDoc.h"
#include "QmolView.h"
#include "MainFrm.h"
#include "Excepts.h"
#include "QmolUtil.h"
#include "PrintOption.h"
#include "RenderMovieDialog.h"
#include "TextOutput.h"
#include "ExportTrajDialog.h"
#include "PasteDialog.h"

// for MM timers (you'll need WINMM.LIB)
#include <mmsystem.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#ifdef _DEBUG
BOOL debug_flag = FALSE;
#endif //_DEBUG

#define		DOT3D(VEC1, VEC2)		(VEC1[0]*VEC2[0] + VEC1[1]*VEC2[1] + VEC1[2]*VEC2[2])

#define		X	0
#define		Y	1
#define		Z	2

// Prevent the smoothly interpolated ribbon diameter from getting to small
// (or even negative).
#define		RIBBON_SIZE_OFFSET		0.25f

// Global variables!
extern int g_bpp;
extern int g_z_buf;
extern int g_purge_mru;

/////////////////////////////////////////////////////////////////////////////
// CQmolView

IMPLEMENT_DYNCREATE(CQmolView, CView)

BEGIN_MESSAGE_MAP(CQmolView, CView)
	//{{AFX_MSG_MAP(CQmolView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_ADJUST_TORSIONANGLE, OnAdjustTorsionangle)
	ON_MESSAGE(WM_TOR_CLOSE, OnAdjustTorsionangleClose)
	ON_MESSAGE(WM_TOR_ADJUST, OnAdjustTorsionangleModify)
	ON_COMMAND(ID_OPTIONS_MEASURE, OnOptionsMeasure)
	ON_MESSAGE(WM_MEASURE_CLOSE, OnOptionsMeasureClose)
	ON_COMMAND(ID_OPTIONS_CENTER, OnOptionsCenter)
	ON_COMMAND(ID_RENDER_ATOMLABELS, OnRenderAtomLabels)
	ON_UPDATE_COMMAND_UI(ID_RENDER_ATOMLABELS, OnUpdateRenderAtomLabels)
	ON_COMMAND(ID_OPTIONS_FONT, OnOptionsFont)
	ON_COMMAND(ID_OPTIONS_COLOR, OnOptionsColor)
	ON_COMMAND(ID_RENDER_SOLVENT, OnRenderSolvent)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SOLVENT, OnUpdateRenderSolvent)
	ON_COMMAND(ID_OPTIONS_MOMENTUM, OnOptionsMomentum)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_MOMENTUM, OnUpdateOptionsMomentum)
	ON_COMMAND(ID_OPTIONS_ANIMATE, OnOptionsAnimate)
	ON_MESSAGE(WM_ANIMATE_CLOSE, OnAnimateClose)
	ON_MESSAGE(WM_ANIMATE_ADJUST, OnAnimateAdjust)
	ON_MESSAGE(WM_ANIMATE_FORWARD, OnAnimateForward)
	ON_MESSAGE(WM_ANIMATE_STOP, OnAnimateStop)
	ON_MESSAGE(WM_ANIMATE_REVERSE, OnAnimateReverse)
	ON_COMMAND(ID_RENDER_DEPTHCUE, OnRenderDepthCue)
	ON_UPDATE_COMMAND_UI(ID_RENDER_DEPTHCUE, OnUpdateRenderDepthCue)
	ON_COMMAND(ID_RENDER_CPK, OnRenderCpk)
	ON_UPDATE_COMMAND_UI(ID_RENDER_CPK, OnUpdateRenderCpk)
	ON_COMMAND(ID_RENDER_WIREFRAME, OnRenderWireFrame)
	ON_UPDATE_COMMAND_UI(ID_RENDER_WIREFRAME, OnUpdateRenderWireFrame)
	ON_COMMAND(ID_RENDER_ALIAS_LINE, OnRenderAliasLine)
	ON_UPDATE_COMMAND_UI(ID_RENDER_ALIAS_LINE, OnUpdateRenderAliasLine)
	ON_COMMAND(ID_RENDER_HIGH_RES, OnRenderHighRes)
	ON_UPDATE_COMMAND_UI(ID_RENDER_HIGH_RES, OnUpdateRenderHighRes)
	ON_COMMAND(ID_RENDER_LOW_RES, OnRenderLowRes)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LOW_RES, OnUpdateRenderLowRes)
	ON_COMMAND(ID_RENDER_MED_RES, OnRenderMedRes)
	ON_UPDATE_COMMAND_UI(ID_RENDER_MED_RES, OnUpdateRenderMedRes)
	ON_COMMAND(ID_RENDER_SMOOTH_3D, OnRenderSmooth3d)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SMOOTH_3D, OnUpdateRenderSmooth3d)
	ON_COMMAND(ID_RENDER_POINT, OnRenderPoint)
	ON_UPDATE_COMMAND_UI(ID_RENDER_POINT, OnUpdateRenderPoint)
	ON_COMMAND(ID_FILE_PRINT_OPTION, OnFilePrintOption)
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_RENDER_MONITOR, OnRenderMonitor)
	ON_UPDATE_COMMAND_UI(ID_RENDER_MONITOR, OnUpdateRenderMonitor)
	ON_COMMAND(ID_RENDER_STICK, OnRenderStick)
	ON_UPDATE_COMMAND_UI(ID_RENDER_STICK, OnUpdateRenderStick)
	ON_COMMAND(ID_RENDER_BALL_AND_STICK, OnRenderBallAndStick)
	ON_UPDATE_COMMAND_UI(ID_RENDER_BALL_AND_STICK, OnUpdateRenderBallAndStick)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_OPTIONS_ATOM_COLOR, OnOptionsAtomColor)
	ON_MESSAGE(WM_COLOR_CLOSE, OnOptionsColorClose)
	ON_COMMAND(ID_RENDER_AXES, OnRenderAxes)
	ON_UPDATE_COMMAND_UI(ID_RENDER_AXES, OnUpdateRenderAxes)
	ON_MESSAGE(WM_FLEX_COLOR, OnFlexColorMsg)
	ON_MESSAGE(WM_SEQUENCE_DIALOG, OnSequenceDialogMsg)
	ON_MESSAGE(WM_ATOM_SELECT_DIALOG, OnAtomSelectDialogMsg)
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(ID_ADJUST_RADIUS, OnAdjustRadius)
	ON_MESSAGE(WM_RADIUS_CLOSE, OnAdjustRadiusClose)
	ON_COMMAND(ID_OPTIONS_SELECT_COLOR, OnOptionsSelectColor)
	ON_COMMAND(ID_RENDER_LIGHTING, OnRenderLighting)
	ON_MESSAGE(WM_LIGHTING, OnLightingMessage)
	ON_COMMAND(ID_FILE_MOVIE, OnFileMovie)
	ON_UPDATE_COMMAND_UI(ID_FILE_MOVIE, OnUpdateFileMovie)
	ON_COMMAND(ID_FILE_COORD_OPEN, OnFileCoordOpen)
	ON_COMMAND(ID_FILE_COORD_SAVE, OnFileCoordSave)
	ON_COMMAND(ID_FILE_OPEN_COORD_TRAJ, OnFileOpenCoordTraj)
	ON_MESSAGE(WM_RESOLUTION, OnResolutionMessage)
	ON_COMMAND(ID_RENDER_CUSTOM_RES, OnRenderCustomRes)
	ON_UPDATE_COMMAND_UI(ID_RENDER_CUSTOM_RES, OnUpdateRenderCustomRes)
	ON_COMMAND(ID_RENDER_TRACE_STICK, OnRenderTraceStick)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TRACE_STICK, OnUpdateRenderTraceStick)
	ON_COMMAND(ID_RENDER_TRACE_WIREFRAME, OnRenderTraceWireframe)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TRACE_WIREFRAME, OnUpdateRenderTraceWireframe)
	ON_COMMAND(ID_RENDER_TRACE_SPLINE, OnRenderTraceSpline)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TRACE_SPLINE, OnUpdateRenderTraceSpline)
	ON_COMMAND(ID_RENDER_LINE_WIDTH_1, OnRenderLineWidth1)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINE_WIDTH_1, OnUpdateRenderLineWidth1)
	ON_COMMAND(ID_RENDER_LINE_WIDTH_2, OnRenderLineWidth2)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINE_WIDTH_2, OnUpdateRenderLineWidth2)
	ON_COMMAND(ID_RENDER_LINE_WIDTH_3, OnRenderLineWidth3)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINE_WIDTH_3, OnUpdateRenderLineWidth3)
	ON_COMMAND(ID_RENDER_LINE_WIDTH_4, OnRenderLineWidth4)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINE_WIDTH_4, OnUpdateRenderLineWidth4)
	ON_COMMAND(ID_RENDER_LINE_WIDTH_5, OnRenderLineWidth5)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINE_WIDTH_5, OnUpdateRenderLineWidth5)
	ON_COMMAND(ID_RENDER_RIBBON, OnRenderRibbon)
	ON_UPDATE_COMMAND_UI(ID_RENDER_RIBBON, OnUpdateRenderRibbon)
	ON_COMMAND(ID_RENDER_RIBBON_FLAT, OnRenderRibbonFlat)
	ON_UPDATE_COMMAND_UI(ID_RENDER_RIBBON_FLAT, OnUpdateRenderRibbonFlat)
	ON_COMMAND(ID_RENDER_TUBE, OnRenderTube)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TUBE, OnUpdateRenderTube)
	ON_COMMAND(ID_RENDER_TUBE_OCC, OnRenderTubeOcc)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TUBE_OCC, OnUpdateRenderTubeOcc)
	ON_COMMAND(ID_RENDER_TUBE_T, OnRenderTubeT)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TUBE_T, OnUpdateRenderTubeT)
	ON_COMMAND(ID_RENDER_RIBBON_SHOW_WIRE, OnRenderRibbonShowWire)
	ON_UPDATE_COMMAND_UI(ID_RENDER_RIBBON_SHOW_WIRE, OnUpdateRenderRibbonShowWire)
	ON_COMMAND(ID_OPTIONS_STEREO, OnOptionsStereo)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_STEREO, OnUpdateOptionsStereo)
	ON_COMMAND(FILE_NET_COORD_OPEN, OnNetCoordOpen)
	ON_COMMAND(ID_FILE_TRAJ_EXPORT, OnFileTrajExport)
	ON_UPDATE_COMMAND_UI(ID_FILE_COORD_SAVE, OnUpdateFileCoordSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_TRAJ_EXPORT, OnUpdateFileTrajExport)
	ON_COMMAND(ID_RENDER_HBONDS, OnRenderHbonds)
	ON_UPDATE_COMMAND_UI(ID_RENDER_HBONDS, OnUpdateRenderHbonds)
	ON_COMMAND(ID_OPTIONS_HBOND_COLOR, OnOptionsHbondColor)
	ON_WM_MOUSEWHEEL()
	ON_COMMAND(ID_OPTIONS_MOUSE_DEC, OnOptionsMouseDec)
	ON_COMMAND(ID_OPTIONS_MOUSE_INC, OnOptionsMouseInc)
	ON_COMMAND(ID_OPTIONS_MOUSE_DEF, OnOptionsMouseDef)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_RENDER_CLIP_COLOR, OnRenderClipColor)
	ON_COMMAND(ID_RENDER_CLIP_SHOW, OnRenderClipShow)
	ON_COMMAND(ID_RENDER_CLIP_VIEW, OnRenderClipView)
	ON_UPDATE_COMMAND_UI(ID_RENDER_CLIP_SHOW, OnUpdateRenderClipShow)
	ON_UPDATE_COMMAND_UI(ID_RENDER_CLIP_VIEW, OnUpdateRenderClipView)
	ON_COMMAND(ID_RENDER_CLIP_CENTER, OnRenderClipCenter)
	ON_COMMAND(ID_OPTIONS_STRUC_ASSIGN, OnOptionsStrucAssign)
	ON_COMMAND(ID_OPTIONS_SEQUENCE_COLOR, OnOptionsSequenceColor)
	ON_COMMAND(FILE_EXPORT_IMAGE, OnExportImage)
	ON_COMMAND(ID_RENDER_SURFACE, OnRenderSurface)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SURFACE, OnUpdateRenderSurface)
	ON_COMMAND(ID_RENDER_CLIP_SURFACE, OnRenderClipSurface)
	ON_UPDATE_COMMAND_UI(ID_RENDER_CLIP_SURFACE, OnUpdateRenderClipSurface)
	ON_COMMAND(ID_RENDER_SURFACE_OPT, OnRenderSurfaceOpt)
	ON_MESSAGE(WM_SURF_OPT, OnSurfaceOptMessage)
	ON_MESSAGE(WM_SURF_REBUILD, OnSurfaceBuildMessage)
	ON_COMMAND(ID_OPTIONS_SURFACE_COLOR, OnOptionsSurfaceColor)
	ON_COMMAND(ID_RENDER_MESH, OnRenderMesh)
	ON_UPDATE_COMMAND_UI(ID_RENDER_MESH, OnUpdateRenderMesh)
	ON_COMMAND(ID_RENDER_NONE, OnRenderNone)
	ON_UPDATE_COMMAND_UI(ID_RENDER_NONE, OnUpdateRenderNone)
	ON_COMMAND(ID_RENDER_SURFACE_POINT, OnRenderSurfacePoint)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SURFACE_POINT, OnUpdateRenderSurfacePoint)
	ON_COMMAND(ID_MEASURE_SURFACE_RESIDUE, OnMeasureSurfaceResidue)
	ON_UPDATE_COMMAND_UI(ID_MEASURE_SURFACE_RESIDUE, OnUpdateMeasureSurfaceResidue)
	ON_COMMAND(ID_MEASURE_SURFACE_ATOM, OnMeasureSurfaceAtom)
	ON_UPDATE_COMMAND_UI(ID_MEASURE_SURFACE_ATOM, OnUpdateMeasureSurfaceAtom)
	ON_COMMAND(ID_OPTIONS_DEFAULT, OnOptionsDefault)
	ON_COMMAND(ID_OPTIONS_PURGE_REG, OnOptionsPurgeReg)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_PURGE_REG, OnUpdateOptionsPurgeReg)
	ON_COMMAND(ID_RENDER_MONITOR_LABELS, OnRenderMonitorLabels)
	ON_UPDATE_COMMAND_UI(ID_RENDER_MONITOR_LABELS, OnUpdateRenderMonitorLabels)
	ON_COMMAND(ID_OPTIONS_MOL_SEL, OnOptionsMolSel)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SURFACE_OPT, OnUpdateRenderSurfaceOpt)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_SURFACE_COLOR, OnUpdateOptionsSurfaceColor)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_SEQUENCE_COLOR, OnUpdateOptionsSequenceColor)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_ATOM_COLOR, OnUpdateOptionsAtomColor)
	ON_MESSAGE(WM_MOL_SELECT_DIALOG, OnMolSelectDialogMsg)
	ON_COMMAND(ID_OPTIONS_SCALE_FIT, OnOptionsScaleFit)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_SCALE_FIT, OnUpdateOptionsScaleFit)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_STRUC_ASSIGN, OnUpdateOptionsStrucAssign)
	ON_COMMAND(ID_MOLECULE_ALIGN, OnMoleculeAlign)
	ON_UPDATE_COMMAND_UI(ID_MOLECULE_ALIGN, OnUpdateMoleculeAlign)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_MEASURE, OnUpdateOptionsMeasure)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CQmolView construction/destruction

CQmolView::CQmolView() :
	 m_hRC(0), m_pDC(0)
{	
	// Allocate memory the adjust torsion dialog
	m_pATDlg = new CAdjustTorsionDialog(this);
	
	// Allocate memory the measure dialog
	m_pMDlg = new CMeasureDialog(this);
	
	// Allocate memory the animate dialog
	m_pAnimateDlg = new CAnimateDialog(this);
	
	// Allocate memory the color option dialog
	m_qmColorDlg = new CQMColorDialog(this);

	m_flexClrDlg = new CFlexColorDialog(this);

	// Allocate memory a new atom radius dialog box
	m_pARDlg = new CAtomRadiusDialog(this);

	// Allocate memory a lighting options dialog
	m_lightDlg = new CLightDialog(this);	

	// Allocate memory for a custom resolution dialog
	m_customResDlg = new CCustomResDialog(this);

	// Allocate memory for the sequence display dialog
	m_seqDlg = new CSequenceDialog(this);

	// Allocate memory for the surface options dialog
	m_surfDlg = new CSurfDialog("Surface Control", this);

	// Allocate memory for the molecule selection dialog
	m_molSelDlg = new CMoleculeSelectDialog(this);

	// Allocate memory for the atom selection dialog
	m_atomSelectDlg = new CAtomSelectionDialog(this);

	bpp = g_bpp;
	zbuf = g_z_buf;

	// No font call list yet!
	font_list_id = -1;

	// The default font
	font.lfHeight = -18;
	font.lfWidth = 0;
	font.lfEscapement = 0;
	font.lfOrientation = font.lfEscapement;
	font.lfWeight = FW_BOLD /*FW_NORMAL*/;
	font.lfItalic = FALSE;
	font.lfUnderline = FALSE;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = ANSI_CHARSET;
	font.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FF_DONTCARE|DEFAULT_PITCH;
	lstrcpy ( font.lfFaceName, "Arial" );

	printScaleMul = printScaleDiv = 82; // Empirical font scaling value

	// By default, don't animate!
	animate = FALSE;
	animateDirection = QM_FORWARD;
	animateTraj = FALSE;

	// Initialize to a big number
	m_ElapsedTime = ~(DWORD)0;
	m_previousElapsedTime = 0;

#ifdef COMPUTE_FPS
	fps_fames = 50;
	fps_cur_frame = 0;
	fps_time = 0;
	fps = 0.0f;
#endif // COMPUTE_FPS

	l_button_down = FALSE;
	shift_key_down = FALSE;

	// By default, DON'T remove any registry entries
	purgeRegistryEntries = FALSE;

	// Load application defaults from the registry
	LoadDefaults(TRUE);
}

void CQmolView::LoadDefaults(BOOL m_useRegistry = FALSE)
{	
	// By default, make 16 bit color images for printing and 
	// rendering to clipboard
	print_color_depth = 16;

	mat_specular[0] = 0.3f;
	mat_specular[1] = 0.3f;
	mat_specular[2] = 0.3f;
	mat_specular[3] = 0.3f;

	mat_shininess[0] = 15.0f;

	// Background color
	color_scheme = QM_COLOR_DEFAULT;

	SetBackgroundColor();
	
	selection_color[0] = 0.0f;
	selection_color[1] = 0.9f;
	selection_color[2] = 0.0f;
	selection_color[3] = 0.6f;

	hbond_color[0] = 1.0f;
	hbond_color[1] = 0.0f;
	hbond_color[2] = 0.0f;

	depthCue = TRUE;
	lineAlias = TRUE;
	smooth3D = TRUE;

	// Fog for depth cueing
	fogDensity = 0.25f;
	
	// By default, disable stero display
	displayStereo = QM_STEREO_NONE;

	// Are we currently rendering the right or left eye of a split
	// screen stereo view?
	renderSplitStereoRightEye = FALSE;
	renderSplitStereoLeftEye = FALSE;

	// The stereo offset in degrees (for splitscreen)
	stereoOffset = 3.5f;

	// Clipping defaults
	clipView = FALSE;
	clipSurface = FALSE;
	displayClipPlane = FALSE;

	clipPlaneClr[0] = 0.5f;
	clipPlaneClr[1] = 0.5f;
	clipPlaneClr[2] = 0.5f;
	clipPlaneClr[3] = 0.5f;

	clipper.SetColor(clipPlaneClr);

	// No limit on bitmap size when printing
	mem_size = -1;

	// Use a medium resolution for geometric objects.
	resolution = MED_RES;

	// Initial line width
	line_width = 2.0;

	// Scale the molecular box in response to shifting
	// molecular center-of-masses.
	scaleToFit = TRUE;

	// The Light model for wire frame and solid objects. This should only be
	// initialized once (hence it is NOT in the InitializeOpenGL() function).

	// Bond light model
	ls_wire_frame.SetAmbient(1.0f, 1.0f, 1.0f);
	ls_wire_frame.SetSpecular(0.0f, 0.0f, 0.0f);
	ls_wire_frame.SetDiffuse(0.0f, 0.0f, 0.0f);
	ls_wire_frame.SetPosition(1.0f, 0.0f, 0.0f);
	
	// Solid object light model
	for(int i = 0;i < QM_NUM_LIGHTS - 1;i++){
		lb_solid.SetSpecular(i, 1.0f, 1.0f, 1.0f);
		lb_solid.SetDiffuse(i, 0.5f, 0.5f, 0.5f);
	}

	// Set the ambient light source
	lb_solid.SetAmbient(QM_AMBIENT, 0.2f, 0.2f, 0.2f);

	// Turn on the following solid lights
	lb_solid.TurnOn(0); // Front
	//lb_solid.TurnOn(2); // Top
	//lb_solid.TurnOn(5);	// Bottom
	//lb_solid.TurnOff(6); // Turn off ambient lighting

	perspective = QM_PERSPECTIVE;

	maxDim = 1.0f;

	// Set the center of the view
	center[X] = center[Y] = center[Z] = 0.0f;

	// Don't start with coordinate axes
	axes = FALSE;

	// No free-spinning molecules to start with
	momentum = FALSE;
	spinning = FALSE;

	showHBonds = FALSE;

	if(m_useRegistry){
		LoadRegistry();
	}
}

CQmolView::~CQmolView()
{
	if(purgeRegistryEntries){
		// Remove all registry entries
		CleanRegistry();
	}
	else{
		// Save the current configuration to the registry
		SaveRegistry();
	}

	// Remove the adjust torsion dialog
	if(m_pATDlg)
		delete m_pATDlg;
	
	// Remove the measure dialog
	if(m_pMDlg)
		delete m_pMDlg;

	// Remove the animate dialog
	if(m_pAnimateDlg)
		delete m_pAnimateDlg;

	// Remove the color options dialog
	if(m_qmColorDlg)
		delete m_qmColorDlg;

	// Remove the modeless color selection dialog
	if(m_flexClrDlg)
		delete m_flexClrDlg;

	if(m_pARDlg)
		delete m_pARDlg;
	
	if(m_lightDlg)
		delete m_lightDlg;

	if(m_customResDlg)
		delete m_customResDlg;

	if(m_seqDlg)
		delete m_seqDlg;

	if(m_surfDlg)
		delete m_surfDlg;

	if(m_molSelDlg)
		delete m_molSelDlg;
	
	if(m_atomSelectDlg)
		delete m_atomSelectDlg;

}

BOOL CQmolView::PreCreateWindow(CREATESTRUCT& cs)
{
	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

void CQmolView::Update()
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Update the molecule selection dialog
	if(m_molSelDlg->GetSafeHwnd() != 0)
		m_molSelDlg->PostMessage(WM_MOL_SELECT_UPDATE);

	pDoc->UpdateTraj();

	molecule *ptrMol = pDoc->activeMol;

	if(!ptrMol){
		// No molecules are loaded, so stop animating.
		animate = FALSE;
		return;
	}

	// Get the size of the largest dimension
	maxDim = pDoc->GetMaxDim(scaleToFit);


	// If this is the first molecule loaded, adjust the center of the view to 
	// match the center of mass coordinates for the active molecule
	if(pDoc->molList.Length() == 1){
		center[X] = ptrMol->centerX;
		center[Y] = ptrMol->centerY;
		center[Z] = ptrMol->centerZ;
	}

	// Start animation if needed
	animateDirection = QM_FORWARD;

	if(ptrMol->steps){
		animateTraj = TRUE;
		animate = TRUE;
	}
	else{
		// No animation is needed
		animateTraj = FALSE;

		if(!spinning)
			animate = FALSE;
	}
	
	tor_adjust.Init();

	// Set atom colors according to the current color scheme
	if(m_qmColorDlg != NULL){
		// Only update the color scheme if the user has modified the 
		// default scheme
		m_qmColorDlg->ApplyCurrColor();
	}
	
	if(m_seqDlg->GetSafeHwnd() != 0){
		// Reinitialize the structure dialog if it is 
		// currently visable.
		m_seqDlg->PostMessage(WM_SEQUENCE_RELOAD);
	}

	if(m_atomSelectDlg->GetSafeHwnd() != 0){
		// Reinitialize the atom selection dialog if it is 
		// currently visable.
		m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_UPDATE);
	}

	// Set the current ribbon dimensions (according to the current displayType).
	ptrMol->SetRibbonSize(ptrMol->displayType);
	
	// Update the hydrogen bonds
	if(showHBonds)
		pDoc->UpdateHydrogenBonding();

	// Redraw the screen 
	postRedisplay();
}

void CQmolView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	// This function is called when a document has been loaded (or reloaded)
	Update();
}
/////////////////////////////////////////////////////////////////////////////
// CQmolView drawing

void CQmolView::OnDraw(CDC* pDC)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if (animate){
		// The accuracy depends upon the OS that you're running
		// Win95 has accuracy to 1 ms, while WinNT will average about
		// 5 ms or more (fine for our purposes). If you're running NT you can
		// set the accuracy to a finer value. See the documentation.
		// (However, this runs fine under NT as it is...)
		m_ElapsedTime = timeGetTime(); // get current time

		postRedisplay();

		// This statement will limit the refresh rate of the
		// window. Note that we NEED to limit the refresh rate to
		// prevent slower win95/98/ME machines from clogging the
		// message loop with WM_PAINT messages (which will interfere
		// with all other messages - like mouse movement and GUI
		// operations).
		if((m_ElapsedTime - m_previousElapsedTime) < 15)
			return;
		
		// If the molecule is spinning, update the rotation matricies
		if(spinning){
			ListIterator<molecule*> next(pDoc->molList);
			molecule **tmpMol;

			// The refernce frame
			referenceFrame.update();

			// The individual molecules
			while(tmpMol = next())
				(*tmpMol)->Update();
		}
	}

	RenderScene(pDoc->molList, QM_RENDER_SCREEN);

	// If the measure dialog box is currently being displayed,
	// update it.
	if(m_pMDlg->GetSafeHwnd())
		m_pMDlg->UpdateMeasurements();

	// Update the animate dialog:
	if(m_pAnimateDlg->GetSafeHwnd())
		m_pAnimateDlg->SetCurrentStep(pDoc->CurrentTrajStep());

	// Increment the current trajectory step
	if(animateTraj){
		pDoc->IncrementTrajStep(animateDirection);

		if(showHBonds)
			pDoc->UpdateHydrogenBonding();

		if(m_atomSelectDlg->GetSafeHwnd() != 0){
			// Update the rmsd calculation
			m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_RMSD_UPDATE);
		}
	}

	if(FALSE == SwapBuffers( m_pDC->GetSafeHdc())){
		throw QmolError("SwapBuffers failed", __FILE__, __LINE__);
	}

	// the very last thing we do is to save
	// the elapsed time, this is used with the
	// next elapsed time to calculate the
	// elapsed time since a render and the frame rate
	m_previousElapsedTime = m_ElapsedTime;
}

/////////////////////////////////////////////////////////////////////////////
// CQmolView diagnostics

#ifdef COMPUTE_FPS
void CQmolView::renderFPS()
{
	// Compute the number of frames per second
	if(fps_cur_frame >= fps_fames){
		fps_cur_frame = 0;

		unsigned long tmp_time = timeGetTime();

		fps = fps_fames*1.0e3f/((tmp_time - fps_time));

		fps_time = tmp_time;
	}
	else{
		fps_cur_frame ++;
	}

	char fps_buffer[64];
	sprintf(fps_buffer, "fps = %.2g ",fps);

	if(clipView)
		clipper.Disable();

	glPushMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	// Set up a perspective for 2D viewing
	gluOrtho2D(0.0, 1.0, 0.0, 1.0);

	// Color the atom labels black (so that they may be clearly visable)
	glColor3f(1.0f - background[0], 
			  1.0f - background[1], 
			  1.0f - background[2]);

	glRasterPos2f(0.01f,  0.01f);
  	glCallLists(strlen(fps_buffer), GL_UNSIGNED_BYTE, fps_buffer);

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	if(clipView)
		clipper.Enable();
}

#endif // COMPUTE_FPS

#ifdef _DEBUG
void CQmolView::AssertValid() const
{
	CView::AssertValid();
}

void CQmolView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CQmolDoc* CQmolView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CQmolDoc)));
	return (CQmolDoc*)m_pDocument;
}

void CQmolView::GetOpenGLError(char *m_file, int m_line)
{
	CString error_msg;
	GLenum error_code;

	while((error_code = glGetError()) != GL_NO_ERROR){
		switch(error_code){
			case GL_INVALID_ENUM:
				error_msg.Format("INVALID_ENUM in %s @ %d", m_file, m_line);
				AfxMessageBox(error_msg);
				break;
			case GL_INVALID_VALUE:
				error_msg.Format("INVALID_VALUE in %s @ %d", m_file, m_line);
				AfxMessageBox(error_msg);
				break;
			case GL_INVALID_OPERATION:
				error_msg.Format("INVALID_OPERATION in %s @ %d", m_file, m_line);
				AfxMessageBox(error_msg);
				break;
			case GL_STACK_OVERFLOW:
				error_msg.Format("STACK_OVERFLOW in %s @ %d", m_file, m_line);
				AfxMessageBox(error_msg);
				break;
			case GL_STACK_UNDERFLOW:
				error_msg.Format("STACK_UNDERFLOW in %s @ %d", m_file, m_line);
				AfxMessageBox(error_msg);
				break;
			case GL_OUT_OF_MEMORY:
				error_msg.Format("OUT_OF_MEMORY in %s @ %d", m_file, m_line);
				AfxMessageBox(error_msg);
				break;
		};
	}
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CQmolView message handlers
/////////////////////////////////////////////////////////////////////////////

int CQmolView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_pDC = new CClientDC(this);

    if(NULL == m_pDC){
		// failure to get DC
		throw QmolError("Unable to get a DC", __FILE__, __LINE__);
	}

	if(!SetupPixelFormat(bpp, zbuf)){
        return -1;
	}

    if(0 == (m_hRC = ::wglCreateContext(m_pDC->GetSafeHdc()))){
		throw QmolError("wglCreateContext failed", __FILE__, __LINE__);
	}

    if(FALSE == ::wglMakeCurrent(m_pDC->GetSafeHdc(), m_hRC)){
		throw QmolError("wglMakeCurrent failed", __FILE__, __LINE__);
	}

	InitializeOpenGL();

	// Turn on the lights ...
	ls_wire_frame.Enable();

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// GL helper functions
BOOL CQmolView::InitializeOpenGL()
{
	SetGeomObjRes(resolution);

	glEnable(GL_DEPTH_TEST);
  
  	glClearColor(background[0], background[1], background[2], background[3]);
  	glClearDepth(1.0);
  	glEnable(GL_NORMALIZE);

	// Anti-aliasted lines
	glEnable(GL_LINE_SMOOTH);

	// Cull back faces of polygons
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);

	// Need to turn on GL_DITHER for 3D objects and turn it off for
	// lines and points.
	glDisable(GL_DITHER);
	
	// Need to set glShadeModel to GL_SMOOTH for 3D objects 
	// and set it to GL_FLAT for lines and points.
	glShadeModel(GL_FLAT);

	// Set the current lighting parameters
	ls_wire_frame.Apply();
	lb_solid.Apply();

	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
  	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

  	glEnable(GL_LIGHTING);

  	// Use fog to give the molecule a feeling of depth.
	glFogi(GL_FOG_MODE, GL_LINEAR);
  	glFogf(GL_FOG_DENSITY, fogDensity);
	glFogf(GL_FOG_START, 3.0f);
	glFogf(GL_FOG_END, 6.0f);
  	glFogfv(GL_FOG_COLOR, background);

	if(depthCue){
  		glEnable(GL_FOG);
	}
	else{
		glDisable(GL_FOG);
	}

	// Delete any previous font list
	if(font_list_id >= 0)
		glDeleteLists( font_list_id, 256 );

	// Generate the display list for holding the selected font
	// and set the list base to the appropriate offset.
	glListBase(font_list_id = GenerateBitmapListForFont(font));

	// Allow translucent spheres to indicate selected atoms
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_BLEND);

	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

  	glMatrixMode(GL_MODELVIEW);
  	glLoadIdentity();
  	glTranslatef(0, 0, QM_DEFAULT_DEPTH);

	return TRUE;
}

void CQmolView::SetMeasureStipple()
{
	// Set the stipple pattern for measurement lines
	glLineStipple(1, 0xf0f0);
}

// This function is not currently used.
// HBond stippling is performed "by hand".
void CQmolView::SetHBondStipple()
{
	// Set the stipple pattern for H bond lines

	// A solid line --> 0xffff
	glLineStipple(1, 0xf0f0);
}


BOOL CQmolView::SetupPixelFormat(int bpp, int zbuf)
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
        PFD_SUPPORT_OPENGL |			// support OpenGL
        PFD_DOUBLEBUFFER,				// double buffered
        PFD_TYPE_RGBA,                  // RGBA type
		bpp,							// bpp
        0, 0, 0, 0, 0, 0,               // color bits ignored
		0,                              // Don't use the alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
		zbuf,							// z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };

    int pixelformat;

    if(0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd))){
		throw QmolError("ChoosePixelFormat failed", __FILE__, __LINE__);
	}

    if(FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd)){
		throw QmolError("SelectPixelFormat failed", __FILE__, __LINE__);
	}

	// What is the final Pixel format?
	if(DescribePixelFormat(m_pDC->GetSafeHdc(), pixelformat, 
		sizeof(PIXELFORMATDESCRIPTOR), &pfd) != 0){

		g_bpp = pfd.cColorBits;
		g_z_buf = pfd.cDepthBits;
	}
 
	
    return TRUE;
}

void CQmolView::OnDestroy() 
{
	CView::OnDestroy();

    if(FALSE == ::wglDeleteContext( m_hRC ))
	{
		throw QmolError("wglDeleteContext failed", __FILE__, __LINE__);
 	}

    if(m_pDC){
        delete m_pDC;
		m_pDC = NULL;
	}
}


BOOL CQmolView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE; // tell Windows not to erase the background
}

void CQmolView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	if ( 0 >= cx || 0 >= cy ){
		return;
	}

	// Save the window dimensions
	viewWidth = cx;
	viewHeight = cy;

	SetupGLView(viewWidth, viewHeight);

	// Keep the user informed of the window size
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame){
		CString str;
		CStatusBar* pStatus = &pFrame->m_wndStatusBar;

		str.Format("Window Size %d x %d", cx, cy);
		pStatus->SetPaneText(INFO_PANE, str);
	}
}

 /////////////////////////////////////////////////////////////////////////////
// COpenGLView helper functions

BOOL CQmolView::SetupGLView(int m_width, int m_height)
{
	GLfloat aspect;

	if(displayStereo == QM_STEREO_SPLIT_SCREEN){
		m_width /= 2;
	}

	// Select the projection matrix and clear it
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	// Compute the aspect ratio
	// this will keep all dimension scales equal
	aspect = ((GLfloat)m_width)/m_height;

	// Select the viewing volume
	gluPerspective(perspective, aspect, 
		QM_NEAR_CLIP_Z, QM_FAR_CLIP_Z);
	
	// Switch back to the modelview matrix
    glMatrixMode(GL_MODELVIEW);

	// Do we need to turn on clipping? We need to check here 
	// (as opposed to in InitializeOpenGL()) to make sure printing,
	// copying and movie create are corrrectly initialized.
	if(clipView){
		clipper.Enable();
	}
	else{
		clipper.Disable();
	}

	return TRUE;
}

void CQmolView::renderAtoms(molecule *mol, GLfloat m[][4])
{
  	glScalef(SCALE_FACTOR / maxDim,
    		 SCALE_FACTOR / maxDim,
    		 SCALE_FACTOR / maxDim);

	// Apply the translation vector in the rotated coordinates
	glTranslatef(mol->R[X],
    			 mol->R[Y],
    			 mol->R[Z]);

	if(mol->atom_labels)
		renderLabels(mol);
	
	switch(mol->displayType){
	case QM_RENDER_NONE:
			// Don't redner the molecule
			break;
		case QM_RENDER_WIREFRAME:
			// Render a wire frame model
			renderWireFrame(mol);
			break;
		case QM_RENDER_TRACE_WIREFRAME:
			// Render a wire frame model of the trace
			renderTraceWireFrame(mol);
			break;
		case QM_RENDER_TRACE_SPLINE:
			// Render a spline model of the trace
			renderTraceSpline(mol);
			break;
		case QM_RENDER_TRACE_STICK:
			// Render a stick model of the trace
			renderTraceStick(mol, m);
			break;
		case QM_RENDER_POINT:
			// Render a point model
			renderPoints(mol, m);
			break;
		case QM_RENDER_CPK:
			// Render space-filling atoms
			renderVanDerWaals(mol, m);
			break;
		case QM_RENDER_STICK:
		case QM_RENDER_BALL_AND_STICK:
			renderBallAndStick(mol, m);
			break;
		case QM_RENDER_RIBBON_FLAT:
			if(mol->showWireFrame)
				// Render a wire frame model with the
				// selected ribbon superimposed.
				renderWireFrame(mol);

			renderFlatRibbon(mol);
			break;
		case QM_RENDER_RIBBON:
		case QM_RENDER_TUBE:
		case QM_RENDER_TUBE_T:
		case QM_RENDER_TUBE_OCC:
			if(mol->showWireFrame)
				// Render a wire frame model with the
				// selected ribbon superimposed.
				renderWireFrame(mol);

			renderRibbon(mol);
			break;
	};

	// Render the selected torsion angle (if any)
	if(mol == tor_adjust.GetMol())
		renderTorsion();

	// Render any atoms monitors
	if(mol->monitors)
		renderMonitors(mol, m);

	// Render the axes indicators
	if(axes)
		renderAxes(0.3f*mol->maxdim, 0.3f*mol->maxdim, 0.3f*mol->maxdim, m);

	// *** Render translucent objects after this point ***
	// Render the molecular surface (if any).
	if(mol->surf.ValidSurface())
		RenderSurface(mol);
}

void CQmolView::renderMeasure()
{
	// Only display measurements if the measure dialog is currently 
	// open (this will reduce GUI clutter).
	if(!m_pMDlg->GetSafeHwnd())
		return;

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if(pDoc->molMetric.NumPicked() == 0)
		return;

	// Apply any molecular rotations or
	// translations
	pDoc->molMetric.Update();

	glPushMatrix();

	glScalef(SCALE_FACTOR / maxDim,
			 SCALE_FACTOR / maxDim,
			 SCALE_FACTOR / maxDim);

	// Set the line stippling to reflect measurement
	SetMeasureStipple();

	ls_wire_frame.Disable();
	lb_solid.Enable();
	
	glEnable(GL_DITHER);
	
	if(smooth3D)
		glShadeModel(GL_SMOOTH);
	
  	for(int i = 0;i < pDoc->molMetric.NumPicked();i++){

		glPushMatrix();
		glTranslatef(pDoc->molMetric.x_coord(i), 
				pDoc->molMetric.y_coord(i), 
				pDoc->molMetric.z_coord(i));
			
		glScalef(MEASURE_RADIUS, MEASURE_RADIUS, MEASURE_RADIUS);
			
		// **** "Undo" any stero rotation ****
		// Are we currently rendering the right eye of a split
		// screen stereo view? Note that in the current scheme of
		// things, the positive y axis is "up".
		if(renderSplitStereoRightEye)
			glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);
			
		if(renderSplitStereoLeftEye)
			glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);
		
		unsigned char *color = pDoc->molMetric.Color(i);

		glColor4ub(color[0], color[1], color[2], SELECT_ALPHA);
			
		glCallList(SELECTION_SPHERE);
		glPopMatrix();
	}
		
	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);
		
	lb_solid.Disable();
	ls_wire_frame.Enable();
		
	glEnable(GL_LINE_STIPPLE);
	glLineWidth(MEASURE_LINE_WIDTH);
		
	// Disable anitaliasing when drawing stippled lines.
	glDisable(GL_LINE_SMOOTH);
		
	if(pDoc->molMetric.NumPicked() > 1){
		glBegin(GL_LINES);
		glColor3fv(selection_color);
			
		for(i = 1;i < pDoc->molMetric.NumPicked();i++){
			float ra[3];
			float rb[3];

			ra[X] = pDoc->molMetric.x_coord(i);
			ra[Y] = pDoc->molMetric.y_coord(i);
			ra[Z] = pDoc->molMetric.z_coord(i);

			rb[X] = pDoc->molMetric.x_coord(i - 1);
			rb[Y] = pDoc->molMetric.y_coord(i - 1);
			rb[Z] = pDoc->molMetric.z_coord(i - 1);

			glVertex3fv(rb);
			glVertex3fv(ra);
		}
			
		glEnd();
	}
		
	if(lineAlias)
		glEnable(GL_LINE_SMOOTH);
		
	glDisable(GL_LINE_STIPPLE);

	glPopMatrix();
}

void CQmolView::renderWireFrame(molecule *mol)
{
	int i;
	float ra[3], rb[3], rMid[3];	
	int num_bonds =  mol->bonds.GetLength();

	ls_wire_frame.Enable();

	// Render just the bonds
	glLineWidth(line_width);
	glBegin(GL_LINES);		

		if(mol->current_step == 0){
			for(i = 0;i < num_bonds;i++){
				// Save a reference to the current bond for fast access
				bond &bRef = mol->bonds[i];

				if(!(!mol->solvent && bRef.solvent)){
					// Traverse the bond list
					mol->atoms[bRef.a].GetCoord(ra);
					mol->atoms[bRef.b].GetCoord(rb);
			
					// Determine how many verticies must be rendered
					// Is the bond between the same color atoms or
					// different color atoms?
					if(bRef.colorBond){
				
						rMid[0] = 0.5f*(rb[0] + ra[0]);
						rMid[1] = 0.5f*(rb[1] + ra[1]);
						rMid[2] = 0.5f*(rb[2] + ra[2]);
			
						// To get colored bonds, each bond has to be
						// contstructed out of two different colored lines.
						glColor4ubv(mol->atoms[bRef.a].rgb_color);
						glVertex3fv(ra);
						glVertex3fv(rMid);
			
						glColor4ubv(mol->atoms[bRef.b].rgb_color);
						glVertex3fv(rMid);
						glVertex3fv(rb);
					}
					else{
						glColor4ubv(mol->atoms[bRef.a].rgb_color);
						glVertex3fv(ra);
						glVertex3fv(rb);
					}
				}
			}
			
			// Update the measurements of the measure dialog
			#ifdef REVAMP_MEASURE_CODE
			molState->Update(mol);
			#endif // REVAMP_MEASURE_CODE
		}else{
			//int index = (mol->current_step - 1)*3*mol->atoms.Length();
			int index = mol->current_step - 1;

			for(i = 0;i < num_bonds;i++){
				// Save a reference to the current bond for fast access
				bond &bRef = mol->bonds[i];

				if(!(!mol->solvent && bRef.solvent)){
					// Traverse the bond list
					memcpy(ra, &(mol->traj[index][3*bRef.a]), 3*sizeof(float));
					memcpy(rb, &(mol->traj[index][3*bRef.b]), 3*sizeof(float));
				
					// Determine how many verticies must be rendered
					// Is the bond between the same color atoms or
					// different color atoms?
					if(mol->bonds[i].colorBond){
				
						rMid[0] = 0.5f*(rb[0] + ra[0]);
						rMid[1] = 0.5f*(rb[1] + ra[1]);
						rMid[2] = 0.5f*(rb[2] + ra[2]);
			
						// To get colored bonds, each bond has to be
						// contstructed out of two different colored lines.
						glColor4ubv(mol->atoms[bRef.a].rgb_color);
						glVertex3fv(ra);
						glVertex3fv(rMid);
			
						glColor4ubv(mol->atoms[bRef.b].rgb_color);
						glVertex3fv(rMid);
						glVertex3fv(rb);
					}
					else{
						glColor4ubv(mol->atoms[bRef.a].rgb_color);
						glVertex3fv(ra);
						glVertex3fv(rb);
					}
				}	
			}

			#ifdef REVAMP_MEASURE_CODE
			// Update the measurements of the measure dialog
			molState->Update(index, mol);
			#endif // REVAMP_MEASURE_CODE
		}
	
	glEnd();
}

// Render the alpha carbon trace of a protein
void CQmolView::renderTraceWireFrame(molecule *mol)
{
	int a, i, j;
	
	int num_chain = mol->secondaryStruc.Length();

	for(j = 0;j < num_chain;j++){
		array<amino> &sequence_ref = mol->secondaryStruc[j].sequence;

		int num_res = sequence_ref.Length();

		// Only need to render a trace if we have two or more amino
		// acids in the protein!
		if(num_res >= 2){

			ls_wire_frame.Enable();

			// Render a line from C_{\alpha i} to C_{\alpha i + 1}
			glLineWidth(line_width);
			glBegin(GL_LINE_STRIP);		

				if(mol->current_step == 0){
					for(i = 0;i < num_res;i++){
						a = sequence_ref[i].c_alpha_index;
					
						glColor4ubv(mol->atoms[a].rgb_color);
						glVertex3fv(mol->atoms[a].coord);
					}

					#ifdef REVAMP_MEASURE_CODE
					// Update the measurements of the measure dialog
					molState->Update(mol);
					#endif // REVAMP_MEASURE_CODE
				}else{
					//int index = (mol->current_step - 1)*3*mol->atoms.Length();
					int index = mol->current_step - 1;

					for(i = 0;i < num_res;i++){
						a = sequence_ref[i].c_alpha_index;

						glColor4ubv(mol->atoms[a].rgb_color);
						glVertex3fv(&(mol->traj[index][3*a]));
					}
					
					#ifdef REVAMP_MEASURE_CODE
					// Update the measurements of the measure dialog
					molState->Update(index, mol);
					#endif // REVAMP_MEASURE_CODE
				}
			
			glEnd();
		}
	}
}

// Render the alpha carbon trace of a protein
void CQmolView::renderTraceSpline(molecule *mol)
{
	int cur_atom, j;
	GLfloat r[3];
	
	int num_chain = mol->secondaryStruc.Length();

	for(j = 0;j < num_chain;j++){
		array<amino> &sequence_ref = mol->secondaryStruc[j].sequence;

		int num_res = sequence_ref.Length();

		// Only need to render a trace if we have two or more amino
		// acids in the protein!
		if(num_res >= 2){
			ls_wire_frame.Enable();

			moleculeSpline spline(mol, j, splineRes, splineTension);

			// Render a line from C_{\alpha i} to C_{\alpha i + 1}
			glLineWidth(line_width);
			glBegin(GL_LINE_STRIP);		

				if(mol->current_step == 0){
					while(spline.NextTrace(r, cur_atom)){
						glColor4ubv(mol->atoms[cur_atom].rgb_color);

						glVertex3fv(r);
					}

					#ifdef REVAMP_MEASURE_CODE
					// Update the measurements of the measure dialog
					molState->Update(mol);
					#endif // REVAMP_MEASURE_CODE
				}else{
					//int index = (mol->current_step - 1)*3*mol->atoms.Length();
					int index = mol->current_step - 1;

					while(spline.NextTrace(r, cur_atom, index)){
						glColor4ubv(mol->atoms[cur_atom].rgb_color);

						glVertex3fv(r);
					}

					#ifdef REVAMP_MEASURE_CODE
					// Update the measurements of the measure dialog
					molState->Update(index, mol);
					#endif // REVAMP_MEASURE_CODE
				}
			
			glEnd();
		}
	}
}

void CQmolView::RenderSurface(molecule *mol)
{
	if(smooth3D)
		glShadeModel(GL_SMOOTH);

	// If only the surface needs to be clipped, turn on clipping
	if(clipSurface && !clipView){
		clipper.Enable();
	}

	// Adjust the face culling depending on the (a) clipping
	// and (b) transparency.
	if(clipView || clipSurface){
		glDisable(GL_CULL_FACE);

		// Illuminate both sides of the surface
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	}

	glEnable(GL_DITHER);

	// Apply the solid style to both solid and mesh style
	// surfaces
	ls_wire_frame.Disable();
	lb_solid.Enable();

	// Set the line width for mesh rendering
	glLineWidth(line_width);

	mol->surf.RenderSurface(mol->atoms);

	if(clipView || clipSurface){
		glEnable(GL_CULL_FACE);
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
	}

	// If only the surface needed to be clipped, turn off clipping
	if(clipSurface && !clipView){
		clipper.Disable();
	}

	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);

	lb_solid.Disable();
	ls_wire_frame.Enable();
}

// Render a solid ribbon representation of a protein
void CQmolView::renderRibbon(molecule *mol)
{
	int cur_atom, next_atom, first_atom, i, j, k;
	GLfloat r[3], major[3], minor[3], normal[3], tmp[3];
	GLfloat last_r[3], last_major[3], last_minor[3];
	
	// x == major axis
	// y == minor axis
	GLfloat x, y;

	int num_seg = 1 + 2*ribbonRes;

	array<GLfloat> cos_t(num_seg);
	array<GLfloat> sin_t(num_seg);

	glEnable(GL_DITHER);
	
	if(smooth3D)
		glShadeModel(GL_SMOOTH);

	ls_wire_frame.Disable();
	lb_solid.Enable();

	int num_chain = mol->secondaryStruc.Length();

	// Precompute sin and cos. Is it worth moving this precomputation out of this
	// outer rendering loop?
	cos_t[0] = 1.0f;
	sin_t[0] = 0.0f;

	// Use x as a temporary variable
	x = (3.141592653589793238f)/ribbonRes;

	for(i = 1;i < ribbonRes;i++){
		GLfloat t = x*i;
		cos_t[i] = (GLfloat)(cos(t));

		// Note that the sign here (+/-) determines the
		// winding direction of the triangle strip below.
		sin_t[i] = -(GLfloat)(sin(t));
		
		cos_t[i + ribbonRes] = -cos_t[i];
		sin_t[i + ribbonRes] = -sin_t[i];
	}

	cos_t[ribbonRes] = -1.0f;
	sin_t[ribbonRes] = 0.0f;

	cos_t[num_seg - 1] = 1.0f;
	sin_t[num_seg - 1] = 0.0f;

	for(j = 0;j < num_chain;j++){
		array<amino> &sequence_ref = mol->secondaryStruc[j].sequence;

		int num_res = sequence_ref.Length();

		if(num_res >= 2){
			moleculeSpline spline(mol, j, splineRes, splineTension);
			
			if(mol->current_step == 0){
				
				// If we can't compute a valid spline interpolation for this
				// chain, skip to the next chain
				if(spline.NextRibbon(last_r, last_major, last_minor, first_atom) == FALSE)
					continue;

				glBegin(GL_TRIANGLE_FAN);

				// Render an end cap before rendering the rest of the ribbon
				spline.GetNormal(normal, TRUE /* Head of ribbon */);

				glColor4ubv(mol->atoms[first_atom].rgb_color);

				glNormal3fv(normal);
				glVertex3fv(last_r);

				// Draw the end cap (covering the elliptical ribbon
				// cross section).
				for(i = 0;i < num_seg;i++){
					x = cos_t[i];
					y = sin_t[i];

					tmp[X] = last_r[X] + last_major[X]*x + last_minor[X]*y;
					tmp[Y] = last_r[Y] + last_major[Y]*x + last_minor[Y]*y;
					tmp[Z] = last_r[Z] + last_major[Z]*x + last_minor[Z]*y;

					glVertex3fv(tmp);
				}

				glEnd();

				glBegin(GL_TRIANGLE_STRIP);

				while(spline.NextRibbon(r, major, minor, cur_atom)){

					if(spline.isHalfWay()){
						// If we are more than half way to the next c alpha, switch
						// to the next color!
						next_atom = min(spline.CurKnot() + 1, num_res - 1);
						next_atom = sequence_ref[next_atom].c_alpha_index;

						glColor4ubv(mol->atoms[next_atom].rgb_color);
					}
					else{
						glColor4ubv(mol->atoms[cur_atom].rgb_color);
					}

					// Draw the ellipse segment
					for(i = 0;i < num_seg;i++){
						x = cos_t[i];
						y = sin_t[i];

						for(k = 0;k < 3;k++){
							normal[k] = last_major[k]*x + last_minor[k]*y;
							tmp[k] = last_r[k] + normal[k];
						}

						glNormal3fv(normal);
						glVertex3fv(tmp);

						for(k = 0;k < 3;k++){
							normal[k] = major[k]*x + minor[k]*y;
							tmp[k] = r[k] + normal[k];
						}

						glNormal3fv(normal);
						glVertex3fv(tmp);
					}
					
					memcpy(last_r, r, 3*sizeof(float));
					memcpy(last_major, major, 3*sizeof(float));
					memcpy(last_minor, minor, 3*sizeof(float));	
				}

				glEnd();

				glBegin(GL_TRIANGLE_FAN);

				// Render an end cap before rendering the rest of the ribbon
				spline.GetNormal(normal, FALSE /* Tail of ribbon */);

				glNormal3fv(normal);
				glVertex3fv(last_r);

				// Draw the end cap (covering the elliptical ribbon
				// cross section).
				for(i = 0;i < num_seg;i++){
					x = cos_t[i];

					// Note that the minus sign here determines the
					// winding direction of the triangle strip below
					// and insures that we don't cull the endcap.
					y = -sin_t[i];

					tmp[X] = last_r[X] + last_major[X]*x + last_minor[X]*y;
					tmp[Y] = last_r[Y] + last_major[Y]*x + last_minor[Y]*y;
					tmp[Z] = last_r[Z] + last_major[Z]*x + last_minor[Z]*y;

					glVertex3fv(tmp);
				}

				glEnd();

				#ifdef REVAMP_MEASURE_CODE
				// Update the measurements of the measure dialog
				molState->Update(mol);
				#endif // REVAMP_MEASURE_CODE
			}
			else{
				int index = mol->current_step - 1;

				// If we can't compute a valid spline interpolation for this
				// chain, skip to the next chain
				if(spline.NextRibbon(last_r, last_major, last_minor, first_atom, index) == FALSE)
					continue;

				glBegin(GL_TRIANGLE_FAN);

				// Render an end cap before rendering the rest of the ribbon
				spline.GetNormal(normal, TRUE /* Head of ribbon */);

				glColor4ubv(mol->atoms[first_atom].rgb_color);

				glNormal3fv(normal);
				glVertex3fv(last_r);

				// Draw the end cap (covering the elliptical ribbon
				// cross section).
				for(i = 0;i < num_seg;i++){
					x = cos_t[i];
					y = sin_t[i];

					tmp[X] = last_r[X] + last_major[X]*x + last_minor[X]*y;
					tmp[Y] = last_r[Y] + last_major[Y]*x + last_minor[Y]*y;
					tmp[Z] = last_r[Z] + last_major[Z]*x + last_minor[Z]*y;

					glVertex3fv(tmp);
				}

				glEnd();

				glBegin(GL_TRIANGLE_STRIP);

				while(spline.NextRibbon(r, major, minor, cur_atom, index)){

					if(spline.isHalfWay()){
						// If we are more than half way to the next c alpha, switch
						// to the next color!
						next_atom = min(spline.CurKnot() + 1, num_res - 1);
						next_atom = sequence_ref[next_atom].c_alpha_index;

						glColor4ubv(mol->atoms[next_atom].rgb_color);
					}
					else{
						glColor4ubv(mol->atoms[cur_atom].rgb_color);
					}

					// Draw the ellipse segment
					for(i = 0;i < num_seg;i++){
						x = cos_t[i];
						y = sin_t[i];

						for(k = 0;k < 3;k++){
							normal[k] = last_major[k]*x + last_minor[k]*y;
							tmp[k] = last_r[k] + normal[k];
						}

						glNormal3fv(normal);
						glVertex3fv(tmp);

						for(k = 0;k < 3;k++){
							normal[k] = major[k]*x + minor[k]*y;
							tmp[k] = r[k] + normal[k];
						}

						glNormal3fv(normal);
						glVertex3fv(tmp);
					}

					memcpy(last_r, r, 3*sizeof(float));
					memcpy(last_major, major, 3*sizeof(float));
					memcpy(last_minor, minor, 3*sizeof(float));		
				}

				glEnd();

				glBegin(GL_TRIANGLE_FAN);

				// Render an end cap before rendering the rest of the ribbon
				spline.GetNormal(normal, FALSE /* Tail of ribbon */);

				glNormal3fv(normal);
				glVertex3fv(last_r);

				// Draw the end cap (covering the elliptical ribbon
				// cross section).
				for(i = 0;i < num_seg;i++){
					x = cos_t[i];

					// Note that the minus sign here determines the
					// winding direction of the triangle strip below
					// and insures that we don't cull the endcap.
					y = -sin_t[i];

					tmp[X] = last_r[X] + last_major[X]*x + last_minor[X]*y;
					tmp[Y] = last_r[Y] + last_major[Y]*x + last_minor[Y]*y;
					tmp[Z] = last_r[Z] + last_major[Z]*x + last_minor[Z]*y;

					glVertex3fv(tmp);
				}

				glEnd();

				#ifdef REVAMP_MEASURE_CODE
				// Update the measurements of the measure dialog
				molState->Update(index, mol);
				#endif // REVAMP_MEASURE_CODE
			}
		}
	}

	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);
		
	lb_solid.Disable();
	ls_wire_frame.Enable();
}

void CQmolView::renderFlatRibbon(molecule *mol)
{
	int cur_atom, next_atom, j;
	GLfloat r[3], major[3], minor[3], tmp[3];
	GLfloat last_r[3], last_major[3], last_minor[3];
	
	glDisable(GL_CULL_FACE);
	glEnable(GL_DITHER);

	// Illuminate both sides of the ribbon
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	if(smooth3D)
		glShadeModel(GL_SMOOTH);

	ls_wire_frame.Disable();
	lb_solid.Enable();

	int num_chain = mol->secondaryStruc.Length();

	for(j = 0;j < num_chain;j++){
		array<amino> &sequence_ref = mol->secondaryStruc[j].sequence;

		int num_res = sequence_ref.Length();

		if(num_res >= 2){
			moleculeSpline spline(mol, j, splineRes, splineTension);
			
			glBegin(GL_QUAD_STRIP);

			if(mol->current_step == 0){
				// If we can't compute a valid spline interpolation for this
				// chain, skip to the next chain
				if(spline.NextRibbon(last_r, last_major, last_minor, cur_atom) == FALSE)
					continue;

				while(spline.NextRibbon(r, major, minor, cur_atom)){

					if(spline.isHalfWay()){
						// If we are more than half way to the next c alpha, switch
						// to the next color!
						next_atom = min(spline.CurKnot() + 1, num_res - 1);
						next_atom = sequence_ref[next_atom].c_alpha_index;

						glColor4ubv(mol->atoms[next_atom].rgb_color);
					}
					else{
						// This is the color of the current residue
						glColor4ubv(mol->atoms[cur_atom].rgb_color);
					}

					// Draw a flat ribbon -- the vertex winding order is important
					// for directional lighting!
					tmp[X] = last_r[X] + last_major[X];
					tmp[Y] = last_r[Y] + last_major[Y];
					tmp[Z] = last_r[Z] + last_major[Z];

					glNormal3fv(last_minor);
					glVertex3fv(tmp);

					tmp[X] = last_r[X] - last_major[X];
					tmp[Y] = last_r[Y] - last_major[Y];
					tmp[Z] = last_r[Z] - last_major[Z];

					glVertex3fv(tmp);

					tmp[X] = r[X] + major[X];
					tmp[Y] = r[Y] + major[Y];
					tmp[Z] = r[Z] + major[Z];

					glNormal3fv(minor);
					glVertex3fv(tmp);

					tmp[X] = r[X] - major[X];
					tmp[Y] = r[Y] - major[Y];
					tmp[Z] = r[Z] - major[Z];

					glVertex3fv(tmp);

					memcpy(last_r, r, 3*sizeof(float));
					memcpy(last_major, major, 3*sizeof(float));
					memcpy(last_minor, minor, 3*sizeof(float));	
				}

				#ifdef REVAMP_MEASURE_CODE
				// Update the measurements of the measure dialog
				molState->Update(mol);
				#endif // REVAMP_MEASURE_CODE
			}
			else{
				int index = mol->current_step - 1;

				// If we can't compute a valid spline interpolation for this
				// chain, skip to the next chain
				if(spline.NextRibbon(last_r, last_major, last_minor, cur_atom, index) == FALSE)
					continue;

				while(spline.NextRibbon(r, major, minor, cur_atom, index)){

					if(spline.isHalfWay()){
						// If we are more than half way to the next c alpha, switch
						// to the next color!
						next_atom = min(spline.CurKnot() + 1, num_res - 1);
						next_atom = sequence_ref[next_atom].c_alpha_index;

						glColor4ubv(mol->atoms[next_atom].rgb_color);
					}
					else{
						glColor4ubv(mol->atoms[cur_atom].rgb_color);
					}

					// Draw a flat ribbon -- the vertex winding order is important
					// for directional lighting!
					tmp[X] = last_r[X] + last_major[X];
					tmp[Y] = last_r[Y] + last_major[Y];
					tmp[Z] = last_r[Z] + last_major[Z];

					glNormal3fv(last_minor);
					glVertex3fv(tmp);

					tmp[X] = last_r[X] - last_major[X];
					tmp[Y] = last_r[Y] - last_major[Y];
					tmp[Z] = last_r[Z] - last_major[Z];

					glVertex3fv(tmp);

					tmp[X] = r[X] + major[X];
					tmp[Y] = r[Y] + major[Y];
					tmp[Z] = r[Z] + major[Z];

					glNormal3fv(minor);
					glVertex3fv(tmp);

					tmp[X] = r[X] - major[X];
					tmp[Y] = r[Y] - major[Y];
					tmp[Z] = r[Z] - major[Z];

					glVertex3fv(tmp);

					memcpy(last_r, r, 3*sizeof(float));
					memcpy(last_major, major, 3*sizeof(float));
					memcpy(last_minor, minor, 3*sizeof(float));	
				}
				
				#ifdef REVAMP_MEASURE_CODE
				// Update the measurements of the measure dialog
				molState->Update(index, mol);
				#endif // REVAMP_MEASURE_CODE
			}

			glEnd();
		}
	}

	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
	glEnable(GL_CULL_FACE);
	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);
		
	lb_solid.Disable();
	ls_wire_frame.Enable();
}

// Render a ball and stick model of the molecule. When the ball
// and radius are equal, the model reduces to the stick (or "tube", or
// "liquorice" model).
void CQmolView::renderBallAndStick(molecule *mol, GLfloat m[][4])
{
	int a, b, i;
	float ra[3], rb[3];

	ls_wire_frame.Disable();
	lb_solid.Enable();
	
	glEnable(GL_DITHER);
	
	if(smooth3D)
		glShadeModel(GL_SMOOTH);

	if(mol->current_step == 0){
		for(i = 0;i < mol->bonds.GetLength();i++)
			if(!(!mol->solvent && mol->bonds[i].solvent)){
				// Traverse the bond list drawing tubes
				a = mol->bonds[i].a;
				b = mol->bonds[i].b;
			
				mol->atoms[a].GetCoord(ra);
				mol->atoms[b].GetCoord(rb);

				rb[0] -= ra[0];
				rb[1] -= ra[1];
				rb[2] -= ra[2];
			
				glPushMatrix();
				
				float bond_length = (float)sqrt(rb[0]*rb[0] + rb[1]*rb[1] + rb[2]*rb[2]);

				rb[0] = rb[0]/bond_length;
				rb[1] = rb[1]/bond_length;
				rb[2] = rb[2]/bond_length;
				
				glTranslatef(ra[0], ra[1], ra[2]);
				
				glRotatef((float)(-acos(rb[2])*57.2957795131), rb[1], -rb[0], 0.0f);

				// Determine how many verticies must be rendered
				// Is the bond between the same color atoms or
				// different color atoms?
				if(mol->bonds[i].colorBond){					
					glScalef(stickRadius, stickRadius, 0.5f*bond_length/CYLINDER_HEIGHT);
			
					// To get colored bonds, each bond has to be
					// contstructed out of two different colored sticks.
					glColor4ubv(mol->atoms[a].rgb_color);
					glCallList(BOND_CYL);
					
					glTranslatef(0.0f, 0.0f, CYLINDER_HEIGHT);

					glColor4ubv(mol->atoms[b].rgb_color);
					glCallList(BOND_CYL);
				}
				else{
					glScalef(stickRadius, stickRadius, bond_length/CYLINDER_HEIGHT);
					glColor4ubv(mol->atoms[a].rgb_color);
					glCallList(BOND_CYL);
				}

				glPopMatrix();
			}
			
		for(i = 0;i < mol->NumAtoms();i++){
			if(!(!mol->solvent && mol->atoms[i].isSolvent)){
    			glPushMatrix();
    			glTranslatef(mol->atoms[i].coord[X], mol->atoms[i].coord[Y], 
					mol->atoms[i].coord[Z]);
				glScalef(ballRadius, ballRadius, ballRadius);

				// **** "Undo" any perspective rotation ****
				glMultMatrixf(&m[0][0]);

				// **** "Undo" any stero rotation ****
				// Are we currently rendering the right eye of a split
				// screen stereo view? Note that in the current scheme of
				// things, the positive y axis is "up".
				if(renderSplitStereoRightEye)
					glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

				if(renderSplitStereoLeftEye)
					glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

				glColor4ubv(mol->atoms[i].rgb_color);
    			glCallList(ATOM_SPHERE);
    			glPopMatrix();
			}
  		}

		#ifdef REVAMP_MEASURE_CODE
		// Update the measurements of the measure dialog
		molState->Update(mol);
		#endif // REVAMP_MEASURE_CODE
	}else{
		//int index = (mol->current_step - 1)*3*mol->atoms.Length();
		int index = mol->current_step - 1;

		for(i = 0;i < mol->bonds.GetLength();i++)
			if(!(!mol->solvent && mol->bonds[i].solvent)){
				// Traverse the bond list
				a = mol->bonds[i].a;
				b = mol->bonds[i].b;
			
				memcpy(ra, &(mol->traj[index][3*a]), 3*sizeof(float));
				memcpy(rb, &(mol->traj[index][3*b]), 3*sizeof(float));

				rb[0] -= ra[0];
				rb[1] -= ra[1];   
				rb[2] -= ra[2];
				
				glPushMatrix();
				
				float bond_length = (float)sqrt(rb[0]*rb[0] + rb[1]*rb[1] + rb[2]*rb[2]);

				rb[0] = rb[0]/bond_length;
				rb[1] = rb[1]/bond_length;
				rb[2] = rb[2]/bond_length;
				
				glTranslatef(ra[0], ra[1], ra[2]);
				
				glRotatef((float)(-acos(rb[2])*57.2957795131), rb[1], -rb[0], 0.0f);

				// Determine how many verticies must be rendered
				// Is the bond between the same color atoms or
				// different color atoms?
				if(mol->bonds[i].colorBond){					
					glScalef(stickRadius, stickRadius, 0.5f*bond_length/CYLINDER_HEIGHT);
			
					// To get colored bonds, each bond has to be
					// contstructed out of two different colored sticks.
					glColor4ubv(mol->atoms[a].rgb_color);
					glCallList(BOND_CYL);
					
					glTranslatef(0.0f, 0.0f, CYLINDER_HEIGHT);

					glColor4ubv(mol->atoms[b].rgb_color);
					glCallList(BOND_CYL);
				}
				else{
					glScalef(stickRadius, stickRadius, bond_length/CYLINDER_HEIGHT);
					glColor4ubv(mol->atoms[a].rgb_color);
					glCallList(BOND_CYL);
				}

				glPopMatrix();
			}
			
		for(i = 0;i < mol->NumAtoms();i++){
			if(!(!mol->solvent && mol->atoms[i].isSolvent)){
    			glPushMatrix();
    			glTranslatef(mol->traj[index][3*i], 
					mol->traj[index][3*i + 1], 
					mol->traj[index][3*i + 2]);
				glScalef(ballRadius, ballRadius, ballRadius);

				// **** "Undo" any perspective rotation ****
				glMultMatrixf(&m[0][0]);

				// **** "Undo" any stero rotation ****
				// Are we currently rendering the right eye of a split
				// screen stereo view? Note that in the current scheme of
				// things, the positive y axis is "up".
				if(renderSplitStereoRightEye)
					glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

				if(renderSplitStereoLeftEye)
					glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

				glColor4ubv(mol->atoms[i].rgb_color);
    			glCallList(ATOM_SPHERE);
    			glPopMatrix();
			}
  		}

		#ifdef REVAMP_MEASURE_CODE
		// Update the measurements of the measure dialog
		molState->Update(index, mol);
		#endif // REVAMP_MEASURE_CODE
	}
}

void CQmolView::renderTraceStick(molecule *mol, GLfloat m[][4])
{
	int a, b, i, j;
	float ra[3], rb[3];

	int num_chain = mol->secondaryStruc.Length();

	for(j = 0;j < num_chain;j++){
		array<amino> &sequence_ref = mol->secondaryStruc[j].sequence;

		int num_res = sequence_ref.Length();

		// Only need to render a trace if we have two or more amino
		// acids in the protein!
		if(num_res >= 2){

			ls_wire_frame.Disable();
			lb_solid.Enable();
			
			glEnable(GL_DITHER);
			
			if(smooth3D)
				glShadeModel(GL_SMOOTH);

			if(mol->current_step == 0){
				for(i = 0;i < num_res - 1;i++){
					// Traverse the bond list drawing tubes
					a = sequence_ref[i].c_alpha_index;
					b = sequence_ref[i + 1].c_alpha_index;
					
					mol->atoms[a].GetCoord(ra);
					mol->atoms[b].GetCoord(rb);

					rb[0] -= ra[0];
					rb[1] -= ra[1];
					rb[2] -= ra[2];
					
					glPushMatrix();
						
					float bond_length = (float)sqrt(rb[0]*rb[0] + rb[1]*rb[1] + rb[2]*rb[2]);

					rb[0] = rb[0]/bond_length;
					rb[1] = rb[1]/bond_length;
					rb[2] = rb[2]/bond_length;
						
					glTranslatef(ra[0], ra[1], ra[2]);
						
					glRotatef((float)(-acos(rb[2])*57.2957795131), rb[1], -rb[0], 0.0f);

					glScalef(stickRadius, stickRadius, bond_length/CYLINDER_HEIGHT);
					glColor4ubv(mol->atoms[a].rgb_color);
					glCallList(BOND_CYL);

					glPopMatrix();
				}
					
				for(i = 0;i < num_res;i++){
					a = sequence_ref[i].c_alpha_index;

    				glPushMatrix();
    				glTranslatef(mol->atoms[a].coord[X], mol->atoms[a].coord[Y], 
						mol->atoms[a].coord[Z]);
					glScalef(ballRadius, ballRadius, ballRadius);

					// **** "Undo" any perspective rotation ****
					glMultMatrixf(&m[0][0]);

					// **** "Undo" any stero rotation ****
					// Are we currently rendering the right eye of a split
					// screen stereo view? Note that in the current scheme of
					// things, the positive y axis is "up".
					if(renderSplitStereoRightEye)
						glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

					if(renderSplitStereoLeftEye)
						glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

					glColor4ubv(mol->atoms[a].rgb_color);
    				glCallList(ATOM_SPHERE);
    				glPopMatrix();
				}

				#ifdef REVAMP_MEASURE_CODE
				// Update the measurements of the measure dialog
				molState->Update(mol);
				#endif // REVAMP_MEASURE_CODE
			}
			else{
				//int index = (mol->current_step - 1)*3*mol->atoms.Length();
				int index = mol->current_step - 1;

				for(i = 0;i < num_res - 1;i++){
					// Traverse the bond list
					a = sequence_ref[i].c_alpha_index;
					b = sequence_ref[i + 1].c_alpha_index;
					
					memcpy(ra, &(mol->traj[index][3*a]), 3*sizeof(float));
					memcpy(rb, &(mol->traj[index][3*b]), 3*sizeof(float));

					rb[0] -= ra[0];
					rb[1] -= ra[1];   
					rb[2] -= ra[2];
						
					glPushMatrix();
						
					float bond_length = (float)sqrt(rb[0]*rb[0] + rb[1]*rb[1] + rb[2]*rb[2]);

					rb[0] = rb[0]/bond_length;
					rb[1] = rb[1]/bond_length;
					rb[2] = rb[2]/bond_length;
						
					glTranslatef(ra[0], ra[1], ra[2]);
						
					glRotatef((float)(-acos(rb[2])*57.2957795131), rb[1], -rb[0], 0.0f);

					glScalef(stickRadius, stickRadius, bond_length/CYLINDER_HEIGHT);
					glColor4ubv(mol->atoms[a].rgb_color);
					glCallList(BOND_CYL);

					glPopMatrix();
				}
					
				for(i = 0;i < num_res;i++){
					a = sequence_ref[i].c_alpha_index;
    				glPushMatrix();
    				glTranslatef(mol->traj[index][3*a], 
						mol->traj[index][3*a + 1], 
						mol->traj[index][3*a + 2]);
					glScalef(ballRadius, ballRadius, ballRadius);

					// **** "Undo" any perspective rotation ****
					glMultMatrixf(&m[0][0]);

					// **** "Undo" any stero rotation ****
					// Are we currently rendering the right eye of a split
					// screen stereo view? Note that in the current scheme of
					// things, the positive y axis is "up".
					if(renderSplitStereoRightEye)
						glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

					if(renderSplitStereoLeftEye)
						glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

					glColor4ubv(mol->atoms[a].rgb_color);
    				glCallList(ATOM_SPHERE);
    				glPopMatrix();
				}

				#ifdef REVAMP_MEASURE_CODE
				// Update the measurements of the measure dialog
				molState->Update(index, mol);
				#endif // REVAMP_MEASURE_CODE
			}
		}
	}
}


void CQmolView::renderVanDerWaals(molecule *mol, GLfloat m[][4])
{
	// Note! I'm assuming that the proper translation and
	// scale factors have allready been set BEFORE this function
	// is called!!!
	int i;

	ls_wire_frame.Disable();
	lb_solid.Enable();

	glEnable(GL_DITHER);

	if(smooth3D)
		glShadeModel(GL_SMOOTH);
	
	int num_atom = mol->NumAtoms();

	if(mol->current_step == 0){
		
  		for(i = 0;i < num_atom;i++) {

			atom &atom_ref = mol->atoms[i];

			if(!(!mol->solvent && atom_ref.isSolvent)){
				if(clipView){
					// Does the current atom intersect the clipping plane?
					BOOL clip_status;
					
					// Due to the abysmal performance of consumer graphics
					// cards (that lack hardware support for clipping planes)
					// pre-sort the atoms into one of three catagories:
					// RENDERED (i.e. turn clipping off), NOT_RENDERED (i.e.
					// totally clipped, so skip the atom) and CLIPED (let OpenGL
					// clip the atom for us).
					clip_status = clipper.SphereClipTest(
							 atom_ref.coord,
							 atom_ref.radius,
							 SCALE_FACTOR / maxDim, 
							 center[X],
    						 -center[Y],
    						 center[Z],
							 m);
					switch(clip_status){
						case ClipPlane::NOT_RENDERED:
							continue;
							break;
						case ClipPlane::RENDERED:
							clipper.Disable();
							break;
						case ClipPlane::CLIPED:
							clipper.Enable();
							break;
					};
				}

    			glPushMatrix();
    			glTranslatef(atom_ref.coord[X], 
					 		 atom_ref.coord[Y], 
					 		 atom_ref.coord[Z]);
				glScalef(atom_ref.radius, 
						 atom_ref.radius, 
						 atom_ref.radius);
				
				// **** "Undo" any perspective rotation ****
				glMultMatrixf(&m[0][0]);

				// **** "Undo" any stero rotation ****
				// Are we currently rendering the right eye of a split
				// screen stereo view? Note that in the current scheme of
				// things, the positive y axis is "up".
				if(renderSplitStereoRightEye){
					glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);
				}
				else{
					if(renderSplitStereoLeftEye)
						glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);
				}

				glColor4ubv(atom_ref.rgb_color);
    			glCallList(ATOM_SPHERE);
    			glPopMatrix();
			}
  		}
		
		// If clipping is enabled, reactivate on the chance that we
		// disabled it for fast atom rendering.
		if(clipView){
			clipper.Enable();

			clipper.RenderClipSphere(mol->atoms, atomRes, mol->solvent,
									 SCALE_FACTOR / maxDim, 
									 center[X],
    								 -center[Y],
    								 center[Z],
									 m);
		}

		#ifdef REVAMP_MEASURE_CODE
		// Update the measurements of the measure dialog
		molState->Update(mol);
		#endif // REVAMP_MEASURE_CODE
	}
	else{
		int index = mol->current_step - 1;
		
		for(i = 0;i < num_atom;i++) {
			atom &atom_ref = mol->atoms[i];
			
			if(!(!mol->solvent && atom_ref.isSolvent)){

				if(clipView){
					// Does the current atom intersect the clipping plane?
					BOOL clip_status;
					
					// Due to the abysmal performance of consumer graphics
					// cards (that lack hardware support for clipping planes)
					// pre-sort the atoms into one of three catagories:
					// RENDERED (i.e. turn clipping off), NOT_RENDERED (i.e.
					// totally clipped, so skip the atom) and CLIPED (let OpenGL
					// clip the atom for us).
					clip_status = clipper.SphereClipTest(
							 &(mol->traj[index][3*i]),
							 atom_ref.radius,
							 SCALE_FACTOR / maxDim, 
							 center[X],
    						 -center[Y],
    						 center[Z],
							 m);
					switch(clip_status){
						case ClipPlane::NOT_RENDERED:
							continue;
							break;
						case ClipPlane::RENDERED:
							clipper.Disable();
							break;
						case ClipPlane::CLIPED:
							clipper.Enable();
							break;
					};
				}

    			glPushMatrix();
    			glTranslatef(mol->traj[index][3*i], 
							 mol->traj[index][3*i + 1], 
							 mol->traj[index][3*i + 2]);
				glScalef(atom_ref.radius, 
						 atom_ref.radius, 
						 atom_ref.radius);

				// **** "Undo" any perspective rotation ****
				glMultMatrixf(&m[0][0]);

				// **** "Undo" any stero rotation ****
				// Are we currently rendering the right eye of a split
				// screen stereo view? Note that in the current scheme of
				// things, the positive y axis is "up".
				if(renderSplitStereoRightEye)
					glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

				if(renderSplitStereoLeftEye)
					glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

				glColor4ubv(atom_ref.rgb_color);

    			glCallList(ATOM_SPHERE);
    			glPopMatrix();
			}
  		}

		// If clipping is enabled, reactivate on the chance that we
		// disabled it for fast atom rendering.
		if(clipView){
			clipper.Enable();
		
			clipper.RenderClipSphere(index, mol->traj, 
									 mol->atoms, atomRes, mol->solvent,
									 SCALE_FACTOR / maxDim, 
									 center[X],
    								 -center[Y],
    								 center[Z],
									 m);
		}

		#ifdef REVAMP_MEASURE_CODE
		// Update the measurements of the measure dialog
		molState->Update(index, mol);
		#endif // REVAMP_MEASURE_CODE
	}
	
	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);
		
	lb_solid.Disable();
	ls_wire_frame.Enable();
}

// The point rendering style is usefull for very large
// molecules.
void CQmolView::renderPoints(molecule *mol, GLfloat m[][4])
{
	// Note! I'm assuming that the proper translation and
	// scale factors have allready been set BEFORE this function
	// is called!!!
	int i;
	
	// Use the wireframe light model
	lb_solid.Disable();
	ls_wire_frame.Enable();

	// Make the points larg-ish so they are easy to see
	glPointSize(2.0);

	// Handle clipping manually
	if(clipView)
		clipper.Disable();

	glBegin(GL_POINTS);
		
	if(mol->current_step == 0){
		
  		for(i = 0;i < mol->NumAtoms();i++){
			if(!(!mol->solvent && mol->atoms[i].isSolvent)){
				if(clipView){

					// Does the current atom intersect the clipping plane?
					BOOL clip_status;
					
					// Due to the abysmal performance of consumer graphics
					// cards (that lack hardware support for clipping planes)
					// pre-sort the atoms into one of three catagories:
					// RENDERED (i.e. turn clipping off), NOT_RENDERED (i.e.
					// totally clipped, so skip the atom) and CLIPED (let OpenGL
					// clip the atom for us).
					clip_status = clipper.SphereClipTest(
							 mol->atoms[i].coord, 
							 0.0f,	// Set the radius to zero for points
							 SCALE_FACTOR / maxDim, 
							 center[X],
    						 -center[Y],
    						 center[Z],
							 m);
					if(clip_status == ClipPlane::NOT_RENDERED)
						continue;
				}

				glColor4ubv(mol->atoms[i].rgb_color);
				glVertex3fv(mol->atoms[i].coord);
			}
  		}

		#ifdef REVAMP_MEASURE_CODE
		// Update the measurements of the measure dialog
		molState->Update(mol);
		#endif // REVAMP_MEASURE_CODE
	}
	else{
		int index = mol->current_step - 1;
		
		for(i = 0;i < mol->NumAtoms();i++) {
			if(!(!mol->solvent && mol->atoms[i].isSolvent)){
				if(clipView){
					// Does the current atom intersect the clipping plane?
					BOOL clip_status;
					
					// Due to the abysmal performance of consumer graphics
					// cards (that lack hardware support for clipping planes)
					// pre-sort the atoms into one of three catagories:
					// RENDERED (i.e. turn clipping off), NOT_RENDERED (i.e.
					// totally clipped, so skip the atom) and CLIPED (let OpenGL
					// clip the atom for us).
					clip_status = clipper.SphereClipTest(
							 &(mol->traj[index][3*i]), 
							 0.0f,	// Set the radius to zero for points
							 SCALE_FACTOR / maxDim, 
							 mol->centerX,
    						 -mol->centerY,
    						 mol->centerZ,
							 m);
					if(clip_status == ClipPlane::NOT_RENDERED)
						continue;
				}

				glColor4ubv(mol->atoms[i].rgb_color);
				glVertex3fv(&(mol->traj[index][3*i]));
			}
  		}

		#ifdef REVAMP_MEASURE_CODE
		// Update the measurements of the measure dialog
		molState->Update(index, mol);
		#endif // REVAMP_MEASURE_CODE
	}
	
	glEnd();

	// Return to the small point size
	glPointSize(1.0);

	if(clipView)
		clipper.Enable();

	ls_wire_frame.Disable();
	lb_solid.Enable();
}

void CQmolView::renderTorsion()
{		
	// Note that the adjust torsion angle dialog
	// box must be currently displayed to display selected torsion angles.
	if(!m_pATDlg->GetSafeHwnd() || !tor_adjust.IsSelected())
		return;

	// Note! I'm assuming that the proper translation and
	// scale factors have allready been set BEFORE this function
	// is called!!!
	float c[3], r[3], tmp[3];
	float mag;

	// Use translucent disk to indicate selected torsion angle
	ls_wire_frame.Disable();
	lb_solid.Enable();
	
	glEnable(GL_DITHER);
	glDisable(GL_CULL_FACE);

	if(smooth3D)
		glShadeModel(GL_SMOOTH);
	
	glColor4fv(selection_color);
	
	molecule *mol = tor_adjust.GetMol();
	int a = tor_adjust.GetAtomA();
	int b = tor_adjust.GetAtomB();

	if(mol->current_step == 0){
		memcpy(tmp, mol->atoms[a].coord, 3*sizeof(float));
		memcpy(r, mol->atoms[b].coord, 3*sizeof(float));
	}
	else{
		int index = mol->current_step - 1;

		memcpy(tmp, &(mol->traj[index][3*a]), 
			3*sizeof(float));
		memcpy(r, &(mol->traj[index][3*b]), 
			3*sizeof(float));

	}
	
	c[X] = 0.5f*(tmp[X] + r[X]);
	c[Y] = 0.5f*(tmp[Y] + r[Y]);
	c[Z] = 0.5f*(tmp[Z] + r[Z]);

	r[X] -= tmp[X];
	r[Y] -= tmp[Y];
	r[Z] -= tmp[Z];

	mag = 1.0f/(float)(sqrt(DOT3D(r, r)));

	r[X] *= mag;
	r[Y] *= mag;
	r[Z] *= mag;
		
	glPushMatrix();
		
	glTranslatef(c[0], c[1], c[2]);

	glRotatef((float)(-acos(r[2])*57.2957795131), r[1], -r[0], 0.0f);
		
	glTranslatef(0.0f, 0.0f, - 0.5f*CYLINDER_HEIGHT);
	glCallList(TORSION_DISK);
	glCallList(TORSION_CYL);
		
	glTranslatef(0.0f, 0.0f, CYLINDER_HEIGHT);
	glCallList(TORSION_DISK);
	glPopMatrix();
	
	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);

	lb_solid.Disable();
	ls_wire_frame.Enable();
	glEnable(GL_CULL_FACE);
}

// Render atom monitors (currently indicated by a cylindrical bar
// connecting two atom spheres).
void CQmolView::renderMonitors(molecule *mol, GLfloat m[][4])
{
	int num_monitors = mol->mon.Length();

	// Do we any thing to render?
	if(num_monitors == 0)
		return;

	// Note! I'm assuming that the proper translation and
	// scale factors have allready been set BEFORE this function
	// is called!!!
	int i;
	float a[3], b[3];
	float mag;

	// A static buffer for monitor labels
	char buffer[16];

	ls_wire_frame.Disable();
	lb_solid.Enable();
	
	glEnable(GL_DITHER);
	
	if(smooth3D)
		glShadeModel(GL_SMOOTH);
		

	for(i = 0;i < num_monitors;i++){
		
		if(mol->current_step == 0){
			memcpy(a, mol->atoms[mol->mon[i].atom_id[0]].coord, 3*sizeof(float));

			glPushMatrix();
    		glTranslatef(a[X], a[Y], a[Z]);

			glScalef(MONITOR_RADIUS, MONITOR_RADIUS, MONITOR_RADIUS);

			// **** "Undo" any perspective rotation ****
			glMultMatrixf(&m[0][0]);

			// **** "Undo" any stero rotation ****
			// Are we currently rendering the right eye of a split
			// screen stereo view? Note that in the current scheme of
			// things, the positive y axis is "up".
			if(renderSplitStereoRightEye)
				glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

			if(renderSplitStereoLeftEye)
				glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

			glColor3ubv(mol->mon[i].color);
    		glCallList(ATOM_SPHERE);
    		glPopMatrix();

			glPushMatrix();
				
			memcpy(b, mol->atoms[mol->mon[i].atom_id[1]].coord, 3*sizeof(float));

		}
		else{
			int index = mol->current_step - 1;
			memcpy(a, &(mol->traj[index][3*mol->mon[i].atom_id[0]]), 3*sizeof(float));

			glPushMatrix();
    		glTranslatef(a[X], a[Y], a[Z]);

			glScalef(MONITOR_RADIUS, MONITOR_RADIUS, MONITOR_RADIUS);

			// **** "Undo" any perspective rotation ****
			glMultMatrixf(&m[0][0]);

			// **** "Undo" any stero rotation ****
			// Are we currently rendering the right eye of a split
			// screen stereo view? Note that in the current scheme of
			// things, the positive y axis is "up".
			if(renderSplitStereoRightEye)
				glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

			if(renderSplitStereoLeftEye)
				glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

			glColor3ubv(mol->mon[i].color);
    		glCallList(ATOM_SPHERE);
    		glPopMatrix();

			glPushMatrix();
				
			memcpy(b, &(mol->traj[index][3*mol->mon[i].atom_id[1]]), 3*sizeof(float));

		}

		glTranslatef(b[X], b[Y], b[Z]);

		b[X] -= a[X];
		b[Y] -= a[Y];
		b[Z] -= a[Z];

		glScalef(MONITOR_RADIUS, MONITOR_RADIUS, MONITOR_RADIUS);

		// **** "Undo" any perspective rotation ****
		glMultMatrixf(&m[0][0]);

		// **** "Undo" any stero rotation ****
		// Are we currently rendering the right eye of a split
		// screen stereo view? Note that in the current scheme of
		// things, the positive y axis is "up".
		if(renderSplitStereoRightEye)
			glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

		if(renderSplitStereoLeftEye)
			glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

		glColor3ubv(mol->mon[i].color);
    	glCallList(ATOM_SPHERE);
    	glPopMatrix();
			
		// Add labels to the monitors
		glPushMatrix();

		// Turn off lighting effects for easy to read labels
		glDisable(GL_LIGHTING);

		if(mol->monitorLabels){
			sprintf(buffer,"%d", mol->mon[i].pairId);
			
			// Color the atom labels black (so that they may be clearly visable)
			glColor3f(1.0f - background[0], 
					  1.0f - background[1], 
					  1.0f - background[2]);
			
			glRasterPos3f(a[X] + 0.5f*b[X], a[Y] + 0.5f*b[Y], a[Z] + 0.5f*b[Z]);
  			glCallLists(strlen(buffer), GL_UNSIGNED_BYTE, buffer);
		}
			
		glEnable(GL_LIGHTING);

		glPopMatrix();
			
		// Restore the appropriate atom color
		glColor3ubv(mol->mon[i].color);

		mag = 1.0f/(float)sqrt(DOT3D(b, b));

		b[X] *= mag;
		b[Y] *= mag;
		b[Z] *= mag;
		
		glPushMatrix();
		
		glTranslatef(a[X], a[Y], a[Z]);

		glRotatef((float)(-acos(b[Z])*57.2957795131), b[Y], -b[X], 0.0f);

		glScalef(0.5*MONITOR_RADIUS, 0.5*MONITOR_RADIUS, 1.0f/(mag*CYLINDER_HEIGHT));

		glCallList(BOND_CYL);
		
		glPopMatrix();
  	}

	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);

	lb_solid.Disable();
	ls_wire_frame.Enable();
}

void CQmolView::renderHBonds()
{
	// Note! I'm assuming that the proper translation and
	// scale factors have allready been set BEFORE this function
	// is called!!!
	int j;

	GLfloat tmpA[3], tmpB[3];
	GLfloat rA[3], rB[3];

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Nothing to render!
	if(pDoc->hbonds.Length() == 0)
		return;

	const int num_hbond_stipple = 6;	
	const GLfloat mag = 0.5f/(num_hbond_stipple);

	glPushMatrix();

	glScalef(SCALE_FACTOR / maxDim,
			 SCALE_FACTOR / maxDim,
			 SCALE_FACTOR / maxDim);

	glLineWidth(line_width);

	lb_solid.Disable();
	ls_wire_frame.Enable();
	
	glBegin(GL_LINES);
	glColor3fv(hbond_color);
	
	ListIterator<QmolHBond> next(pDoc->hbonds);
	QmolHBond *tmpBond;

	while(tmpBond = next()){
		molecule *molA = tmpBond->molA;
		molecule *molB = tmpBond->molB;
		
		// Use a fixed number of line segments to render a stippled line.
		// This is NOT equvalent to setting line stippling in OpenGL (as
		// the number of segments will NOT be constant).
		if(molA->current_step == 0)
			memcpy(tmpA, molA->atoms[tmpBond->indexA].coord, 3*sizeof(GLfloat));
		else
			memcpy(tmpA, &(molA->traj[molA->current_step - 1][3*tmpBond->indexA]), 
				3*sizeof(GLfloat));

		if(molB->current_step == 0)
			memcpy(tmpB, molB->atoms[tmpBond->indexB].coord, 3*sizeof(GLfloat));
		else
			memcpy(tmpB, &(molB->traj[molB->current_step - 1][3*tmpBond->indexB]), 
				3*sizeof(GLfloat));
		
		
		TRANSLATE(tmpA,molA->R);
		TRANSLATE(tmpB,molB->R);

		float m[4][4];

		// Are both atoms from the same molecule?
		if(molA == molB){

			molA->BuildRotationMatrix(m);

			rA[X] = m[X][X]*tmpA[X] + m[Y][X]*tmpA[Y] + m[Z][X]*tmpA[Z];
			rA[Y] = m[X][Y]*tmpA[X] + m[Y][Y]*tmpA[Y] + m[Z][Y]*tmpA[Z];
			rA[Z] = m[X][Z]*tmpA[X] + m[Y][Z]*tmpA[Y] + m[Z][Z]*tmpA[Z];

			rB[X] = m[X][X]*tmpB[X] + m[Y][X]*tmpB[Y] + m[Z][X]*tmpB[Z];
			rB[Y] = m[X][Y]*tmpB[X] + m[Y][Y]*tmpB[Y] + m[Z][Y]*tmpB[Z];
			rB[Z] = m[X][Z]*tmpB[X] + m[Y][Z]*tmpB[Y] + m[Z][Z]*tmpB[Z];

		}
		else{
			float m[4][4];

			molA->BuildRotationMatrix(m);

			rA[X] = m[X][X]*tmpA[X] + m[Y][X]*tmpA[Y] + m[Z][X]*tmpA[Z];
			rA[Y] = m[X][Y]*tmpA[X] + m[Y][Y]*tmpA[Y] + m[Z][Y]*tmpA[Z];
			rA[Z] = m[X][Z]*tmpA[X] + m[Y][Z]*tmpA[Y] + m[Z][Z]*tmpA[Z];

			molB->BuildRotationMatrix(m);

			rB[X] = m[X][X]*tmpB[X] + m[Y][X]*tmpB[Y] + m[Z][X]*tmpB[Z];
			rB[Y] = m[X][Y]*tmpB[X] + m[Y][Y]*tmpB[Y] + m[Z][Y]*tmpB[Z];
			rB[Z] = m[X][Z]*tmpB[X] + m[Y][Z]*tmpB[Y] + m[Z][Z]*tmpB[Z];
		}

		rA[X] -= rB[X];
		rA[Y] -= rB[Y];
		rA[Z] -= rB[Z];

		rA[X] *= mag;
		rA[Y] *= mag;
		rA[Z] *= mag;
		
		for(j = 0;j < num_hbond_stipple;j ++){
			glVertex3fv(rB);
			
			rB[X] += rA[X];
			rB[Y] += rA[Y];
			rB[Z] += rA[Z];
			
			glVertex3fv(rB);
			
			rB[X] += rA[X];
			rB[Y] += rA[Y];
			rB[Z] += rA[Z];
		}
	}
	
	glEnd();

	glPopMatrix();
}

// A function for rendering the current molecule into the openGL
// selection  context
void CQmolView::renderSelect(molecule *mol, int &offset)
{
	// Only render to the selection buffer if the user
	// has selected one of the following display settings
	switch(mol->displayType){
		case QM_RENDER_TRACE_SPLINE:
		case QM_RENDER_RIBBON:
		case QM_RENDER_RIBBON_FLAT:
		case QM_RENDER_TUBE:
		case QM_RENDER_TUBE_OCC:
		case QM_RENDER_TUBE_T:
			if(!mol->showWireFrame)
				return;

			break;
		case QM_RENDER_NONE:
			return;
	};

	float r[3], tmp[3];
	int i, a, b;
	int num_atoms = mol->atoms.Length();
	int num_bonds = mol->bonds.GetLength();

	glPushMatrix();

  	glScalef(SCALE_FACTOR / maxDim,
			 SCALE_FACTOR / maxDim,
			 SCALE_FACTOR / maxDim);
	

	// Apply the translation vector in the rotated coordinates
	glTranslatef(mol->R[X],
    			 mol->R[Y],
    			 mol->R[Z]);

	if(mol->current_step == 0){
	
		// Render atoms into the selection buffer
		if(mol->solvent){
			for(i = 0;i < num_atoms;i++) {
				// Note that in order for the selection process to work,
				// glLoadName must be called BEFORE glBegin(GL_POINTS)!!!
				glLoadName(i + offset);
				glBegin(GL_POINTS);
				glVertex3fv(mol->atoms[i].coord);
				glEnd();
			}
		}
		else{ // !mol->solvent
			for(i = 0;i < num_atoms;i++) {
				// Don't render solvent atoms into the selection buffer
				if(!mol->atoms[i].isSolvent){
					// Note that in order for the selection process to work,
					// glLoadName must be called BEFORE glBegin(GL_POINTS)!!!
					glLoadName(i + offset);
					glBegin(GL_POINTS);
					glVertex3fv(mol->atoms[i].coord);
					glEnd();
				}
			}
		}
		
		// Render bonds in the selection buffer (as points placed at the
		// midpoint of the bond.
		for(i = 0;i < num_bonds;i++){
			// Traverse the bond list
			a = mol->bonds[i].a;
			b = mol->bonds[i].b;
			
			// Don't render solvent bonds into the selection buffer
			// if the user has turned off solvent display.
			if(!mol->solvent && (mol->atoms[a].isSolvent && mol->atoms[b].isSolvent))
				continue;

			mol->atoms[a].GetCoord(r);
			mol->atoms[b].GetCoord(tmp);

			r[X] = 0.5f*(r[X] + tmp[X]);
			r[Y] = 0.5f*(r[Y] + tmp[Y]);
			r[Z] = 0.5f*(r[Z] + tmp[Z]);

			glLoadName(i + num_atoms + offset);
			glBegin(GL_POINTS);
			glVertex3fv(r);
			glEnd();
		}
	} else{
		//int index = (mol->current_step - 1)*3*mol->atoms.Length();
		int index = mol->current_step - 1;
		
		// Render atoms into the selection buffer
		if(mol->solvent){
			for(i = 0;i < num_atoms;i++) {
				// Note that in order for the selection process to work,
				// glLoadName must be called BEFORE glBegin(GL_POINTS)!!!
				glLoadName(i + offset);
				glBegin(GL_POINTS);
				glVertex3fv(&(mol->traj[index][3*i]));
				glEnd();
			}
		}
		else{ // !mol->solvent
			for(i = 0;i < num_atoms;i++) {
				// Don't render solvent atoms into the selection buffer
				if(!mol->atoms[i].isSolvent){
					// Note that in order for the selection process to work,
					// glLoadName must be called BEFORE glBegin(GL_POINTS)!!!
					glLoadName(i + offset);
					glBegin(GL_POINTS);
					glVertex3fv(&(mol->traj[index][3*i]));
					glEnd();
				}
			}
		}

		// Render bonds in the selection buffer (as points placed at the
		// midpoint of the bond.
		for(i = 0;i < num_bonds;i++){
			// Traverse the bond list
			a = mol->bonds[i].a;
			b = mol->bonds[i].b;
			
			// Don't render solvent bonds into the selection buffer
			// if the user has turned off solvent display.
			if(!mol->solvent && (mol->atoms[a].isSolvent && mol->atoms[b].isSolvent))
				continue;

			memcpy(r, &(mol->traj[index][3*a]), 3*sizeof(float));
			memcpy(tmp, &(mol->traj[index][3*b]), 3*sizeof(float));

			r[X] = 0.5f*(r[X] + tmp[X]);
			r[Y] = 0.5f*(r[Y] + tmp[Y]);
			r[Z] = 0.5f*(r[Z] + tmp[Z]);

			glLoadName(i + num_atoms + offset);
			glBegin(GL_POINTS);
			glVertex3fv(r);
			glEnd();
		}
	}

	// Increment the selection index
	offset += num_atoms + num_bonds;

	glPopMatrix();
}

void CQmolView::renderMolecule(molecule *mol, int mode, int &offset)
{	
	GLfloat m[4][4];

	glPushMatrix();
  	
	mol->BuildRotationMatrix(m);

	glTranslatef(mol->centerX,
    			-mol->centerY,
    			 mol->centerZ);


	// Are we currently rendering the right eye of a split
	// screen stereo view? Note that in the current scheme of
	// things, the positive y axis is "up".
	if(renderSplitStereoRightEye)
		glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

	if(renderSplitStereoLeftEye)
		glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

	glMultMatrixf(&m[0][0]);

	// Compute the inverse matrix
	InvertMatrix(m);

	if(mode == QM_RENDER_SCREEN){
  		renderAtoms(mol, m);
	}
	else if(mode == QM_RENDER_SELECT)
		renderSelect(mol, offset);
	else
		throw QMOL_ERROR("Unknown rendering mode!");

	glPopMatrix();
}

void CQmolView::renderLabels(molecule *mol)
{
	// Only sixteen characters allowed for atom labels.
	char buffer[16];
	
	glDisable(GL_LIGHTING);

	if(depthCue)
		glDisable(GL_FOG);
	
	glPushMatrix();
	
	if(mol->current_step == 0){
  		for(int i = 0;i < mol->atoms.Length();i++){
			// Create the text label
			sprintf(buffer,"%d",mol->atoms[i].id);
		
			// Color the atom labels the same color as the atom
			// (ignore transparancy).
			glColor3ubv(mol->atoms[i].rgb_color);

			glRasterPos3fv(mol->atoms[i].coord);
  			glCallLists(strlen(buffer), GL_UNSIGNED_BYTE, buffer);
  		}
	} else{
		//int index = (mol->current_step - 1)*3*mol->atoms.Length();
		int index = mol->current_step - 1;

		for(int i = 0;i < mol->atoms.Length();i++){	
			// Create the text label
			sprintf(buffer,"%d",mol->atoms[i].id);
		
			// Color the atom labels the same color as the atom
			glColor3ubv(mol->atoms[i].rgb_color);
		
			glRasterPos3fv(&(mol->traj[index][3*i]));
  			glCallLists(strlen(buffer), GL_UNSIGNED_BYTE, buffer);
  		}
	}
	
	glPopMatrix();
	
	if(depthCue)
		glEnable(GL_FOG);

	glEnable(GL_LIGHTING);
}

void CQmolView::renderAxes(float scale_x, float scale_y, float scale_z, GLfloat m[][4])
{
	ls_wire_frame.Disable();
	lb_solid.Enable();

	glDisable(GL_CULL_FACE);
	glEnable(GL_DITHER);
	
	if(smooth3D)
		glShadeModel(GL_SMOOTH);
	
	// Don't clip the axes!
	if(clipView)
		clipper.Disable();

    glPushMatrix();

	glColor3f(1.0f - background[0],
			  1.0f - background[1],
			  1.0f - background[2]);

	// Draw a sphere at the base of the axes to hide the ends of the
	// cylinders
	glPushMatrix();
		
	glTranslatef(0.0f, 0.0f, 0.0f);

	glScalef(0.5f*MONITOR_RADIUS, 0.5f*MONITOR_RADIUS, 0.5f*MONITOR_RADIUS);

	// **** "Undo" any perspective rotation ****
	glMultMatrixf(&m[0][0]);

	// **** "Undo" any stero rotation ****
	// Are we currently rendering the right eye of a split
	// screen stereo view? Note that in the current scheme of
	// things, the positive y axis is "up".
	if(renderSplitStereoRightEye)
		glRotatef(-stereoOffset, 0.0f, 1.0f, 0.0f);

	if(renderSplitStereoLeftEye)
		glRotatef(stereoOffset, 0.0f, 1.0f, 0.0f);

	glCallList(SELECTION_SPHERE);

	glPopMatrix();

	// The X Axis cylinder
	glPushMatrix();
		
	glTranslatef(0.0f, 0.0f, 0.0f);

	glRotatef(90.0f, 0.0f, 1.0f, 0.0f);

	glScalef(0.5f*MONITOR_RADIUS, 0.5f*MONITOR_RADIUS, 0.75f*scale_x/CYLINDER_HEIGHT);

	glCallList(TORSION_CYL);

	glPopMatrix();

	// The X Axis cone
	glPushMatrix();
		
	glTranslatef(0.0f, 0.0f, 0.0f);

	glRotatef(90.0f, 0.0f, 1.0f, 0.0f);

	glTranslatef(0.0f, 0.0f, 0.75f*scale_x);

	glScalef(MONITOR_RADIUS, MONITOR_RADIUS, 0.25f*scale_x/CYLINDER_HEIGHT);

	glCallList(TORSION_DISK);
	glCallList(AXIS_CONE);

	glPopMatrix();

	// The Y Axis
	glPushMatrix();
		
	glTranslatef(0.0f, 0.0f, 0.0f);

	glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);

	glScalef(0.5f*MONITOR_RADIUS, 0.5f*MONITOR_RADIUS, 0.75f*scale_y/CYLINDER_HEIGHT);

	glCallList(TORSION_CYL);

	glPopMatrix();

	// The Y Axis cone
	glPushMatrix();
		
	glTranslatef(0.0f, 0.0f, 0.0f);

	glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);

	glTranslatef(0.0f, 0.0f, 0.75f*scale_y);

	glScalef(MONITOR_RADIUS, MONITOR_RADIUS, 0.25f*scale_y/CYLINDER_HEIGHT);

	glCallList(TORSION_DISK);
	glCallList(AXIS_CONE);

	glPopMatrix();

	// The Z Axis
	glPushMatrix();
		
	glTranslatef(0.0f, 0.0f, 0.0f);

	glScalef(0.5f*MONITOR_RADIUS, 0.5f*MONITOR_RADIUS, 0.75f*scale_z/CYLINDER_HEIGHT);

	glCallList(TORSION_CYL);

	glPopMatrix();

	// The Z Axis cone
	glPushMatrix();

	glTranslatef(0.0f, 0.0f, 0.75f*scale_z);

	glScalef(MONITOR_RADIUS, MONITOR_RADIUS, 0.25f*scale_z/CYLINDER_HEIGHT);

	glCallList(TORSION_DISK);
	glCallList(AXIS_CONE);

	glPopMatrix();

	// Add labels to the axes
	lb_solid.Disable();
	ls_wire_frame.Enable();

	glRasterPos3f(scale_x, 0.0f, 0.0f);
  	glCallLists(1, GL_UNSIGNED_BYTE, "X");
	
	glRasterPos3f(0.0f, scale_y, 0.0f);
  	glCallLists(1, GL_UNSIGNED_BYTE, "Y");

	glRasterPos3f(0.0f, 0.0f, scale_z);
  	glCallLists(1, GL_UNSIGNED_BYTE, "Z");

	glPopMatrix();

	if(clipView)
		clipper.Enable();

	glDisable(GL_DITHER);
	glShadeModel(GL_FLAT);
	glEnable(GL_CULL_FACE);
}

BOOL CQmolView::RenderScene(list<molecule *> &m_molList, int mode)
{
	#ifdef _DEBUG
	GetOpenGLError(__FILE__, __LINE__);
	#endif // _DEBUG

	// Prepare to iterate through the list of molecules
	ListIterator<molecule *> next(m_molList);
	molecule **mol;

	int selectionOffset = 0;

	switch(displayStereo){
		case QM_STEREO_NONE:
			// Select the full client area
			glViewport(0, 0, viewWidth, viewHeight);
				
			if(mode == QM_RENDER_SCREEN)
				glClear(GL_DEPTH_BUFFER_BIT	| GL_COLOR_BUFFER_BIT);

			// Perform clipping (with NO stereo correction)
			clipper.Clip(0.0f);
			
			while(mol = next()){
				if((*mol)->NumAtoms()){
					renderMolecule((*mol), mode, selectionOffset);
				}
			}
			
			if(mode != QM_RENDER_SELECT){
				// Render any measurement indicators
				renderMeasure();

				// Render any hydrogen bonds
				if(showHBonds)
					renderHBonds();

				// Render the clipping plane (if enabled)
				clipper.RenderClipPlane();

				#ifdef COMPUTE_FPS
				if(mode == QM_RENDER_SCREEN)
					renderFPS();
				#endif // COMPUTE_FPS
			}

			#ifdef _DEBUG
			GetOpenGLError(__FILE__, __LINE__);
			#endif // _DEBUG
			break;
		case QM_STEREO_SPLIT_SCREEN:
			// Select the left client area
			glViewport(0, 0, viewWidth/2, viewHeight);
				
			if(mode == QM_RENDER_SCREEN)
				glClear(GL_DEPTH_BUFFER_BIT	| GL_COLOR_BUFFER_BIT);

			renderSplitStereoLeftEye = TRUE;

			// Perform clipping (with left eye stereo correction)
			clipper.Clip(-stereoOffset);

			while(mol = next()){
				if((*mol)->NumAtoms()){
					renderMolecule((*mol), mode, selectionOffset);
				}
			}

			if(mode != QM_RENDER_SELECT){
				renderMeasure();

				// Render any hydrogen bonds
				if(showHBonds)
					renderHBonds();

				// Clip the view (if enabled)
				clipper.RenderClipPlane();
			}

			renderSplitStereoLeftEye = FALSE;

			// Select the right client area
			glViewport(viewWidth/2, 0, viewWidth/2, viewHeight);
				
			renderSplitStereoRightEye = TRUE;

			selectionOffset = 0;

			// Perform clipping (with right eye stereo correction)
			clipper.Clip(stereoOffset);
			
			// Reset the list iterator
			next.Init(m_molList);
			while(mol = next()){
				if((*mol)->NumAtoms()){
					renderMolecule((*mol), mode, selectionOffset);
				}
			}

			if(mode != QM_RENDER_SELECT){
				renderMeasure();

				// Render any hydrogen bonds
				if(showHBonds)
					renderHBonds();

				// Clip the view (if enabled)
				clipper.RenderClipPlane();
			}

			renderSplitStereoRightEye = FALSE;

			if(mode != QM_RENDER_SELECT){
				#ifdef COMPUTE_FPS
				if(mode == QM_RENDER_SCREEN)
					renderFPS();
				#endif // COMPUTE_FPS
			}

			#ifdef _DEBUG
			GetOpenGLError(__FILE__, __LINE__);
			#endif // _DEBUG
			break;
	};

    return TRUE;
}

void CQmolView::postRedisplay()
{
	InvalidateRect(0, FALSE);
	GetParent()->PostMessage(WM_PAINT);
}

void CQmolView::ActiveAdjustTorsionScale()
{
	if(m_pATDlg->GetSafeHwnd()){

		// Since we are either (a) selecting a new torsion
		// angle to modify, or (b) deselecting a torsion angle,
		// set the slider position to the center.
		//m_pATDlg->m_slider.SetPos(360);
		m_pATDlg->Set(0.0);

		// Should the scale be sensitive?
		if(tor_adjust.IsSelected()){
			m_pATDlg->m_slider.EnableWindow(TRUE);
		}
		else{
			// Disable the slider
			m_pATDlg->m_slider.EnableWindow(FALSE);
		}
	}	
}

void CQmolView::startRotation(int x, int y)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	referenceFrame.StartRotation(x, y);

	while(tmpMol = next())
		(*tmpMol)->StartRotation(x, y);

  	stopSpinning();
}

void CQmolView::stopSpinning()
{
 	if(spinning){
    	spinning = FALSE;
			
		if(!animateTraj)
    		animate = FALSE;
  	}
}

// Add the various mouse handling routines here
void CQmolView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// Allow the user to simulate a middle button by chording
	// left an right mouse buttons
	if(nFlags & MK_RBUTTON){
		// The right button has already been pressed
		StartMove(point.x, point.y);
		CView::OnLButtonDown(nFlags, point);
		return;
	}

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Grab control of the mouse until the user
	// releases the left mouse button.
	SetCapture();

	// Remember that the left button is down
	l_button_down = TRUE;

	SelectObj picked = pickScene(pDoc->molList, point.x, point.y);

	// Did the user pick an atom?
	if(picked.isAtom()){

		// Update the status bar (if present)
		UpdateStatusBar(picked);

		// Only allow atom selection for measurement if the
		// measure dialog is currently open (this will reduce GUI
		// clutter).
		if(m_pMDlg->GetSafeHwnd() && pDoc->molMetric.Push(picked)){

			// Update the measure dialog box 
			m_pMDlg->UpdateMeasurements();

			// Redraw the molecule if new measurements have been made.
			postRedisplay();
		}
	}
	// Did they pick a bond? Note that the adjust torsion angle dialog
	// box must be currently displayed for the user to select torsion angles.
	else if(m_pATDlg->GetSafeHwnd() && picked.isBond()){
		// Find the atoms whose positions will change when the 
		// torsion angle is varried.
		tor_adjust.Set(picked);
		
		// Either enable or disable the torsion angle slider
		// on the adjust torsion dialog box
		ActiveAdjustTorsionScale();

		postRedisplay();
	}
	
	// Now handle rotations of the molecule and clipping plane
	if(nFlags & MK_CONTROL){
		clipper.StartRotate(point.x, point.y);
	}
	else{
		startRotation(point.x, point.y);
	}

	CView::OnLButtonDown(nFlags, point);
}

void CQmolView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// The left button is no longer down
	l_button_down = FALSE;
	
	// Correct molecule rotation
	if(!referenceFrame.HasRotated() && !(nFlags & MK_CONTROL))
		rotation(point.x, point.y);

	// Give up control of the mouse
	ReleaseCapture();
	
	if(momentum){
      	spinning = TRUE;

		if(!animateTraj){
      		animate = TRUE;
			postRedisplay();
		}
	}

	CView::OnLButtonUp(nFlags, point);
}

void CQmolView::OnMButtonDown(UINT nFlags, CPoint point)
{
	// Grab control of the mouse
	SetCapture();
	
	StartMove(point.x, point.y);
}

void CQmolView::OnMButtonUp(UINT nFlags, CPoint point)
{
	// Grab control of the mouse
	ReleaseCapture();
}

void CQmolView::OnRButtonDown(UINT nFlags, CPoint point)
{
	// Allow the user to simulate a middle button by chording
	// left an right mouse buttons
	if(nFlags & MK_LBUTTON){
		// The left button has already been pressed
		StartMove(point.x, point.y);
		CView::OnRButtonDown(nFlags, point);
		return;
	}

	// Grab control of the mouse
	SetCapture();

	if(nFlags & MK_CONTROL){
		// Start to scale the clip plane
		clipper.StartZoom(point.x);
	}
	else{
		// Start to scale the molecule
		StartScale(point.x);
	}

	CView::OnRButtonDown(nFlags, point);
}

void CQmolView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// Give up mouse control
	ReleaseCapture();
	
	CView::OnRButtonUp(nFlags, point);
}

void CQmolView::OnMouseMove(UINT nFlags, CPoint point)
{
	if((nFlags & MK_LBUTTON) && !(nFlags & MK_RBUTTON)){

		if(nFlags & MK_CONTROL){
			// Rotate the clipping plane
			clipper.Rotate(point.x, point.y, viewWidth, viewHeight);
			postRedisplay();
		}
		else{
			// Rotate the molecule
			rotation(point.x, point.y);
		}

		CView::OnMouseMove(nFlags, point);
		return;
	}
	
	if((nFlags & MK_RBUTTON) && !(nFlags & MK_LBUTTON)){
		if(nFlags & MK_CONTROL){
			// Scale the clip plane
			clipper.Zoom(point.x, viewWidth);
			postRedisplay();
		}
		else{
			// Scale the molecule
			Scale(point.x);
		}

		CView::OnMouseMove(nFlags, point);
		return;
	}

	if((nFlags & MK_MBUTTON) || ((nFlags & MK_RBUTTON) && 
		(nFlags & MK_LBUTTON))){
		Move(point.x, point.y);
		CView::OnMouseMove(nFlags, point);
		return;
	}
}

void CQmolView::rotation(int x, int y)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	referenceFrame.Rotation(x, y, viewWidth, viewHeight);

	while(tmpMol = next())
		(*tmpMol)->Rotation(x, y, viewWidth, viewHeight);

	if(showHBonds && !(pDoc->AllDynamic()))
		pDoc->UpdateHydrogenBonding();

	if(m_atomSelectDlg->GetSafeHwnd() != 0){
		// Update the rmsd calculation
		m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_RMSD_UPDATE);
	}

	// Don't spin with mometum while the user is rotating the 
	// molecule
  	if(momentum && !l_button_down){
      	spinning = TRUE;

		if(!animateTraj){
      		animate = TRUE;
			postRedisplay();
		}

  	}else{
		referenceFrame.Update();

		next.Init(pDoc->molList);
		while(tmpMol = next())
			(*tmpMol)->Update();

    	postRedisplay();
  	}
}

void CQmolView::StartScale(int x)
{

  	beginScaleX = x;

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if(pDoc->AllDynamic())
		return;

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;
	
	referenceFrame.StartScale(x);

	while(tmpMol = next())
		(*tmpMol)->StartScale(x);
}

void CQmolView::Scale(int x)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->AllDynamic()){
		perspective -= (40.0f*(x - beginScaleX))/viewWidth;
		
		beginScaleX = x;
		
		if(perspective > 180.0f)
			perspective = 180.0f;
		
		if(perspective < 0.0f)
			perspective = 0.0f;

		SetupGLView(viewWidth, viewHeight);
	}
	else{

		ListIterator<molecule*> next(pDoc->molList);
		molecule **tmpMol;
		
		float moveFrac = maxDim/MAX(viewWidth, viewHeight);

		referenceFrame.Scale(x, moveFrac);

		while(tmpMol = next())
			(*tmpMol)->Scale(x, moveFrac);


		// Adjust the maximum dimension to account for shifting molecules
		maxDim = pDoc->GetMaxDim(scaleToFit);

		if(showHBonds)
			pDoc->UpdateHydrogenBonding();

		if(m_atomSelectDlg->GetSafeHwnd() != 0){
			// Update the rmsd calculation
			m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_RMSD_UPDATE);
		}
	}

	postRedisplay();
}

void CQmolView::ScaleWheel(int num_rot)
{
	// The variable num_rot is the number of clicks (positive or
	// negative) that the user has rotated with the wheel mouse).
	// The prefactor to num_rot (i.e. 5 in this case) controlls
	// the sensitivity of the wheel based scrolling).
	perspective += 5*num_rot;
	
	if(perspective > 180.0f)
		perspective = 180.0f;
	
	if(perspective < 0.0f)
		perspective = 0.0f;
	
	//renderReshape(viewWidth, viewHeight);
	SetupGLView(viewWidth, viewHeight);

	postRedisplay();
}

void CQmolView::StartMove(int x, int y)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;
	
	referenceFrame.StartMove(x, y);

	while(tmpMol = next())
		(*tmpMol)->StartMove(x, y);
	
}

void CQmolView::Move(int x, int y)
{	
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;
	
	float moveFrac = maxDim/MAX(viewWidth, viewHeight);

	referenceFrame.Move(x, y, moveFrac);

	while(tmpMol = next())
		(*tmpMol)->Move(x, y, moveFrac);

	// Adjust the maximum dimension to account for shifting molecules
	maxDim = pDoc->GetMaxDim(scaleToFit);

	if(showHBonds && !(pDoc->AllDynamic()))
		pDoc->UpdateHydrogenBonding();

	if(m_atomSelectDlg->GetSafeHwnd() != 0){
		// Update the rmsd calculation
		m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_RMSD_UPDATE);
	}

	postRedisplay();
}

// Open the Adjust Torsion dialog
void CQmolView::OnAdjustTorsionangle() 
{
	// Create the Adjust Torsion Dialog
	if(m_pATDlg->GetSafeHwnd() == 0){
		m_pATDlg->Create();

		// Redraw the scene to show the currently selected torsion angle
		postRedisplay();
	}
}

// The slider has been adjusted
LRESULT CQmolView::OnAdjustTorsionangleModify(WPARAM wParam, LPARAM lParam)
{
	if(m_pATDlg->GetSafeHwnd()){

		CQmolDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		ASSERT(pDoc->IsValidMol(tor_adjust.GetMol()));

		tor_adjust.Update((float)(m_pATDlg->m_slider.GetPos() - 180));
		
		// Update the hydrogen bonds
		if(showHBonds)
			pDoc->UpdateHydrogenBonding();

		if(m_atomSelectDlg->GetSafeHwnd() != 0){
			// Update the rmsd calculation
			m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_RMSD_UPDATE);
		}

		postRedisplay();
	}

	return 0L;
}

// Close the Adjust Torsion dialog
LRESULT CQmolView::OnAdjustTorsionangleClose(WPARAM wParam, LPARAM lParam) 
{
	m_pATDlg->DestroyWindow();

	// Redraw the display to hide the currently selected torsion angle (if any).
	postRedisplay();

	return 0L;
}

void CQmolView::OnUpdateOptionsMeasure(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);
}

void CQmolView::OnOptionsMeasure() 
{
	if(m_pMDlg->GetSafeHwnd() == 0){
		m_pMDlg->Create();

		// Redraw the scene to show any current measurements
		postRedisplay();
	}
}

LRESULT CQmolView::OnOptionsMeasureClose(WPARAM wParam, LPARAM lParam) 
{
	m_pMDlg->DestroyWindow();
	
	// Redraw the display to hide any measurements that the user has made
	postRedisplay();

	return 0L;
}

LRESULT CQmolView::OnOptionsColorClose(WPARAM wParam, LPARAM lParam) 
{
	m_qmColorDlg->DestroyWindow();

	return 0L;
}

LRESULT CQmolView::OnFlexColorMsg(WPARAM wParam, LPARAM lParam) 
{
	int r, g, b, a;

	switch(wParam){
		case IDOK:			
			OnFlexColorMsg(FLEX_COLOR_APPLY, lParam);
			m_flexClrDlg->DestroyWindow();
			break;
		case IDCANCEL:
			m_flexClrDlg->DestroyWindow();
			break;
		case FLEX_COLOR_APPLY:
			m_flexClrDlg->GetRGBA(r, g, b, a);
			
			switch(lParam){
				case BACKGROUND_CLR:
					background[0] = r/255.0f;
					background[1] = g/255.0f;
					background[2] = b/255.0f;

					glClearColor(background[0],			
								 background[1],	
								 background[2],	
								 1.0);
					
					// Set the fog color
					glFogfv(GL_FOG_COLOR, background);
					break;
				case SELECTION_CLR:
					selection_color[0] = r/255.0f;
					selection_color[1] = g/255.0f;
					selection_color[2] = b/255.0f;
					selection_color[3] = a/255.0f;
					break;
				case HBOND_CLR:
					hbond_color[0] = r/255.0f;
					hbond_color[1] = g/255.0f;
					hbond_color[2] = b/255.0f;
					break;
				case CLIP_PLANE_CLR:
					clipPlaneClr[0] = r/255.0f;
					clipPlaneClr[1] = g/255.0f;
					clipPlaneClr[2] = b/255.0f;
					clipPlaneClr[3] = a/255.0f;

					clipper.SetColor(clipPlaneClr);
					break;
			};

			postRedisplay();
	};

	return 0L;
}

void CQmolView::OnOptionsCenter() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	while(tmpMol = next())
		(*tmpMol)->Translate(0.0f, 0.0f, 0.0f);

	// Redraw the molecule
	postRedisplay();
}

/////////////////////////////////////////////////////////////////////////////
// GenerateBitmapListForFont
// The routines used for generating flat text
GLuint CQmolView::GenerateBitmapListForFont(LOGFONT &font)
{
	GLuint id;

	if ( 0 == m_pDC ||
		 (GLuint)0 == (id = GetNewDisplayListIDs(256)) )
		{
        return 0;
		}

	CFont newfont;
	CFont* oldfont;
	BOOL success;

	int old_size = font.lfHeight;

	// Scale the font as needed
	font.lfHeight = -MulDiv(-font.lfHeight, printScaleMul, printScaleDiv);

	// returns 0 if it fails
	success = newfont.CreateFontIndirect(&font);
	oldfont = m_pDC->SelectObject(&newfont);
	ASSERT(oldfont != 0);

	// Return the input font to its origianl state
	font.lfHeight = old_size;

    // Create a set of display lists based on the glyphs of the font 
	// notice that we really waste the first 32 spaces....
	// if there's a problem delete the display lists
   if (	0 == success ||
		FALSE == ::wglUseFontBitmaps( m_pDC->m_hDC, 0, 256, id ) )
		{
		::glDeleteLists( id, 256 );
		id  = 0;
		}
   else
		{
		m_pDC->SelectObject( oldfont );
		}

   return id;
}

/////////////////////////////////////////////////////////////////////////////
// GetNewDisplayListIDs
// This is just a nice wrapper for the glGenLists() function.
GLuint CQmolView::GetNewDisplayListIDs(GLsizei range)
{
	ASSERT( 0 < range );

	return ::glGenLists( range );
}

void CQmolView::OnRenderAtomLabels() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->atom_labels = !(*tmpMol)->atom_labels;

	postRedisplay();
}

void CQmolView::OnUpdateRenderAtomLabels(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->atom_labels);
}

void CQmolView::OnOptionsFont() 
{	
	CFontDialog fDlg(&font);
	
	PauseAnimation(TRUE);

	// Let the user choose a new font on screen rendering
	if(fDlg.DoModal()){
		// Clean up the old list
		glDeleteLists( font_list_id, 256 );

		// Make the new list
		glListBase(font_list_id = GenerateBitmapListForFont(font));
	}

	PauseAnimation(FALSE);
}

// Set the background color
void CQmolView::OnOptionsColor() 
{
	if(m_flexClrDlg->GetSafeHwnd() == 0){
		// Make sure that the dialog displays the current
		// background color.
		m_flexClrDlg->SetRGB(
			(int)(background[0]*255.0f),
			(int)(background[1]*255.0f),
			(int)(background[2]*255.0f));

		m_flexClrDlg->Create(BACKGROUND_CLR);
	}
}

void CQmolView::OnRenderSolvent() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->solvent = !(*tmpMol)->solvent;

	postRedisplay();
}

void CQmolView::OnUpdateRenderSolvent(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->solvent);
}

void CQmolView::OnOptionsMomentum() 
{
	momentum = !momentum;

	if(!momentum && spinning){
		// If we were spinning, stop.
		spinning = FALSE;

		if(!animateTraj)
			// If we're not animating a trajectory, stop
			// the animation loop.
      		animate = FALSE;
	}
}

void CQmolView::OnUpdateOptionsMomentum(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(momentum);
}

void CQmolView::OnOptionsAnimate() 
{
	if(m_pAnimateDlg->GetSafeHwnd() == 0){
		m_pAnimateDlg->Create();
	}
}

LRESULT CQmolView::OnAnimateClose(WPARAM wParam, LPARAM lParam)
{
	m_pAnimateDlg->DestroyWindow();

	return 0L;
}

LRESULT CQmolView::OnAnimateAdjust(WPARAM wParam, LPARAM lParam)
{
	if(m_pAnimateDlg->GetSafeHwnd()){

		CQmolDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		// Stop animating the trajectory
		animateTraj = FALSE;

		// If the molecule is not currently spinning,
		// stop pumping the message cue with WM_PAINT messages.
		if(!spinning)
			animate = FALSE;

		pDoc->CurrentTrajStep(m_pAnimateDlg->m_slider.GetPos());
		
		postRedisplay();
	}

	return 0L;
}

LRESULT CQmolView::OnAnimateForward(WPARAM wParam, LPARAM lParam)
{
	// Start animating the trajectory
	animateTraj = TRUE;
	animate = TRUE;

	animateDirection = QM_FORWARD;
	postRedisplay();

	return 0L;
}

LRESULT CQmolView::OnAnimateStop(WPARAM wParam, LPARAM lParam)
{
	// Stop animating the trajectory
	animateTraj = FALSE;
	
	// If the molecule is not currently spinning,
	// stop pumping the message cue with WM_PAINT messages.
	if(!spinning)
		animate = FALSE;

	return 0L;
}

LRESULT CQmolView::OnAnimateReverse(WPARAM wParam, LPARAM lParam)
{
	// Start animating the trajectory
	animateTraj = TRUE;
	animate = TRUE;

	animateDirection = QM_REVERSE;
	postRedisplay();

	return 0L;
}

void CQmolView::OnRenderDepthCue() 
{
	if(depthCue){
		depthCue = FALSE;
		glDisable(GL_FOG);
		postRedisplay();
	}
	else{
		depthCue = TRUE;
		glEnable(GL_FOG);
		postRedisplay();
	}
	
}

void CQmolView::OnUpdateRenderDepthCue(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(depthCue);
}

void CQmolView::OnRenderCpk() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_CPK;
	
	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderCpk(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_CPK));
}

void CQmolView::OnRenderStick() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_STICK;
	
	// To differentiate from ball and stick
	// set the ball radius equal to the stick radius
	stickRadius = STICK_RADIUS;
	ballRadius = STICK_RADIUS;

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderStick(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_STICK));
}

void CQmolView::OnRenderBallAndStick() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_BALL_AND_STICK;

	
	// To differentiate from ball and stick
	// set the ball radius equal to the stick radius
	stickRadius = STICK_RADIUS;
	ballRadius = BALL_RADIUS;

	// Redraw the molecule
	postRedisplay();
	
}

void CQmolView::OnUpdateRenderBallAndStick(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_BALL_AND_STICK));
}

void CQmolView::OnRenderWireFrame() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_WIREFRAME;
	
	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderWireFrame(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_WIREFRAME));
}

void CQmolView::OnRenderPoint() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_POINT;
	
	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderPoint(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_POINT));
}

void CQmolView::OnRenderTraceStick() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_TRACE_STICK;
	
	// Set the radius of the trace stick (use the slightly larger
	// BALL_RADIUS to make nice, fat sticks).
	stickRadius = BALL_RADIUS;
	ballRadius = BALL_RADIUS;

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderTraceStick(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_TRACE_STICK));
}

void CQmolView::OnRenderTraceWireframe() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_TRACE_WIREFRAME;

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderTraceWireframe(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_TRACE_WIREFRAME));
}

void CQmolView::OnRenderTraceSpline() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_TRACE_SPLINE;

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderTraceSpline(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_TRACE_SPLINE));
}

void CQmolView::OnRenderRibbon() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			(*tmpMol)->displayType = QM_RENDER_RIBBON;
			(*tmpMol)->SetRibbonSize(QM_RENDER_RIBBON);
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderRibbon(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_RIBBON));
}

void CQmolView::OnRenderRibbonFlat() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			(*tmpMol)->displayType = QM_RENDER_RIBBON_FLAT;
			(*tmpMol)->SetRibbonSize(QM_RENDER_RIBBON_FLAT);
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderRibbonFlat(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_RIBBON_FLAT));
}

void CQmolView::OnRenderTube() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			(*tmpMol)->displayType = QM_RENDER_TUBE;
			(*tmpMol)->SetRibbonSize(QM_RENDER_TUBE);
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderTube(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_TUBE));
}

void CQmolView::OnRenderTubeOcc() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			(*tmpMol)->displayType = QM_RENDER_TUBE_OCC;
			(*tmpMol)->SetRibbonSize(QM_RENDER_TUBE_OCC);
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderTubeOcc(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_TUBE_OCC));
}

void CQmolView::OnRenderTubeT() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			(*tmpMol)->displayType = QM_RENDER_TUBE_T;
			(*tmpMol)->SetRibbonSize(QM_RENDER_TUBE_T);
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderTubeT(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_TUBE_T));
}

void CQmolView::OnRenderNone() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->displayType = QM_RENDER_NONE;

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderNone(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && (ptrMol->displayType == QM_RENDER_NONE));
}

void CQmolView::OnRenderSurface() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			if((*tmpMol)->surf.ValidSurface()){
				// Allow the user to switch from mesh to solid without
				// recomputing the surface
				if((*tmpMol)->surf.GetRenderStyle() == MolSurface::SOLID)
					(*tmpMol)->surf.InvalidateSurface();
				else{
					(*tmpMol)->surf.SetRenderStyle(MolSurface::SOLID);
				}
			}
			else{
				
				try{
					// Set the hourglass cursor - this may take a while
					::SetCursor(LoadCursor(NULL, IDC_WAIT));
					
					// Set the rendering style
					(*tmpMol)->surf.SetRenderStyle(MolSurface::SOLID);
					
					(*tmpMol)->surf.BuildSurface(&((*tmpMol)->atoms), 
						&((*tmpMol)->traj), (*tmpMol)->current_step);
				}
				catch(QmolError &e){
					// Restore the cursor
					::SetCursor(LoadCursor(NULL, IDC_ARROW));
					
					AfxMessageBox(e.message());
					
					// Clean up the progress bar
					CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
					if(pFrame){
						pFrame->CloseProgress();
					}
					
				}
				
				// Restore the cursor
				::SetCursor(LoadCursor(NULL, IDC_ARROW));
			}
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderSurface(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->surf.ValidSurface() && 
		(ptrMol->surf.GetRenderStyle() == MolSurface::SOLID));
}

void CQmolView::OnRenderMesh() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			if((*tmpMol)->surf.ValidSurface()){
				// Allow the user to switch from mesh to solid without
				// recomputing the surface
				if((*tmpMol)->surf.GetRenderStyle() == MolSurface::MESH)
					(*tmpMol)->surf.InvalidateSurface();
				else{
					(*tmpMol)->surf.SetRenderStyle(MolSurface::MESH);
				}
			}
			else{
				
				try{
					// Set the hourglass cursor - this may take a while
					::SetCursor(LoadCursor(NULL, IDC_WAIT));
					
					// Set the rendering style
					(*tmpMol)->surf.SetRenderStyle(MolSurface::MESH);
					
					(*tmpMol)->surf.BuildSurface(&((*tmpMol)->atoms), 
						&((*tmpMol)->traj), (*tmpMol)->current_step);
				}
				catch(QmolError &e){
					// Restore the cursor
					::SetCursor(LoadCursor(NULL, IDC_ARROW));
					
					AfxMessageBox(e.message());
					
					// Clean up the progress bar
					CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
					if(pFrame){
						pFrame->CloseProgress();
					}
					
				}
				
				// Restore the cursor
				::SetCursor(LoadCursor(NULL, IDC_ARROW));
			}
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderMesh(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->surf.ValidSurface() && 
		(ptrMol->surf.GetRenderStyle() == MolSurface::MESH));
}

void CQmolView::OnRenderSurfacePoint() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected){
			if((*tmpMol)->surf.ValidSurface()){
				// Allow the user to switch from mesh to solid without
				// recomputing the surface
				if((*tmpMol)->surf.GetRenderStyle() == MolSurface::POINT)
					(*tmpMol)->surf.InvalidateSurface();
				else{
					(*tmpMol)->surf.SetRenderStyle(MolSurface::POINT);
				}
			}
			else{
				
				try{
					// Set the hourglass cursor - this may take a while
					::SetCursor(LoadCursor(NULL, IDC_WAIT));
					
					// Set the rendering style
					(*tmpMol)->surf.SetRenderStyle(MolSurface::POINT);
					
					(*tmpMol)->surf.BuildSurface(&((*tmpMol)->atoms), 
						&((*tmpMol)->traj), (*tmpMol)->current_step);
				}
				catch(QmolError &e){
					// Restore the cursor
					::SetCursor(LoadCursor(NULL, IDC_ARROW));
					
					AfxMessageBox(e.message());
					
					// Clean up the progress bar
					CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
					if(pFrame){
						pFrame->CloseProgress();
					}
					
				}
				
				// Restore the cursor
				::SetCursor(LoadCursor(NULL, IDC_ARROW));
			}
		}

	// Redraw the molecule
	postRedisplay();
}

void CQmolView::OnUpdateRenderSurfacePoint(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->surf.ValidSurface() && 
		(ptrMol->surf.GetRenderStyle() == MolSurface::POINT));
}

void CQmolView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	int nRasterCaps	= pDC->GetDeviceCaps(RASTERCAPS);
	BOOL m_bStretchDIBits, m_bStretchBlt;

	// Does the device support raster operations?
	if (nRasterCaps & RC_STRETCHDIB)
		m_bStretchDIBits = TRUE;
	else 
		m_bStretchDIBits = FALSE;

	if (nRasterCaps & RC_STRETCHBLT)
		m_bStretchBlt = TRUE;
	else 
		m_bStretchBlt = FALSE;


	if (!m_bStretchDIBits && !m_bStretchBlt)
	{
		CString mMsg;

		if(pInfo != NULL){
			LPDEVMODE* pMode = (LPDEVMODE*) pInfo->m_pPD->m_pd.hDevMode;
			DEVMODE* mode = *pMode;
			TCHAR* pChar = (TCHAR*)&mode->dmDeviceName[0];

			mMsg.Format("Printer '%s' does not support required functions.", pChar);
		}
		else{
			mMsg.Format("The clip board does not support required functions.");
		}
		
		AfxMessageBox(mMsg, MB_OK | MB_ICONERROR);
	}

	CRect	rect;			// Page rectangle
	HBRUSH	brush;			// Background brush for page
	POINT	PageSize;		// Size of printed image
	POINT	ImageOffset;	// Offset from edges for image
	CRect	mRect;			// Position of this window's client area
	int		nXRes, nYRes;
	int		tmp_width, tmp_height;
	HDC		hDC, hMemDC;
	HGLRC	hRC, hMemRC;

	BITMAPINFO bitmapInfo;
	HBITMAP hDib;
	LPVOID pBitmapBits;

	// Set our device context mapping mode.
	// Each logical unit is converted to 1 device pixel. Positive x is to
	// the right; positive y is down.
	pDC->SetMapMode(MM_TEXT);

	// Set the font scaling
	printScaleMul = GetDeviceCaps(pDC->GetSafeHdc(), LOGPIXELSY);

	// Get the size of the page (in printer device points)
	rect.top = 0;
	rect.left = 0;

	// Get the Views size (client area!)
	GetClientRect(&mRect);

	if(pInfo != NULL){
		// Get the size of the actual printed page
		rect.right = GetDeviceCaps(pDC->GetSafeHdc(), HORZRES);
		rect.bottom = GetDeviceCaps(pDC->GetSafeHdc(), VERTRES);
	}
	else{
		// pInfo is NULL so we're writing to the clip board. Set the destination
		// image size to be the same as the screen dimensions. Thanks to 
		// either (a) general win32 brain deadness or (b) my infinite
		// ability to make mistakes, we must clamp the maximum size of a bitmap.
		rect.right = min(MAX_DIB_SIZE, mRect.right);
		rect.bottom = min(MAX_DIB_SIZE, mRect.bottom);
	}

	//Determine whether the Views aspect ratio is bigger or smaller than the printers
	if (mRect.Width()*rect.Height() > mRect.Height()*rect.Width()){
		// View is proportionally wider (in x) than the Printer
		nXRes = rect.Width();
		nYRes = (nXRes*mRect.Height())/mRect.Width();
		
		// Reduce the Resolution if the Bitmap size is bigger than mem_size*1024
		// (unless mem_size < 0, in which case we use as much memory as we need to).
		while((mem_size > 0) && (3*nXRes*nYRes > mem_size*1024)){
			nXRes = nXRes/2;
			nYRes = nYRes/2;
		}
	}
	else{
		// Printer area is proportionally taller (in y) than View
		nYRes = rect.Height();
		nXRes = (nYRes*mRect.Width())/mRect.Height();

		// Reduce the Resolution if the Bitmap size is bigger than mem_size*1024
		// (unless mem_size < 0, in which case we use as much memory as we need to).
		while((mem_size > 0) && (3*nXRes*nYRes > mem_size*1024)){
			nXRes = nXRes/2;
			nYRes = nYRes/2;
		}
	}

	// First of all, initialize the bitmap header information...
	memset(&bitmapInfo, 0, sizeof(BITMAPINFO));
	bitmapInfo.bmiHeader.biSize	= sizeof(BITMAPINFOHEADER);

	if(pInfo != NULL){
		bitmapInfo.bmiHeader.biWidth = pInfo->m_bPreview ? mRect.Width() : nXRes;
		bitmapInfo.bmiHeader.biHeight = pInfo->m_bPreview ? mRect.Height() : nYRes;
	}
	else{
		// pInfo is NULL so we're writing to the clip board
		bitmapInfo.bmiHeader.biWidth = nXRes;
		bitmapInfo.bmiHeader.biHeight = nYRes;
	}

	bitmapInfo.bmiHeader.biPlanes = 1;
	bitmapInfo.bmiHeader.biBitCount = print_color_depth;
	bitmapInfo.bmiHeader.biCompression = BI_RGB;
	bitmapInfo.bmiHeader.biSizeImage = 0; // Can be set to zero for BI_RGB
 
	// Create DIB
	hDib = CreateDIBSection(m_pDC->GetSafeHdc(), &bitmapInfo, DIB_RGB_COLORS, 
		&pBitmapBits, NULL, (DWORD)0);
	
	if(hDib == NULL){
		AfxMessageBox("Unable to create DIB for storing image");
		return;
	}

	// create memory device context
	if((hMemDC = CreateCompatibleDC(m_pDC->GetSafeHdc())) == NULL){
		DeleteObject(hDib);
		return;
	}
	HGDIOBJ hOldDib = SelectObject(hMemDC, hDib);

	// Setup pixel format
	if (!SetMemDcPixelFormat(hMemDC, print_color_depth)){
		if (hOldDib != NULL)
			SelectObject(hMemDC, hOldDib);
		DeleteObject(hDib);
		DeleteDC(hMemDC);
		return;
	}

	// Create memory rendering context
	if ((hMemRC = wglCreateContext(hMemDC)) == NULL){
		if (hOldDib != NULL)
			SelectObject(hMemDC, hOldDib);

		DeleteObject(hDib);
		DeleteDC(hMemDC);
		return;
	}

	// Store current rendering and device contexts
	hDC= wglGetCurrentDC();
	hRC = wglGetCurrentContext();

	// Make this hMemRC the current OpenGL rendering context.
	wglMakeCurrent(hMemDC, hMemRC);
		
	// Set up OpenGL - this must be done for each new rendering context.
	InitializeOpenGL();

	SetupGLView(nXRes, nYRes);
	
	// Save the current viewport dimensions
	tmp_width = viewWidth;
	tmp_height = viewHeight;

	viewWidth = nXRes;
	viewHeight = nYRes;

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	RenderScene(pDoc->molList, QM_RENDER_SCREEN);
	
	// Make sure there is no rendering left to do
	glFinish();

	// Restore the original viewport dimensions
	viewWidth = tmp_width;
	viewHeight = tmp_height;

	// The rendering context will be no longer needed
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(hMemRC);

	// Restore the font scaling
	printScaleMul = printScaleDiv;

	// Restore last rendering and device contexts
	if(hDC != NULL && hRC != NULL)
		wglMakeCurrent(hDC, hRC);

	// Prepare to fill the target CDC with the background color.
	// bkgrd_clr = r + g << 8 + b << 16
	COLORREF bkgrd_clr = (int)(background[0]*255) + 
		((int)(background[1]*255) << 8) + 
		((int)(background[2]*255) << 16);

	brush = CreateSolidBrush(bkgrd_clr); // Fill with the current background color
	FillRect(pDC->GetSafeHdc(), &rect, brush);

	PageSize.x = rect.right;
	PageSize.y = PageSize.x * bitmapInfo.bmiHeader.biHeight / bitmapInfo.bmiHeader.biWidth;
	if(PageSize.y > rect.bottom){
		PageSize.y = rect.bottom;
		PageSize.x = PageSize.y * bitmapInfo.bmiHeader.biWidth / bitmapInfo.bmiHeader.biHeight;
	}

	ImageOffset.x = (rect.right - PageSize.x)/2;
	ImageOffset.y = (rect.bottom - PageSize.y)/2;

	// Make sure that all of the operations on the DIB have been completed!
	GdiFlush();

	// Does our device context support StretchDIBits?
	if(m_bStretchDIBits){
		StretchDIBits(pDC->GetSafeHdc(),
					  ImageOffset.x, ImageOffset.y,
					  PageSize.x, PageSize.y,
					  0, 0,
					  bitmapInfo.bmiHeader.biWidth,
					  bitmapInfo.bmiHeader.biHeight,
					  pBitmapBits,
					  &bitmapInfo,
					  DIB_RGB_COLORS,
					  SRCCOPY);
	}
	else{
		if(m_bStretchBlt){
			// Not clear if this code works!
			HDC hdc;
			HBITMAP bitmap;

			hdc	= CreateCompatibleDC(pDC->GetSafeHdc());
			bitmap = CreateDIBitmap(hdc,
								&(bitmapInfo.bmiHeader),
								CBM_INIT,
								pBitmapBits,
								&bitmapInfo,
								DIB_RGB_COLORS);
		
			HGDIOBJ oldbitmap = SelectObject(hdc, bitmap);
			StretchBlt(pDC->GetSafeHdc(),
					   ImageOffset.x, ImageOffset.y,
					   PageSize.x, PageSize.y,
					   hdc,
					   0, 0,
					   bitmapInfo.bmiHeader.biWidth,
					   bitmapInfo.bmiHeader.biHeight,
					   SRCCOPY);
			if (oldbitmap != NULL)
				SelectObject(hdc, oldbitmap);

			// Free our bitmap and bitmap device context
			DeleteObject(bitmap);
			DeleteDC(hdc);
		}
	}

	if (hOldDib != NULL)
		SelectObject(hMemDC, hOldDib);

	// Delete our brush, DIB and device context
	DeleteObject(brush);
	DeleteObject(hDib);
	DeleteDC(hMemDC);
}

BOOL CQmolView::SetMemDcPixelFormat(HDC hMemDC, int bpp)
{
	int nGLPixelIndex;

	PIXELFORMATDESCRIPTOR pixelMem;
	pixelMem.nSize = sizeof(PIXELFORMATDESCRIPTOR);
	pixelMem.nVersion = 1;
	
	pixelMem.dwFlags = PFD_DRAW_TO_BITMAP | PFD_SUPPORT_OPENGL |
		PFD_STEREO_DONTCARE;
	
	pixelMem.iPixelType = PFD_TYPE_RGBA;
	pixelMem.cColorBits = bpp;
	
	pixelMem.cRedBits = 0;
	pixelMem.cRedShift = 0;
	pixelMem.cGreenBits = 0;
	pixelMem.cGreenShift = 0;
	pixelMem.cBlueBits = 0;
	pixelMem.cBlueShift = 0;
	pixelMem.cAlphaBits = 0;
	pixelMem.cAlphaShift = 0;

	pixelMem.cAccumBits = 0;
	pixelMem.cAccumRedBits = 0;
	pixelMem.cAccumGreenBits = 0;
	pixelMem.cAccumBlueBits = 0;
	pixelMem.cAccumAlphaBits = 0;
	pixelMem.cDepthBits = 16;
	pixelMem.cStencilBits = 0;
	pixelMem.cAuxBuffers = 0;
	pixelMem.iLayerType = PFD_MAIN_PLANE;
	pixelMem.bReserved = 0;
	pixelMem.dwLayerMask = 0;
	pixelMem.dwVisibleMask = 0;
	pixelMem.dwDamageMask = 0;

	nGLPixelIndex = ChoosePixelFormat(hMemDC, &pixelMem);
	if (nGLPixelIndex == 0) // Choose default
	{
		nGLPixelIndex = 1;
		if (DescribePixelFormat(hMemDC, nGLPixelIndex, 
			sizeof(PIXELFORMATDESCRIPTOR), &pixelMem))
			return FALSE;
	}

	if (!SetPixelFormat(hMemDC, nGLPixelIndex, &pixelMem))
		return FALSE;

	return TRUE;
}

BOOL CQmolView::OnPreparePrinting(CPrintInfo* pInfo) 
{
	// Pause any animation while we print
	PauseAnimation(TRUE);

	return DoPreparePrinting(pInfo);
}

void CQmolView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	// Restore any animation after printing
	PauseAnimation(FALSE);
	
	CView::OnEndPrinting(pDC, pInfo);
}

void CQmolView::OnRenderAliasLine() 
{
	if(lineAlias){
		glDisable(GL_LINE_SMOOTH);
	}
	else{
		glEnable(GL_LINE_SMOOTH);
	}

	lineAlias = !lineAlias;
	postRedisplay();
}

void CQmolView::OnUpdateRenderAliasLine(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(lineAlias);
}

void CQmolView::SetGeomObjRes(int m_res)
{
	switch(resolution){
		case HIGH_RES:
			atomRes = 25;
			bondRes = 20;
			selectionRes = 20;
			splineRes = 10;
			ribbonRes = 10;
			splineTension = 1.25f;
			break;
		case MED_RES:
			atomRes = 15;
			bondRes = 15;
			selectionRes = 20;
			splineRes = 5;
			ribbonRes = 5;
			splineTension = 1.25f;
			break;
		case LOW_RES:
			atomRes = 8;
			bondRes = 5;
			selectionRes = 20;
			splineRes = 2;
			ribbonRes = 3;
			splineTension = 1.25f;
			break;
		case CUSTOM_RES:
			break;
	};

	BuildGeomObj();
}

void CQmolView::BuildGeomObj()
{
	MakeSphere(SELECTION_SPHERE, selectionRes);
	MakeSphere(ATOM_SPHERE, atomRes);
	MakeDisk(TORSION_DISK, selectionRes);
	MakeCylinder(TORSION_CYL, CYLINDER_HEIGHT, selectionRes);
	MakeCylinder(BOND_CYL, CYLINDER_HEIGHT, bondRes);
	MakeCone(AXIS_CONE, CYLINDER_HEIGHT, selectionRes);
}

void CQmolView::OnRenderHighRes() 
{
	SetGeomObjRes(resolution = HIGH_RES);

	postRedisplay();
}

void CQmolView::OnUpdateRenderHighRes(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(resolution == HIGH_RES);
}

void CQmolView::OnRenderLowRes() 
{
	SetGeomObjRes(resolution = LOW_RES);

	postRedisplay();
}

void CQmolView::OnUpdateRenderLowRes(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(resolution == LOW_RES);
}

void CQmolView::OnRenderMedRes() 
{
	SetGeomObjRes(resolution = MED_RES);

	postRedisplay();
}

void CQmolView::OnUpdateRenderMedRes(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(resolution == MED_RES);
}

void CQmolView::OnRenderCustomRes() 
{
	if(m_customResDlg->GetSafeHwnd() == 0)
		m_customResDlg->Create();
}

void CQmolView::OnUpdateRenderCustomRes(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(resolution == CUSTOM_RES);
}

void CQmolView::OnRenderSmooth3d() 
{
	smooth3D = !smooth3D;
	postRedisplay();
}

void CQmolView::OnUpdateRenderSmooth3d(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(smooth3D);
}


void CQmolView::OnFilePrintOption() 
{
	if(mem_size < 0){
		CPrintOption poDlg(QMOL_DEFAULT_MEM_SIZE, 
			CPrintOption::MEM_NO_LIMIT, print_color_depth);

		if(poDlg.DoModal() == IDOK){
			if(poDlg.mem_status == CPrintOption::MEM_LIMIT)
				mem_size = poDlg.m_mem_size;
			else
				mem_size = -1;

			print_color_depth = poDlg.depth;
		}
	}
	else{
		CPrintOption poDlg(mem_size, CPrintOption::MEM_LIMIT,
			print_color_depth);
		
		if(poDlg.DoModal() == IDOK){
			if(poDlg.mem_status == CPrintOption::MEM_LIMIT)
				mem_size = poDlg.m_mem_size;
			else
				mem_size = -1;

			print_color_depth = poDlg.depth;
		}
	}	
}

void CQmolView::OnRenderMonitor() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->monitors = !(*tmpMol)->monitors;

	postRedisplay();
}

void CQmolView::OnUpdateRenderMonitor(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->monitors);
}

void CQmolView::OnRenderMonitorLabels() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->monitorLabels = !(*tmpMol)->monitorLabels;

	postRedisplay();
}

void CQmolView::OnUpdateRenderMonitorLabels(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->SetCheck(ptrMol && ptrMol->monitorLabels);
}

void CQmolView::OnRenderHbonds() 
{
	showHBonds = !showHBonds;

	if(showHBonds){
		CQmolDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		pDoc->UpdateHydrogenBonding();
	}
	
	postRedisplay();
}

void CQmolView::OnUpdateRenderHbonds(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showHBonds);
}

void CQmolView::OnEditCopy() 
{

	// Don't use the SaveMetaFile function to copy to the clipboard
	// as some programs that treat Metafiles as vector data will
	// complain (i.e. Illustrator 9).
	//SaveMetaFile();

	// Set the hourglass cursor - this may take a while
	::SetCursor(LoadCursor(NULL, IDC_WAIT));
	
	CBitmap *pBitmap = new CBitmap();

	if(pBitmap == NULL){
		AfxMessageBox("Unable to create Bitmap for clipboard copy");
		::SetCursor(LoadCursor(NULL, IDC_ARROW));

		return;
	}

	// Get the current (screen) device context
	CDC *pDC = GetDC();
	CDC bitmapDC;

	bitmapDC.CreateCompatibleDC(pDC);

	pBitmap->CreateCompatibleBitmap(pDC, viewWidth, viewHeight);
	
	bitmapDC.SelectObject(pBitmap);

	OnPrint(&bitmapDC, NULL);

	// Copy to clipboard
	if(OpenClipboard()){
		if(!EmptyClipboard()){
			AfxMessageBox("Can't empty clipboard!");

			CloseClipboard();
			delete pBitmap;
			ReleaseDC(pDC);
			::SetCursor(LoadCursor(NULL, IDC_ARROW));

			return;
		}

		::SetClipboardData(CF_BITMAP, pBitmap->GetSafeHandle());
		CloseClipboard();
	}
	else{
		AfxMessageBox("Can't open clipboard -- it is currently opened by "
			"another appilcation!");
	}

	// The bitmap is now owned by the clipboard
	pBitmap->Detach();

	delete pBitmap;

	ReleaseDC(pDC);

	// Set the hourglass cursor
	::SetCursor(LoadCursor(NULL, IDC_ARROW));
}

// A helper function to copy the current view to the clipboard. No longer used!
// Images are now copied to the clipboard as bitmaps.
void CQmolView::SaveMetaFile()
{
	// Set the hourglass cursor - this may take a while
	::SetCursor(LoadCursor(NULL, IDC_WAIT));

	CMetaFileDC m_pMetaDC;
	CDC *pDC = GetDC();

	m_pMetaDC.CreateEnhanced(pDC, NULL, NULL, "qmol\0doc\0\0");

	m_pMetaDC.SetAttribDC(pDC->m_hAttribDC);

	// "Print" to the windows clip board
	OnPrint(&m_pMetaDC, NULL);

	//close meta file dc and prepare for clipboard;
	HENHMETAFILE hMF = m_pMetaDC.CloseEnhanced();

	if(!hMF){
		AfxMessageBox("Unable to prepare Metafile for pasting!");
		ReleaseDC(pDC);
		return;
	}

	//copy to clipboard
	if(OpenClipboard()){
		if(!EmptyClipboard()){
			AfxMessageBox("Can't empty clipboard!");
			CloseClipboard();
			::DeleteEnhMetaFile(hMF);
			ReleaseDC(pDC);
			return;
		}

		::SetClipboardData(CF_ENHMETAFILE, hMF);
		CloseClipboard();
	}
	else{
		AfxMessageBox("Can't open clipboard -- it is currently opened by "
			"another appilcation!");
	}

	// Woops! Don't delete metafile handle -- this memory is now owned by
	// the clipboard!
	// Clean up the metafile
	//::DeleteEnhMetaFile(hMF);

	ReleaseDC(pDC);

	// Set the hourglass cursor
	::SetCursor(LoadCursor(NULL, IDC_ARROW));
}

void CQmolView::SetBackgroundColor()
{
	switch(color_scheme){
		case QM_COLOR_CPK:
		case QM_COLOR_DEFAULT:
		default:
			background[0] = 0.0f;
			background[1] = 0.0f;
			background[2] = 0.0f;
			background[3] = 1.0f;
	};
}

void CQmolView::OnOptionsAtomColor() 
{
	if(m_qmColorDlg->GetSafeHwnd() == 0){
		m_qmColorDlg->Create();
	}
}

void CQmolView::OnUpdateOptionsAtomColor(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);
}

void CQmolView::OnOptionsSequenceColor() 
{
	if(m_seqDlg->GetSafeHwnd() == 0)
		m_seqDlg->Create();
}

void CQmolView::OnUpdateOptionsSequenceColor(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);
}

void CQmolView::OnRenderAxes() 
{
	axes = !axes;

	postRedisplay();
}

void CQmolView::OnUpdateRenderAxes(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(axes);
}

void CQmolView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Allow the user to rotate the view using the keyboard arrow keys
	const float delta = 0.05f;	// delta controls the size of the rotation
	const int delta_scale = 10;	// delta_scale controls the size of the scaling

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	switch(nChar){
		// Allow the user to use the arrow keys on the keyboard to
		// shift the x and y axis (for linear scales only).
		case 37: // Left arrow
			if(shift_key_down){
				beginScaleX = 0;
				Scale(delta_scale);
			}
			else{
				referenceFrame.Update(delta, 0.0f, 0.0f, 0.0f);
  		
  				while(tmpMol = next())
					(*tmpMol)->Update(delta, 0.0f, 0.0f, 0.0f);

    			postRedisplay();
			}
			break;
		case 38: // Up arrow
			if(shift_key_down){
				beginScaleX = 0;
				Scale(-delta_scale);
			}
			else{
				referenceFrame.Update(0.0f, 0.0f, 0.0f, delta);
  				
				while(tmpMol = next())
					(*tmpMol)->Update(0.0f, 0.0f, 0.0f, delta);

    			postRedisplay();
			}
			break;
		case 39: // Right arrow
			if(shift_key_down){
				beginScaleX = 0;
				Scale(-delta_scale);
			}
			else{
				referenceFrame.Update(0.0f, 0.0f, delta, 0.0f);
  				
				while(tmpMol = next())
					(*tmpMol)->Update(0.0f, 0.0f, delta, 0.0f);

    			postRedisplay();
			}
			break;
		case 40: // Down arrow
			if(shift_key_down){
				beginScaleX = 0;
				Scale(delta_scale);
			}
			else{
				referenceFrame.Update(0.0f, delta, 0.0f, 0.0f);
  				
				while(tmpMol = next())
					(*tmpMol)->Update(0.0f, delta, 0.0f, 0.0f);

    			postRedisplay();
			}
			break;
		case 16: // The shift key
			shift_key_down = TRUE;
			break;
#ifdef _DEBUG
		case 'D':	// Toggle debugging
			debug_flag = !debug_flag;
			postRedisplay();
			break;
		case 'P':	// Turn on "point" display of solid shapes
			glPolygonMode(GL_FRONT_AND_BACK, GL_POINT);
			postRedisplay();
			break;
		case 'W':	// Turn on "wire frame" display of solid shapes
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			postRedisplay();
			break;
		case 'S':	// Turn on "solid" display of solid shapes
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			postRedisplay();
			break;
#endif // _DEBUG
	};

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CQmolView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch(nChar){
		case 16: // The shift key
			shift_key_down = FALSE;
	};
	
	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CQmolView::OnAdjustRadius() 
{
	// Create the Adjust Torsion Dialog
	if(m_pARDlg->GetSafeHwnd() == 0){
		m_pARDlg->Create();
	}
}

// Close the Adjust Torsion dialog
LRESULT CQmolView::OnAdjustRadiusClose(WPARAM wParam, LPARAM lParam) 
{
	m_pARDlg->DestroyWindow();

	return 0L;
}

void CQmolView::OnOptionsSelectColor() 
{
	if(m_flexClrDlg->GetSafeHwnd() == 0){
		// Make sure that the dialog displays the current
		// background color.
		m_flexClrDlg->SetRGBA(
			(int)(selection_color[0]*255.0f),
			(int)(selection_color[1]*255.0f),
			(int)(selection_color[2]*255.0f),
			(int)(selection_color[3]*255.0f));

		m_flexClrDlg->Create(SELECTION_CLR);
	}
}

void CQmolView::OnOptionsHbondColor() 
{
	if(m_flexClrDlg->GetSafeHwnd() == 0){
		// Make sure that the dialog displays the current
		// hbond color.
		m_flexClrDlg->SetRGB(
			(int)(hbond_color[0]*255.0f),
			(int)(hbond_color[1]*255.0f),
			(int)(hbond_color[2]*255.0f));

		m_flexClrDlg->Create(HBOND_CLR);
	}
}

void CQmolView::OnRenderLighting() 
{
	// Create the lighting Dialog
	if(m_lightDlg->GetSafeHwnd() == 0){
		// Turn lights on/off
		m_lightDlg->SetLightStatus(CLightDialog::FRONT, lb_solid.Status(0));
		m_lightDlg->SetLightStatus(CLightDialog::BACK, lb_solid.Status(3));
		m_lightDlg->SetLightStatus(CLightDialog::LEFT, lb_solid.Status(4));
		m_lightDlg->SetLightStatus(CLightDialog::RIGHT, lb_solid.Status(1));
		m_lightDlg->SetLightStatus(CLightDialog::TOP, lb_solid.Status(2));
		m_lightDlg->SetLightStatus(CLightDialog::BOTTOM, lb_solid.Status(5));
		m_lightDlg->SetLightStatus(CLightDialog::AMBIENT, lb_solid.Status(6));
		
		// Set the material properties (as percentages [0%, 100%]).
		int shiny = (int)(100.0f*mat_shininess[0]/128.0f);
		int specular = (int)(100.0f*mat_specular[0]);

		m_lightDlg->SetMaterialProperties(shiny, specular);

		// Set the light color
		GLfloat tmpColor[3];

		lb_solid.GetSpecular(0, tmpColor[0], tmpColor[1], tmpColor[2]);
		m_lightDlg->SetRGB(int(255*tmpColor[0]), 
						   int(255*tmpColor[1]), 
						   int(255*tmpColor[2]));

		m_lightDlg->Create();
	}
}

// Close the Adjust Torsion dialog
LRESULT CQmolView::OnLightingMessage(WPARAM wParam, LPARAM lParam) 
{
	if(wParam == IDCANCEL){
		m_lightDlg->DestroyWindow();
		return 0L;
	}

	if(m_lightDlg->GetLightStatus(CLightDialog::FRONT))
		lb_solid.TurnOn(0);
	else
		lb_solid.TurnOff(0);

	if(m_lightDlg->GetLightStatus(CLightDialog::BACK))
		lb_solid.TurnOn(3);
	else
		lb_solid.TurnOff(3);

	if(m_lightDlg->GetLightStatus(CLightDialog::LEFT))
		lb_solid.TurnOn(4);
	else
		lb_solid.TurnOff(4);

	if(m_lightDlg->GetLightStatus(CLightDialog::RIGHT))
		lb_solid.TurnOn(1);
	else
		lb_solid.TurnOff(1);

	if(m_lightDlg->GetLightStatus(CLightDialog::TOP))
		lb_solid.TurnOn(2);
	else
		lb_solid.TurnOff(2);

	if(m_lightDlg->GetLightStatus(CLightDialog::BOTTOM))
		lb_solid.TurnOn(5);
	else
		lb_solid.TurnOff(5);

	if(m_lightDlg->GetLightStatus(CLightDialog::AMBIENT))
		lb_solid.TurnOn(6);
	else
		lb_solid.TurnOff(6);

	// Get the light color
	int tmp_r, tmp_g, tmp_b;
	float f_r, f_g, f_b;

	m_lightDlg->GetRGB(tmp_r, tmp_g, tmp_b);
			
	f_r = tmp_r/255.0f;
	f_g = tmp_g/255.0f;
	f_b = tmp_b/255.0f;

	for(int i = 0;i < QM_NUM_LIGHTS - 1;i++){
		lb_solid.SetSpecular(i, f_r, f_g, f_b);
		lb_solid.SetAmbient(i, 0.0f, 0.0f, 0.0f);
		lb_solid.SetDiffuse(i, 0.5f*f_r, 0.5f*f_g, 0.5f*f_b);
	}

	lb_solid.SetAmbient(QM_AMBIENT, 0.2f*f_r, 0.2f*f_g, 0.2f*f_b);

	lb_solid.Apply();

	// Get the material properties (as percentages [0%, 100%]).
	int shiny;
	int specular;

	m_lightDlg->GetMaterialProperties(shiny, specular);
	
	mat_shininess[0] = shiny*128.0f/100.0f;

	// Note that the alpha values are NOT updated here!
	mat_specular[0] = mat_specular[1] = mat_specular[2] = 
		specular/100.0f;

	// Update the material properties for OpenGL
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
  	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

	postRedisplay();

	if(wParam == IDOK)
		m_lightDlg->DestroyWindow();

	return 0L;
}

void CQmolView::OnFileMovie() 
{
	PauseAnimation(TRUE);

	CRenderMovieDialog rmDlg(this);
	rmDlg.DoModal();

	PauseAnimation(FALSE);
}

void CQmolView::OnUpdateFileMovie(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->Enable(ptrMol && (ptrMol->steps > 0));
}

void CQmolView::OnFileCoordOpen() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	PauseAnimation(TRUE);
	
	if(pDoc->OnFileCoordOpen()){
		// Reset the view
		Update();
	}
	else{
		PauseAnimation(FALSE);
	}
}

void CQmolView::OnNetCoordOpen() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	PauseAnimation(TRUE);
	
	if(pDoc->OnNetFileCoordOpen()){
		// Reset the view
		Update();
	}
	else{
		PauseAnimation(FALSE);
	}
	
}

void CQmolView::OnFileCoordSave() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	PauseAnimation(TRUE);
	
	pDoc->OnFileCoordSave();
	
	PauseAnimation(FALSE);
}

void CQmolView::OnUpdateFileCoordSave(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);

	pCmdUI->Enable(ptrMol && (ptrMol->NumAtoms() > 0));
}

void CQmolView::OnFileOpenCoordTraj() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	PauseAnimation(TRUE);
	
	if(pDoc->OnFileOpenCoordTraj()){
		// Reset the view
		Update();
	}
	else{
		PauseAnimation(FALSE);
	}
}

LRESULT CQmolView::OnResolutionMessage(WPARAM wParam, LPARAM lParam)
{

	switch(wParam){
		case CCustomResDialog::UPDATE_ATOM:
			resolution = CUSTOM_RES;
			atomRes = lParam;
			MakeSphere(ATOM_SPHERE, atomRes);
			postRedisplay();
			break;
		case CCustomResDialog::UPDATE_BOND:
			resolution = CUSTOM_RES;
			bondRes = lParam;
			MakeCylinder(BOND_CYL, CYLINDER_HEIGHT, bondRes);
			postRedisplay();
			break;
		case CCustomResDialog::UPDATE_SELECT:
			resolution = CUSTOM_RES;
			selectionRes = lParam;
			MakeSphere(SELECTION_SPHERE, selectionRes);
			MakeDisk(TORSION_DISK, selectionRes);
			MakeCylinder(TORSION_CYL, CYLINDER_HEIGHT, selectionRes);
			MakeCone(AXIS_CONE, CYLINDER_HEIGHT, selectionRes);
			postRedisplay();
			break;
		case CCustomResDialog::UPDATE_SPLINE:
			resolution = CUSTOM_RES;
			splineRes = lParam;
			postRedisplay();
			break;
		case CCustomResDialog::UPDATE_RIBBON:
			resolution = CUSTOM_RES;
			ribbonRes = lParam;
			postRedisplay();
			break;
		case CCustomResDialog::UPDATE_TENSION:
			resolution = CUSTOM_RES;

			if(m_customResDlg != NULL)
				splineTension = m_customResDlg->GetSplineTension(lParam);
			else 
				splineTension = 1.5f;

			postRedisplay();
			break;
		case CCustomResDialog::CLOSE_DLG:
			if(m_customResDlg->GetSafeHwnd())
				m_customResDlg->DestroyWindow();

			break;
	};

	return 0L;
}


LRESULT CQmolView::OnSequenceDialogMsg(WPARAM wParam, LPARAM lParam)
{
	// Currently, close is the only message!
	m_seqDlg->DestroyWindow();

	return 0L;
}

void CQmolView::OnUpdateOptionsStrucAssign(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol && !m_seqDlg->GetSafeHwnd());
}

void CQmolView::OnOptionsStrucAssign() 
{
	if(m_seqDlg->GetSafeHwnd() == 0)
		m_seqDlg->Create();
}

moleculeSpline::moleculeSpline(molecule *m_mol, int m_chain, int m_res, 
							   GLfloat m_gamma /*= 2.0f*/)
{
	#ifdef _DEBUG
	if(m_mol == NULL)
		throw QmolError("Bad mol ptr", __FILE__, __LINE__);
	#endif // _DEBUG

	mol = m_mol;

	if(m_res < 0)
		throw QmolError("Bad resolution", __FILE__, __LINE__);

	dt = 1.0f/(m_res + 1);

	if((m_gamma < 1.0) || (m_gamma > 3.0))
		gamma = 2.0f;
	else
		gamma = m_gamma;

	inv_gamma = 1.0f/gamma;

	if(m_chain < 0)
		throw QmolError("Bad chain number -- too small", __FILE__, __LINE__);

	if(m_chain > mol->secondaryStruc.Length())
		throw QmolError("Bad chain number -- too large", __FILE__, __LINE__);
	
	chain = m_chain;

	num_knots = mol->secondaryStruc[chain].sequence.Length();

	Reset();
}

void moleculeSpline::Reset()
{
	update = TRUE;

	cur_knot = 0;

	t = 0.0f;
}

BOOL moleculeSpline::NextTrace(GLfloat r[], int &index, int cur_step)
{
	// Compute a Catmull-Rom spline. See "Computer Graphics Principles and Pratice"
	// by Foley, van Dam, Feiner and Hughes. The spline control parameter form is
	// taken from the VMD program. The actual value of the spline parameter is
	// under user control.
	//
	// The spline has the form:
	// Q(t) = TM(t) . G

	if(t >= 1.0f){
		cur_knot ++;
		t = 0.0f;
	}

	if((cur_knot == num_knots - 1) || (num_knots < 4))
		return FALSE;

	TM[0] = -t*inv_gamma*(1.0f + t*(-2.0f + t));
	TM[1] = -inv_gamma*(-gamma + t*t*(3.0f*gamma - 1.0f + t*(1.0f - 2.0f*gamma)));
	TM[2] = t*inv_gamma*(1.0f + t*(3.0f*gamma - 2.0f + t*(1.0f - 2.0f*gamma)));
	TM[3] = t*t*inv_gamma*(t - 1.0f);

	amino *amino_ptr;
	array<amino> &sequence_ref = mol->secondaryStruc[chain].sequence;

	if(cur_knot == 0){
		amino_ptr = &(sequence_ref[2]);

		memcpy(G_c[3], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
			3*sizeof(float));

		amino_ptr = &(sequence_ref[1]);

		memcpy(G_c[2], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
			3*sizeof(float));

		amino_ptr = &(sequence_ref[0]);

		memcpy(G_c[1], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
			3*sizeof(float));

		G_c[0][0] = 2.0f*G_c[1][0] - G_c[2][0];
		G_c[0][1] = 2.0f*G_c[1][1] - G_c[2][1];
		G_c[0][2] = 2.0f*G_c[1][2] - G_c[2][2];
	}
	else{
		amino_ptr = &(sequence_ref[cur_knot - 1]);

		memcpy(G_c[0], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
			3*sizeof(float));

		amino_ptr = &(sequence_ref[cur_knot]);

		memcpy(G_c[1], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
			3*sizeof(float));

		amino_ptr = &(sequence_ref[cur_knot + 1]);

		memcpy(G_c[2], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
			3*sizeof(float));

		if(cur_knot == num_knots - 2){
			G_c[3][0] = 2.0f*G_c[2][0] - G_c[1][0];
			G_c[3][1] = 2.0f*G_c[2][1] - G_c[1][1];
			G_c[3][2] = 2.0f*G_c[2][2] - G_c[1][2];
		}
		else{
			amino_ptr = &(sequence_ref[cur_knot + 2]);
			
			memcpy(G_c[3], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));
		}
	}

	t += dt;

	// r = TM*G_c;
	r[0] = TM[0]*G_c[0][0] + TM[1]*G_c[1][0] + TM[2]*G_c[2][0] + TM[3]*G_c[3][0];
	r[1] = TM[0]*G_c[0][1] + TM[1]*G_c[1][1] + TM[2]*G_c[2][1] + TM[3]*G_c[3][1];
	r[2] = TM[0]*G_c[0][2] + TM[1]*G_c[1][2] + TM[2]*G_c[2][2] + TM[3]*G_c[3][2];

	index = sequence_ref[cur_knot].c_alpha_index;

	return TRUE;
}

BOOL moleculeSpline::NextTrace(GLfloat r[], int &index)
{
	// Compute a Catmull-Rom spline. See "Computer Graphics Principles and Pratice"
	// by Foley, van Dam, Feiner and Hughes. The spline control parameter form is
	// taken from the VMD program. The actual value of the spline parameter is
	// under user control.
	//
	// The spline has the form:
	// Q(t) = TM(t) . G

	if(t >= 1.0f){
		update = TRUE;
		cur_knot ++;
		t = 0.0f;
	}

	if((cur_knot == num_knots - 1) || (num_knots < 4))
		return FALSE;

	TM[0] = -t*inv_gamma*(1.0f + t*(-2.0f + t));
	TM[1] = -inv_gamma*(-gamma + t*t*(3.0f*gamma - 1.0f + t*(1.0f - 2.0f*gamma)));
	TM[2] = t*inv_gamma*(1.0f + t*(3.0f*gamma - 2.0f + t*(1.0f - 2.0f*gamma)));
	TM[3] = t*t*inv_gamma*(t - 1.0f);

	amino *amino_ptr;
	atom *atom_ptr;
	array<amino> &sequence_ref = mol->secondaryStruc[chain].sequence;
	
	if(update){
		if(cur_knot == 0){
			amino_ptr = &(sequence_ref[2]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[3]);

			amino_ptr = &(sequence_ref[1]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[2]);

			amino_ptr = &(sequence_ref[0]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[1]);

			G_c[0][0] = 2.0f*G_c[1][0] - G_c[2][0];
			G_c[0][1] = 2.0f*G_c[1][1] - G_c[2][1];
			G_c[0][2] = 2.0f*G_c[1][2] - G_c[2][2];
		}
		else{
			memcpy(G_c[0], G_c[1], 3*sizeof(float));
			memcpy(G_c[1], G_c[2], 3*sizeof(float));
			memcpy(G_c[2], G_c[3], 3*sizeof(float));

			if(cur_knot == num_knots - 2){
				G_c[3][0] = 2.0f*G_c[2][0] - G_c[1][0];
				G_c[3][1] = 2.0f*G_c[2][1] - G_c[1][1];
				G_c[3][2] = 2.0f*G_c[2][2] - G_c[1][2];
			}
			else{
				amino_ptr = &(sequence_ref[cur_knot + 2]);
				atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

				atom_ptr->GetCoord(G_c[3]);
			}
		}

		update = !update;
	}

	t += dt;

	//r = TM*G_c;
	r[0] = TM[0]*G_c[0][0] + TM[1]*G_c[1][0] + TM[2]*G_c[2][0] + TM[3]*G_c[3][0];
	r[1] = TM[0]*G_c[0][1] + TM[1]*G_c[1][1] + TM[2]*G_c[2][1] + TM[3]*G_c[3][1];
	r[2] = TM[0]*G_c[0][2] + TM[1]*G_c[1][2] + TM[2]*G_c[2][2] + TM[3]*G_c[3][2];

	index = sequence_ref[cur_knot].c_alpha_index;

	return TRUE;
}

BOOL moleculeSpline::NextRibbon(GLfloat r[], GLfloat major[], GLfloat minor[], 
								int &index)
{
	// Compute a Catmull-Rom spline. See "Computer Graphics Principles and Pratice"
	// by Foley, van Dam, Feiner and Hughes. The spline control parameter form is
	// taken from the VMD program. The actual value of the spline parameter is
	// under user control.
	//
	// The spline has the form:
	// Q(t) = TM(t) . G

	if(t >= 1.0f){
		update = TRUE;
		cur_knot ++;
		t = 0.0f;
	}

	if((cur_knot == num_knots - 1) || (num_knots < 4))
		return FALSE;

	int i;
	GLfloat mag, major_mag, minor_mag;
	GLfloat b_dot_a;
	amino *amino_ptr;
	atom *atom_ptr;
	array<amino> &sequence_ref = mol->secondaryStruc[chain].sequence;

	// Unlike the NextTrace() function, here we also compute a spline through
	// the carbonyl oxygens and the major and minor axes of the ellipse cross
	// section for each animo acid. The carbonyl oxygens provide guide coordinates
	// for aligning the major axis of the ribbon. This follows the work presented in
	// M. Carson (1997) Ribbons. Methods in Enzymology 277:493-505,
	// R.M. Sweet and C.W. Carter, eds, Academic Press.
    // M.Carson (1991) Ribbons 2.0, J.Appl.Cryst. 24:958-961  or
    // M.Carson (1987) Ribbon Models of Macromolecules, J.Mol.Graphics, 5:103-106.
	// (note that a different interpolation scheme is used).
	if(update){
		if(cur_knot == 0){
			amino_ptr = &(sequence_ref[2]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[3]);

			atom_ptr = &(mol->atoms[amino_ptr->o_index]);

			atom_ptr->GetCoord(G_g[3]);

			G_major[3] = amino_ptr->major;
			G_minor[3] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[1]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[2]);

			atom_ptr = &(mol->atoms[amino_ptr->o_index]);

			atom_ptr->GetCoord(G_g[2]);

			G_major[2] = amino_ptr->major;
			G_minor[2] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[0]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[1]);
			
			atom_ptr = &(mol->atoms[amino_ptr->o_index]);

			atom_ptr->GetCoord(G_g[1]);

			G_major[1] = amino_ptr->major;
			G_minor[1] = amino_ptr->minor;

			G_c[0][0] = 2.0f*G_c[1][0] - G_c[2][0];
			G_c[0][1] = 2.0f*G_c[1][1] - G_c[2][1];
			G_c[0][2] = 2.0f*G_c[1][2] - G_c[2][2];

			G_g[0][0] = 2.0f*G_g[1][0] - G_g[2][0];
			G_g[0][1] = 2.0f*G_g[1][1] - G_g[2][1];
			G_g[0][2] = 2.0f*G_g[1][2] - G_g[2][2];
				
			G_major[0] = G_major[1];
			G_minor[0] = G_minor[1];
		}
		else{
			amino_ptr = &(sequence_ref[cur_knot - 1]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[0]);

			atom_ptr = &(mol->atoms[amino_ptr->o_index]);

			atom_ptr->GetCoord(G_g[0]);

			G_major[0] = amino_ptr->major;
			G_minor[0] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[cur_knot]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[1]);

			atom_ptr = &(mol->atoms[amino_ptr->o_index]);

			atom_ptr->GetCoord(G_g[1]);

			G_major[1] = amino_ptr->major;
			G_minor[1] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[cur_knot + 1]);
			atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

			atom_ptr->GetCoord(G_c[2]);

			atom_ptr = &(mol->atoms[amino_ptr->o_index]);

			atom_ptr->GetCoord(G_g[2]);

			G_major[2] = amino_ptr->major;
			G_minor[2] = amino_ptr->minor;

			if(cur_knot == num_knots - 2){
				G_c[3][0] = 2.0f*G_c[2][0] - G_c[1][0];
				G_c[3][1] = 2.0f*G_c[2][1] - G_c[1][1];
				G_c[3][2] = 2.0f*G_c[2][2] - G_c[1][2];

				G_g[3][0] = 2.0f*G_g[2][0] - G_g[1][0];
				G_g[3][1] = 2.0f*G_g[2][1] - G_g[1][1];
				G_g[3][2] = 2.0f*G_g[2][2] - G_g[1][2];

				G_major[3] = 2.0f*G_major[2] - G_major[1];
				G_minor[3] = 2.0f*G_minor[2] - G_minor[1];
			}
			else{
				amino_ptr = &(sequence_ref[cur_knot + 2]);
				atom_ptr = &(mol->atoms[amino_ptr->c_alpha_index]);

				atom_ptr->GetCoord(G_c[3]);

				atom_ptr = &(mol->atoms[amino_ptr->o_index]);

				atom_ptr->GetCoord(G_g[3]);
				
				G_major[3] = amino_ptr->major;
				G_minor[3] = amino_ptr->minor;
			}
		}

		// Flip carbonyl guide coordinates as needed
		if(cur_knot == 0){
			// Determine the guide orientation based on the guide two
			// orientation.
			for(i = 0;i < 3;i++){
				A[i] = G_g[1][i] - G_c[1][i];
				B[i] = G_c[2][i] - G_c[1][i];
			}
		
			//tmp = (B | A) | B;
			mag = DOT3D(B, B);
			b_dot_a = DOT3D(B, A);

			guide_ref[X] = A[X]*mag - B[X]*b_dot_a;
			guide_ref[Y] = A[Y]*mag - B[Y]*b_dot_a;
			guide_ref[Z] = A[Z]*mag - B[Z]*b_dot_a;
			
			mag = (GLfloat)(sqrt(DOT3D(guide_ref, guide_ref)));

			guide_ref[X] /= mag;
			guide_ref[Y] /= mag;
			guide_ref[Z] /= mag;
		}

		// Guide One
		for(i = 0;i < 3;i++){
			A[i] = G_g[0][i] - G_c[0][i];
			B[i] = G_c[1][i] - G_c[0][i];
		}
		
		// The following triple product:
		// guide_ref = (B | A) | B
		// is equivalent to: A(B*B) - B(B*A)

		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);
		
		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
			
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[0][X] = G_c[0][X] - tmp[X];
			G_g[0][Y] = G_c[0][Y] - tmp[Y];
			G_g[0][Z] = G_c[0][Z] - tmp[Z];
		}
		else{
			G_g[0][X] = G_c[0][X] + tmp[X];
			G_g[0][Y] = G_c[0][Y] + tmp[Y];
			G_g[0][Z] = G_c[0][Z] + tmp[Z];
		}

		// Guide Two
		for(i = 0;i < 3;i++){
			A[i] = G_g[1][i] - G_c[1][i];
			B[i] = G_c[2][i] - G_c[1][i];
		}
		
		//tmp = (B | A) | B;
		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
		
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[1][X] = G_c[1][X] - tmp[X];
			G_g[1][Y] = G_c[1][Y] - tmp[Y];
			G_g[1][Z] = G_c[1][Z] - tmp[Z];

			// Save the current reference guide orientation
			// (guide two is used for orienting all spline
			// segments).
			guide_ref[X] = - tmp[X];
			guide_ref[Y] = - tmp[Y];
			guide_ref[Z] = - tmp[Z];
		}
		else{
			G_g[1][X] = G_c[1][X] + tmp[X];
			G_g[1][Y] = G_c[1][Y] + tmp[Y];
			G_g[1][Z] = G_c[1][Z] + tmp[Z];

			// Save the current reference guide orientation
			// (guide two is used for orienting all spline
			// segments).
			guide_ref[X] = tmp[X];
			guide_ref[Y] = tmp[Y];
			guide_ref[Z] = tmp[Z];
		}


		// Guide Three
		for(i = 0;i < 3;i++){
			A[i] = G_g[2][i] - G_c[2][i];
			B[i] = G_c[3][i] - G_c[2][i];
		}

		//tmp = (B | A) | B;
		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
			
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[2][X] = G_c[2][X] - tmp[X];
			G_g[2][Y] = G_c[2][Y] - tmp[Y];
			G_g[2][Z] = G_c[2][Z] - tmp[Z];
		}
		else{
			G_g[2][X] = G_c[2][X] + tmp[X];
			G_g[2][Y] = G_c[2][Y] + tmp[Y];
			G_g[2][Z] = G_c[2][Z] + tmp[Z];
		}

		// Guide Four
		for(i = 0;i < 3;i++){
			A[i] = G_c[3][i] - G_g[3][i];
			B[i] = G_c[2][i] - G_c[3][i];
		}

		//tmp = (B | A) | B;
		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
			
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[3][X] = G_c[3][X] - tmp[X];
			G_g[3][Y] = G_c[3][Y] - tmp[Y];
			G_g[3][Z] = G_c[3][Z] - tmp[Z];
		}
		else{
			G_g[3][X] = G_c[3][X] + tmp[X];
			G_g[3][Y] = G_c[3][Y] + tmp[Y];
			G_g[3][Z] = G_c[3][Z] + tmp[Z];
		}

		update = !update;
	}

	TM[0] = -t*inv_gamma*(1.0f + t*(-2.0f + t));
	TM[1] = -inv_gamma*(-gamma + t*t*(3.0f*gamma - 1.0f + t*(1.0f - 2.0f*gamma)));
	TM[2] = t*inv_gamma*(1.0f + t*(3.0f*gamma - 2.0f + t*(1.0f - 2.0f*gamma)));
	TM[3] = t*t*inv_gamma*(t - 1.0f);	

	// The center of the ellipse
	//r = TM*G_c;
	r[0] = TM[0]*G_c[0][0] + TM[1]*G_c[1][0] + TM[2]*G_c[2][0] + TM[3]*G_c[3][0];
	r[1] = TM[0]*G_c[0][1] + TM[1]*G_c[1][1] + TM[2]*G_c[2][1] + TM[3]*G_c[3][1];
	r[2] = TM[0]*G_c[0][2] + TM[1]*G_c[1][2] + TM[2]*G_c[2][2] + TM[3]*G_c[3][2];

	index = sequence_ref[cur_knot].c_alpha_index;

	// Compute the interpolated ribbon size (both major and minor axes
	major_mag = TM[0]*G_major[0] + TM[1]*G_major[1] + TM[2]*G_major[2] + TM[3]*G_major[3]
		+ RIBBON_SIZE_OFFSET;
	minor_mag = TM[0]*G_minor[0] + TM[1]*G_minor[1] + TM[2]*G_minor[2] + TM[3]*G_minor[3]
		+ RIBBON_SIZE_OFFSET;

	// Compute the major and minor axes of the ellipse
	//B = TM*G_g - r
	major[X] = TM[0]*G_g[0][0] + TM[1]*G_g[1][0] + TM[2]*G_g[2][0] + TM[3]*G_g[3][0] - r[X];
	major[Y] = TM[0]*G_g[0][1] + TM[1]*G_g[1][1] + TM[2]*G_g[2][1] + TM[3]*G_g[3][1] - r[Y];
	major[Z] = TM[0]*G_g[0][2] + TM[1]*G_g[1][2] + TM[2]*G_g[2][2] + TM[3]*G_g[3][2] - r[Z];

	// Compute the derivative of the TM vector with respect to t
	TM[0] = -inv_gamma*(1.0f - t*(4.0f - 3.0f*t));
	TM[1] = t*(-6.0f + 2.0f*inv_gamma + t*(-3.0f*inv_gamma + 6.0f));
	TM[2] = inv_gamma + t*(6.0f - 4.0f*inv_gamma + t*(3.0f*inv_gamma - 6.0f));
	TM[3] = inv_gamma*t*(3.0f*t - 2.0f);

	//dr = d(TM*G_c)/dt
	dr[X] = TM[0]*G_c[0][0] + TM[1]*G_c[1][0] + TM[2]*G_c[2][0] + TM[3]*G_c[3][0];
	dr[Y] = TM[0]*G_c[0][1] + TM[1]*G_c[1][1] + TM[2]*G_c[2][1] + TM[3]*G_c[3][1];
	dr[Z] = TM[0]*G_c[0][2] + TM[1]*G_c[1][2] + TM[2]*G_c[2][2] + TM[3]*G_c[3][2];

	// Compute a TM vector for interpolating ribbon width. This has the same form as the TM vector
	// used above, but has gamma == 2.0 (for a nice looking ribbon).
	TM[0] = -t*0.5f*(1.0f + t*(-2.0f + t));
	TM[1] = -0.5f*(-2.0f + t*t*(5.0f - 3.0f*t));
	TM[2] = t*0.5f*(1.0f + t*(4.0f - 3.0f*t));
	TM[3] = t*t*0.5f*(t - 1.0f);	

	// Compute the interpolated ribbon size (both major and minor axes
	major_mag = TM[0]*G_major[0] + TM[1]*G_major[1] + TM[2]*G_major[2] + TM[3]*G_major[3];
	minor_mag = TM[0]*G_minor[0] + TM[1]*G_minor[1] + TM[2]*G_minor[2] + TM[3]*G_minor[3];

	//minor = dr | major;
	minor[X] = dr[Y]*major[Z] - dr[Z]*major[Y];
	minor[Y] = dr[Z]*major[X] - dr[X]*major[Z];
	minor[Z] = dr[X]*major[Y] - dr[Y]*major[X];

	mag = major_mag/(GLfloat)(sqrt(DOT3D(major, major)));

	major[X] *= mag;
	major[Y] *= mag;
	major[Z] *= mag;

	mag = minor_mag/(GLfloat)(sqrt(DOT3D(minor, minor)));

	minor[X] *= mag;
	minor[Y] *= mag;
	minor[Z] *= mag;

	// Increment the spline position
	t += dt;

	return TRUE;
}

BOOL moleculeSpline::NextRibbon(GLfloat r[], GLfloat major[], GLfloat minor[], 
								int &index, int cur_step)
{
	// Compute a Catmull-Rom spline. See "Computer Graphics Principles and Pratice"
	// by Foley, van Dam, Feiner and Hughes. The spline control parameter form is
	// taken from the VMD program. The actual value of the spline parameter is
	// under user control.
	//
	// The spline has the form:
	// Q(t) = TM(t) . G

	if(t >= 1.0f){
		update = TRUE;
		cur_knot ++;
		t = 0.0f;
	}

	if((cur_knot == num_knots - 1) || (num_knots < 4))
		return FALSE;

	int i;
	GLfloat mag, major_mag, minor_mag;
	GLfloat b_dot_a;
	amino *amino_ptr;
	array<amino> &sequence_ref = mol->secondaryStruc[chain].sequence;

	// Unlike the NextTrace() function, here we also compute a spline through
	// the carbonyl oxygens and the major and minor axes of the ellipse cross
	// section for each animo acid. The carbonyl oxygens provide guide coordinates
	// for aligning the major axis of the ribbon. This follows the work presented in
	// M. Carson (1997) Ribbons. Methods in Enzymology 277:493-505,
	// R.M. Sweet and C.W. Carter, eds, Academic Press.
    // M.Carson (1991) Ribbons 2.0, J.Appl.Cryst. 24:958-961  or
    // M.Carson (1987) Ribbon Models of Macromolecules, J.Mol.Graphics, 5:103-106.
	// (note that a different interpolation scheme is used).
	if(update){
		if(cur_knot == 0){
			amino_ptr = &(sequence_ref[2]);

			memcpy(G_c[3], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));

			memcpy(G_g[3], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
				3*sizeof(float));

			G_major[3] = amino_ptr->major;
			G_minor[3] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[1]);

			memcpy(G_c[2], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));

			memcpy(G_g[2], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
				3*sizeof(float));

			G_major[2] = amino_ptr->major;
			G_minor[2] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[0]);

			memcpy(G_c[1], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));
			
			memcpy(G_g[1], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
				3*sizeof(float));

			G_c[0][0] = 2.0f*G_c[1][0] - G_c[2][0];
			G_c[0][1] = 2.0f*G_c[1][1] - G_c[2][1];
			G_c[0][2] = 2.0f*G_c[1][2] - G_c[2][2];

			G_g[0][0] = 2.0f*G_g[1][0] - G_g[2][0];
			G_g[0][1] = 2.0f*G_g[1][1] - G_g[2][1];
			G_g[0][2] = 2.0f*G_g[1][2] - G_g[2][2];

			G_major[1] = amino_ptr->major;
			G_minor[1] = amino_ptr->minor;

			G_major[0] = G_major[1];
			G_minor[0] = G_minor[1];
		}
		else{
			amino_ptr = &(sequence_ref[cur_knot - 1]);

			memcpy(G_c[0], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));

			memcpy(G_g[0], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
				3*sizeof(float));

			G_major[0] = amino_ptr->major;
			G_minor[0] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[cur_knot]);

			memcpy(G_c[1], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));

			memcpy(G_g[1], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
				3*sizeof(float));

			G_major[1] = amino_ptr->major;
			G_minor[1] = amino_ptr->minor;

			amino_ptr = &(sequence_ref[cur_knot + 1]);

			memcpy(G_c[2], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
				3*sizeof(float));

			memcpy(G_g[2], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
				3*sizeof(float));

			G_major[2] = amino_ptr->major;
			G_minor[2] = amino_ptr->minor;

			if(cur_knot == num_knots - 2){
				G_c[3][0] = 2.0f*G_c[2][0] - G_c[1][0];
				G_c[3][1] = 2.0f*G_c[2][1] - G_c[1][1];
				G_c[3][2] = 2.0f*G_c[2][2] - G_c[1][2];

				G_g[3][0] = 2.0f*G_g[2][0] - G_g[1][0];
				G_g[3][1] = 2.0f*G_g[2][1] - G_g[1][1];
				G_g[3][2] = 2.0f*G_g[2][2] - G_g[1][2];

				G_major[3] = 2.0f*G_major[2] - G_major[1];
				G_minor[3] = 2.0f*G_minor[2] - G_minor[1];
			}
			else{
				amino_ptr = &(sequence_ref[cur_knot + 2]);

				memcpy(G_c[3], &(mol->traj[cur_step][3*amino_ptr->c_alpha_index]), 
					3*sizeof(float));

				memcpy(G_g[3], &(mol->traj[cur_step][3*amino_ptr->o_index]), 
					3*sizeof(float));
				
				G_major[3] = amino_ptr->major;
				G_minor[3] = amino_ptr->minor;
			}
		}

		// Flip carbonyl guide coordinates as needed
		// Guide One
		for(i = 0;i < 3;i++){
			A[i] = G_g[0][i] - G_c[0][i];
			B[i] = G_c[1][i] - G_c[0][i];
		}
		
		// The following triple product:
		// guide_ref = (B | A) | B
		// is equivalent to: A(B*B) - B(B*A)

		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		if(cur_knot == 0){
			guide_ref[0] = A[0]*mag - B[0]*b_dot_a;
			guide_ref[1] = A[1]*mag - B[1]*b_dot_a;
			guide_ref[2] = A[2]*mag - B[2]*b_dot_a;

			mag = (GLfloat)(sqrt(DOT3D(guide_ref, guide_ref)));

			guide_ref[X] /= mag;
			guide_ref[Y] /= mag;
			guide_ref[Z] /= mag;

			for(i = 0;i < 3;i++){
				G_g[0][i] = G_c[0][i] + guide_ref[i];
			}
		}
		else{
			tmp[0] = A[0]*mag - B[0]*b_dot_a;
			tmp[1] = A[1]*mag - B[1]*b_dot_a;
			tmp[2] = A[2]*mag - B[2]*b_dot_a;
			
			mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

			tmp[X] /= mag;
			tmp[Y] /= mag;
			tmp[Z] /= mag;

			if(DOT3D(guide_ref, tmp) < 0.0f){
				G_g[0][X] = G_c[0][X] - tmp[X];
				G_g[0][Y] = G_c[0][Y] - tmp[Y];
				G_g[0][Z] = G_c[0][Z] - tmp[Z];
			}
			else{
				G_g[0][X] = G_c[0][X] + tmp[X];
				G_g[0][Y] = G_c[0][Y] + tmp[Y];
				G_g[0][Z] = G_c[0][Z] + tmp[Z];
			}
		}

		// Guide Two
		for(i = 0;i < 3;i++){
			A[i] = G_g[1][i] - G_c[1][i];
			B[i] = G_c[2][i] - G_c[1][i];
		}
		
		//tmp = (B | A) | B;
		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
		
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[1][X] = G_c[1][X] - tmp[X];
			G_g[1][Y] = G_c[1][Y] - tmp[Y];
			G_g[1][Z] = G_c[1][Z] - tmp[Z];

			// Save the current reference guide orientation
			// (guide two is used for orienting all spline
			// segments).
			guide_ref[X] = - tmp[X];
			guide_ref[Y] = - tmp[Y];
			guide_ref[Z] = - tmp[Z];
		}
		else{
			G_g[1][X] = G_c[1][X] + tmp[X];
			G_g[1][Y] = G_c[1][Y] + tmp[Y];
			G_g[1][Z] = G_c[1][Z] + tmp[Z];

			// Save the current reference guide orientation
			// (guide two is used for orienting all spline
			// segments).
			guide_ref[X] = tmp[X];
			guide_ref[Y] = tmp[Y];
			guide_ref[Z] = tmp[Z];
		}


		// Guide Three
		for(i = 0;i < 3;i++){
			A[i] = G_g[2][i] - G_c[2][i];
			B[i] = G_c[3][i] - G_c[2][i];
		}

		//tmp = (B | A) | B;
		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
			
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[2][X] = G_c[2][X] - tmp[X];
			G_g[2][Y] = G_c[2][Y] - tmp[Y];
			G_g[2][Z] = G_c[2][Z] - tmp[Z];
		}
		else{
			G_g[2][X] = G_c[2][X] + tmp[X];
			G_g[2][Y] = G_c[2][Y] + tmp[Y];
			G_g[2][Z] = G_c[2][Z] + tmp[Z];
		}

		// Guide Four
		for(i = 0;i < 3;i++){
			A[i] = G_c[3][i] - G_g[3][i];
			B[i] = G_c[2][i] - G_c[3][i];
		}

		//tmp = (B | A) | B;
		mag = DOT3D(B, B);
		b_dot_a = DOT3D(B, A);

		tmp[0] = A[0]*mag - B[0]*b_dot_a;
		tmp[1] = A[1]*mag - B[1]*b_dot_a;
		tmp[2] = A[2]*mag - B[2]*b_dot_a;
			
		mag = (GLfloat)(sqrt(DOT3D(tmp, tmp)));

		tmp[X] /= mag;
		tmp[Y] /= mag;
		tmp[Z] /= mag;

		if(DOT3D(guide_ref, tmp) < 0.0f){
			G_g[3][X] = G_c[3][X] - tmp[X];
			G_g[3][Y] = G_c[3][Y] - tmp[Y];
			G_g[3][Z] = G_c[3][Z] - tmp[Z];
		}
		else{
			G_g[3][X] = G_c[3][X] + tmp[X];
			G_g[3][Y] = G_c[3][Y] + tmp[Y];
			G_g[3][Z] = G_c[3][Z] + tmp[Z];
		}

		update = !update;
	}

	TM[0] = -t*inv_gamma*(1.0f + t*(-2.0f + t));
	TM[1] = -inv_gamma*(-gamma + t*t*(3.0f*gamma - 1.0f + t*(1.0f - 2.0f*gamma)));
	TM[2] = t*inv_gamma*(1.0f + t*(3.0f*gamma - 2.0f + t*(1.0f - 2.0f*gamma)));
	TM[3] = t*t*inv_gamma*(t - 1.0f);	

	// The center of the ellipse
	//r = TM*G_c;
	r[0] = TM[0]*G_c[0][0] + TM[1]*G_c[1][0] + TM[2]*G_c[2][0] + TM[3]*G_c[3][0];
	r[1] = TM[0]*G_c[0][1] + TM[1]*G_c[1][1] + TM[2]*G_c[2][1] + TM[3]*G_c[3][1];
	r[2] = TM[0]*G_c[0][2] + TM[1]*G_c[1][2] + TM[2]*G_c[2][2] + TM[3]*G_c[3][2];

	index = sequence_ref[cur_knot].c_alpha_index;

	// Compute the interpolated ribbon size (both major and minor axes
	major_mag = TM[0]*G_major[0] + TM[1]*G_major[1] + TM[2]*G_major[2] + TM[3]*G_major[3]
		+ RIBBON_SIZE_OFFSET;
	minor_mag = TM[0]*G_minor[0] + TM[1]*G_minor[1] + TM[2]*G_minor[2] + TM[3]*G_minor[3]
		+ RIBBON_SIZE_OFFSET;

	// Compute the major and minor axes of the ellipse
	//B = TM*G_g - r
	major[X] = TM[0]*G_g[0][0] + TM[1]*G_g[1][0] + TM[2]*G_g[2][0] + TM[3]*G_g[3][0] - r[X];
	major[Y] = TM[0]*G_g[0][1] + TM[1]*G_g[1][1] + TM[2]*G_g[2][1] + TM[3]*G_g[3][1] - r[Y];
	major[Z] = TM[0]*G_g[0][2] + TM[1]*G_g[1][2] + TM[2]*G_g[2][2] + TM[3]*G_g[3][2] - r[Z];

	// Compute the derivative of the TM vector with respect to t
	TM[0] = -inv_gamma*(1.0f - t*(4.0f - 3.0f*t));
	TM[1] = t*(-6.0f + 2.0f*inv_gamma + t*(-3.0f*inv_gamma + 6.0f));
	TM[2] = inv_gamma + t*(6.0f - 4.0f*inv_gamma + t*(3.0f*inv_gamma - 6.0f));
	TM[3] = inv_gamma*t*(3.0f*t - 2.0f);

	//dr = d(TM*G_c)/dt
	dr[X] = TM[0]*G_c[0][0] + TM[1]*G_c[1][0] + TM[2]*G_c[2][0] + TM[3]*G_c[3][0];
	dr[Y] = TM[0]*G_c[0][1] + TM[1]*G_c[1][1] + TM[2]*G_c[2][1] + TM[3]*G_c[3][1];
	dr[Z] = TM[0]*G_c[0][2] + TM[1]*G_c[1][2] + TM[2]*G_c[2][2] + TM[3]*G_c[3][2];

	// Compute a TM vector for interpolating ribbon width. This has the same form as the TM vector
	// used above, but has gamma == 2.0 (for a nice looking ribbon).
	TM[0] = -t*0.5f*(1.0f + t*(-2.0f + t));
	TM[1] = -0.5f*(-2.0f + t*t*(5.0f - 3.0f*t));
	TM[2] = t*0.5f*(1.0f + t*(4.0f - 3.0f*t));
	TM[3] = t*t*0.5f*(t - 1.0f);	

	// Compute the interpolated ribbon size (both major and minor axes
	major_mag = TM[0]*G_major[0] + TM[1]*G_major[1] + TM[2]*G_major[2] + TM[3]*G_major[3];
	minor_mag = TM[0]*G_minor[0] + TM[1]*G_minor[1] + TM[2]*G_minor[2] + TM[3]*G_minor[3];

	//minor = dr | major;
	minor[X] = dr[Y]*major[Z] - dr[Z]*major[Y];
	minor[Y] = dr[Z]*major[X] - dr[X]*major[Z];
	minor[Z] = dr[X]*major[Y] - dr[Y]*major[X];

	mag = major_mag/(GLfloat)(sqrt(DOT3D(major, major)));

	major[X] *= mag;
	major[Y] *= mag;
	major[Z] *= mag;

	mag = minor_mag/(GLfloat)(sqrt(DOT3D(minor, minor)));

	minor[X] *= mag;
	minor[Y] *= mag;
	minor[Z] *= mag;

	// Increment the spline position
	t += dt;

	return TRUE;
}

void CQmolView::OnRenderLineWidth1() 
{
	line_width = 1.0;

	postRedisplay();
}

void CQmolView::OnUpdateRenderLineWidth1(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(line_width == 1.0f);
}

void CQmolView::OnRenderLineWidth2() 
{
	line_width = 2.0;

	postRedisplay();
}

void CQmolView::OnUpdateRenderLineWidth2(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(line_width == 2.0f);
}

void CQmolView::OnRenderLineWidth3() 
{
	line_width = 3.0;

	postRedisplay();
}

void CQmolView::OnUpdateRenderLineWidth3(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(line_width == 3.0f);
}

void CQmolView::OnRenderLineWidth4() 
{
	line_width = 4.0;

	postRedisplay();
}

void CQmolView::OnUpdateRenderLineWidth4(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(line_width == 4.0f);
}

void CQmolView::OnRenderLineWidth5() 
{
	line_width = 5.0;

	postRedisplay();
}

void CQmolView::OnUpdateRenderLineWidth5(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(line_width == 5.0f);
}

void CQmolView::OnRenderRibbonShowWire() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
				
	while(tmpMol = next())
		if((*tmpMol)->isSelected)
			(*tmpMol)->showWireFrame = !(*tmpMol)->showWireFrame;

	postRedisplay();
}

void CQmolView::OnUpdateRenderRibbonShowWire(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	if(ptrMol){
		pCmdUI->SetCheck(ptrMol->showWireFrame);
	}
	else{
		pCmdUI->Enable(FALSE);
	}
}

void CQmolView::OnOptionsStereo() 
{
	if(displayStereo == QM_STEREO_NONE){
		displayStereo = QM_STEREO_SPLIT_SCREEN;
	}
	else{
		displayStereo = QM_STEREO_NONE;
	}
	
	SetupGLView(viewWidth, viewHeight);
	postRedisplay();
}

void CQmolView::OnUpdateOptionsStereo(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(displayStereo == QM_STEREO_SPLIT_SCREEN);
}


void CQmolView::OnFileTrajExport() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	PauseAnimation(TRUE);
	
	// Start counting at 1
	CExportTrajDialog exTrajDlg(1, pDoc->activeMol->steps);

	if(exTrajDlg.DoModal() == IDOK){
		pDoc->OnFileTrajExport(
			LPCSTR(exTrajDlg.m_file_name),	// The file name
			exTrajDlg.m_file_type,			// The file type
			exTrajDlg.m_start_traj,			// The starting step
			exTrajDlg.m_stop_traj			// The ending step
			);
	}

	PauseAnimation(FALSE);
}

void CQmolView::OnUpdateFileTrajExport(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol && (ptrMol->steps > 0));
}

BOOL CQmolView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	// Allow the user to zoom using a wheel mouse
	ScaleWheel(zDelta/120);
	
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CQmolView::OnOptionsMouseDec() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	referenceFrame.IncreaseBallSize();

	while(tmpMol = next())
		(*tmpMol)->IncreaseBallSize();
}

void CQmolView::OnOptionsMouseInc() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	referenceFrame.DecreaseBallSize();

	while(tmpMol = next())
		(*tmpMol)->DecreaseBallSize();
}

void CQmolView::OnOptionsMouseDef() 
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	referenceFrame.DefaultBallSize();

	while(tmpMol = next())
		(*tmpMol)->DefaultBallSize();
}

UINT CQmolView::GetClipFormat()
{
	// Note that the clipboard must be open before calling this function
	UINT clipFormat = 0;

	while(clipFormat = ::EnumClipboardFormats(clipFormat)){
		if(clipFormat == CF_TEXT)
			return CF_TEXT;

		if(clipFormat == CF_OEMTEXT)
			return CF_OEMTEXT;
	}

	return 0;
}

void CQmolView::OnEditPaste() 
{
	UINT clipFormat;

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Open the clipboard
	if(!OpenClipboard()){
		AfxMessageBox("Unable to open clipboard!");
		return;
	}
	
	// Did we find an allowed import format?
	if((clipFormat = GetClipFormat()) != 0){
		HANDLE clipData = NULL;
		
		clipData = GetClipboardData(clipFormat);

		if(clipData == NULL){
			AfxMessageBox("Can't get valid clipboard handle!");
			CloseClipboard();
			return;
		}

		// Do we need to lock memory from the clipboard?
		GlobalLock(clipData);

		// Make a list to store clipboard data
		list<float> clip_data;
	
		// Read in clipboard data to a list of floats
		if(!ParseClipData(clipData, clip_data)){
			AfxMessageBox("Unable to read clipboard data a column of ASCII numbers.");
			GlobalUnlock(clipData);
			CloseClipboard();
			return;
		}
		
		if(clip_data.Length() == 0){
			// Empty clipboard
			AfxMessageBox("The clipboard is empty - nothing to paste");
			GlobalUnlock(clipData);
			CloseClipboard();
			return;
		}
		
		int i, j, k, count;

		// Count the number of alpha carbons
		count = 0;
		for(i = 0;i < pDoc->activeMol->secondaryStruc.Length();i++){
			count += pDoc->activeMol->secondaryStruc[i].sequence.Length();
		}
		
		CPasteDialog pDlg(clip_data.Length(), 
						  pDoc->activeMol->NumAtoms(), 
						  count);

		if(pDlg.DoModal() == IDOK){
			// Update the molecule
			if(pDlg.m_paste_as == CPasteDialog::PASTE_AS_T){
				switch(pDlg.paste_to){
					case CPasteDialog::PASTE_TO_ALL:
						for(i = 0;i < pDoc->activeMol->NumAtoms();i++){
							pDoc->activeMol->atoms[i].temperature = (float)(fabs(clip_data[i]));
						}

						break;
					case CPasteDialog::PASTE_TO_SINGLE:
						i = pDoc->activeMol->IdToIndex(pDlg.m_single_atom_id);

						if(i == -1){
							AfxMessageBox("Invalid atom id, can't paste data");
							GlobalUnlock(clipData);
							return;
						}
						else{
							pDoc->activeMol->atoms[i].temperature = (float)(fabs(clip_data[0]));
						}

						break;
					case CPasteDialog::PASTE_TO_RANGE:
						count = 0;

						for(i = pDlg.m_atom_range_low;i < pDlg.m_atom_range_high;i++){
							j = pDoc->activeMol->IdToIndex(i);

							if(j != -1){
								pDoc->activeMol->atoms[j].temperature = 
									(float)(fabs(clip_data[count]));
								count ++;
							}
						}

						break;
					case CPasteDialog::PASTE_TO_CA:

						// Assign the temperature factor for all atoms in a
						// residue according to the alpha carbon temperature
						count = 0;
						for(i = 0;i < pDoc->activeMol->secondaryStruc.Length();i++){
							for(j = 0;j < pDoc->activeMol->secondaryStruc[i].sequence.Length();j++){
								list<int> &id_list_ref = 
									pDoc->activeMol->secondaryStruc[i].sequence[j].atom_id;

								for(k = 0;k < id_list_ref.Length();k++){
									pDoc->activeMol->atoms[id_list_ref[k]].temperature = 
										(float)(fabs(clip_data[count]));
								}

								count ++;
							}
						}
						break;
				};

				pDoc->activeMol->UpdateTemperature();
				
				// Do we need to update atom radii?
				if(pDoc->activeMol->atom_radius_method == QM_RADIUS_T){
					pDoc->activeMol->SetAtomicRadiiScheme(QM_RADIUS_T, 
						0,		// Value not used
						0,		// Value not used
						0.0f);	// Value not used
				}
			}
			else{ // CPasteDialog::PASTE_AS_OCC
				switch(pDlg.paste_to){
					case CPasteDialog::PASTE_TO_ALL:
						for(i = 0;i < pDoc->activeMol->NumAtoms();i++){
							pDoc->activeMol->atoms[i].occupancy = (float)(fabs(clip_data[i]));
						}

						break;
					case CPasteDialog::PASTE_TO_SINGLE:
						i = pDoc->activeMol->IdToIndex(pDlg.m_single_atom_id);

						if(i == -1){
							AfxMessageBox("Invalid atom id, can't paste data");
							GlobalUnlock(clipData);
							return;
						}
						else{
							pDoc->activeMol->atoms[i].occupancy = (float)(fabs(clip_data[0]));
						}

						break;
					case CPasteDialog::PASTE_TO_RANGE:
						count = 0;

						for(i = pDlg.m_atom_range_low;i < pDlg.m_atom_range_high;i++){
							j = pDoc->activeMol->IdToIndex(i);

							if(j != -1){
								pDoc->activeMol->atoms[j].occupancy = 
									(float)(fabs(clip_data[count]));
								count ++;
							}
						}

						break;
					case CPasteDialog::PASTE_TO_CA:
						// Assign the temperature factor for all atoms in a
						// residue according to the alpha carbon temperature
						count = 0;
						for(i = 0;i < pDoc->activeMol->secondaryStruc.Length();i++){
							for(j = 0;j < pDoc->activeMol->secondaryStruc[i].sequence.Length();j++){
								list<int> &id_list_ref = 
									pDoc->activeMol->secondaryStruc[i].sequence[j].atom_id;

								for(k = 0;k < id_list_ref.Length();k++){
									pDoc->activeMol->atoms[id_list_ref[k]].occupancy = 
										(float)(fabs(clip_data[count]));
								}

								count ++;
							}
						}
						break;
				};

				pDoc->activeMol->UpdateOccupancy();

				if(pDoc->activeMol->atom_radius_method == QM_RADIUS_OCC){
					pDoc->activeMol->SetAtomicRadiiScheme(QM_RADIUS_OCC, 
						0,		// Value not used
						0,		// Value not used
						0.0f);	// Value not used
				}
			}

			// Update various size and coloring methods to reflect any changes in
			// temperature or occupancy
			
			// Set atom colors according to the current color scheme
			if(m_qmColorDlg != NULL){
				// Update the color scheme
				m_qmColorDlg->ApplyCurrColor();
			}

			// Set the current ribbon dimensions (according to the current displayType).
			pDoc->activeMol->SetRibbonSize(pDoc->activeMol->displayType);

			// Redraw the screen
			postRedisplay();
		}

		GlobalUnlock(clipData);
	}
	else{
		AfxMessageBox("Sorry -- only ASCII text can be pasted!");
	}

	// Close the clipboard
	CloseClipboard();
}

void CQmolView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	// Open the clipboard
	if(!OpenClipboard()){
		// We failed to open the clipboard
		pCmdUI->Enable(FALSE);
	}

	molecule *ptrMol = GetActiveMol();

	// Does the clipboard contain a valid format and do we have a 
	// molecule loaded?
	pCmdUI->Enable((GetClipFormat() != 0) && ptrMol && (ptrMol->NumAtoms() > 0));

	// Close the clipboard
	CloseClipboard();
}

BOOL CQmolView::ParseClipData(HANDLE clipData, list<float> &clip_data)
{
	// We can't work with a NULL pointer!
	if(clipData == NULL)
		return FALSE;
	
	BOOL negative_value_warning = TRUE;

	// View the clipboard data as a '\0' terminated string of floats
	int len = strlen((char*)(clipData));

	char *clip_string = new char [len + 1];
	char *ptr;

	strcpy(clip_string, (char*)(clipData));

	// Terminate string
	clip_string[len] = '\0';

	ptr = strtok(clip_string, " \t\r\n");

	while(ptr != NULL){
		double tmp = atof(ptr);

		if(!_finite(tmp)){
			AfxMessageBox("Warning - non-numeric value found in clipboard");
			delete [] clip_string;

			return FALSE;
		}

		if(negative_value_warning && (tmp < 0.0)){
			AfxMessageBox("Warning - negative value found in clipboard");

			// One warning is enought
			negative_value_warning = FALSE;
		}

		clip_data.Push((float)(tmp));

		ptr = strtok(NULL, " \t\r\n");
	}

	delete [] clip_string;

	return TRUE;
}
void CQmolView::OnRenderClipColor() 
{
	if(m_flexClrDlg->GetSafeHwnd() == 0){
		// Make sure that the dialog displays the current
		// background color.
		m_flexClrDlg->SetRGBA(
			(int)(clipPlaneClr[0]*255.0f),
			(int)(clipPlaneClr[1]*255.0f),
			(int)(clipPlaneClr[2]*255.0f),
			(int)(clipPlaneClr[3]*255.0f));

		m_flexClrDlg->Create(CLIP_PLANE_CLR);
	}
}

void CQmolView::OnRenderClipShow() 
{
	displayClipPlane = !displayClipPlane;

	if(displayClipPlane){
		clipper.EnableDisplay();
	}
	else{
		clipper.DisableDisplay();
	}

	// Redraw the screen 
	postRedisplay();
}

void CQmolView::OnUpdateRenderClipShow(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(displayClipPlane);
}

void CQmolView::OnRenderClipView() 
{
	clipView = !clipView;

	if(clipView){
		clipper.Enable();
	}
	else{
		clipper.Disable();
	}

	// Redraw the screen 
	postRedisplay();
}

void CQmolView::OnUpdateRenderClipView(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(clipView);
}

void CQmolView::OnRenderClipSurface() 
{
	clipSurface = !clipSurface;

	// Redraw the screen 
	postRedisplay();
}

void CQmolView::OnUpdateRenderClipSurface(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(clipSurface);
}

void CQmolView::OnRenderClipCenter() 
{
	// Center the clipping plane in the window viewport
	clipper.CenterClip();

	if(clipView || clipSurface){
		// Redraw the screen if the clipping plane is 
		// currently displayed
		postRedisplay();
	}
}

void CQmolView::OnRenderSurfaceOpt() 
{
	// Create the surface color/options dialog
	if(m_surfDlg->GetSafeHwnd() == 0){
		molecule *ptrMol = GetActiveMol();

		ASSERT(ptrMol != NULL);

		int r, g, b, a;

		// Bring up the rendering options on top
		m_surfDlg->SetRenderFocus();

		// Set the color options
		if(ptrMol->surf.ColorByAtom()){
			m_surfDlg->SetColorStyle(CSurfColorProperty::COLOR_BY_ATOM);
		}
		else{
			m_surfDlg->SetColorStyle(CSurfColorProperty::COLOR_USER);
		}
		
		m_surfDlg->SetAlpha(ptrMol->surf.GetTransparency());
		ptrMol->surf.GetSurfColor(r, g, b, a);
		m_surfDlg->SetRGBA(r, g, b, a);

		// Set the rendering options
		m_surfDlg->SetSurfQuality(ptrMol->surf.GetSurfQuality());
		m_surfDlg->SetSurfType(ptrMol->surf.GetSurfType());
		m_surfDlg->SetProbeRadius(ptrMol->surf.GetProbeSphereRadius());
		m_surfDlg->UseSolventAtoms(ptrMol->surf.UseSolventAtoms());

		m_surfDlg->Create(this);
	}
}

void CQmolView::OnUpdateRenderSurfaceOpt(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol != NULL);
}

LRESULT CQmolView::OnSurfaceOptMessage(WPARAM wParam, LPARAM lParam)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;

	switch(wParam){
		case ID_SURF_OK:
			// First apply changes
			OnSurfaceOptMessage(ID_SURF_APPLY, 0);

			m_surfDlg->DestroyWindow();
			break;
		case ID_SURF_CANCEL:
			m_surfDlg->DestroyWindow();
			break;
		case ID_SURF_APPLY:

			// Apply color changes:
			while(tmpMol = next())
				if((*tmpMol)->isSelected){
					if(m_surfDlg->GetColorStyle() == CSurfColorProperty::COLOR_BY_ATOM){
						(*tmpMol)->surf.ColorByAtom(TRUE);
						
						(*tmpMol)->surf.SetTransparency(m_surfDlg->GetAlpha());
					}
					else{ //CSurfColorProperty::COLOR_USER
						(*tmpMol)->surf.ColorByAtom(FALSE);
						
						int r, g, b, a;
						m_surfDlg->GetRGBA(r, g, b, a);
						(*tmpMol)->surf.SetSurfColor(r, g, b, a);
					}
					
					// Apply surface option changes:
					(*tmpMol)->surf.SetSurfQuality(m_surfDlg->GetSurfQuality());
					(*tmpMol)->surf.SetSurfType(m_surfDlg->GetSurfType());
					(*tmpMol)->surf.SetProbeSphereRadius(m_surfDlg->GetProbeRadius());
					(*tmpMol)->surf.UseSolventAtoms(m_surfDlg->UseSolventAtoms());
				}

			// Redraw the screen
			postRedisplay();

			break;
	};

	return 0L;
}

LRESULT CQmolView::OnSurfaceBuildMessage(WPARAM wParam, LPARAM lParam)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	ListIterator<molecule *> next(pDoc->molList);
	molecule **tmpMol;
	
	try{
		// Set the hourglass cursor - this may take a while
		::SetCursor(LoadCursor(NULL, IDC_WAIT));

		while(tmpMol = next())
			if((*tmpMol)->isSelected){
				(*tmpMol)->surf.BuildSurface(&((*tmpMol)->atoms), 
					&((*tmpMol)->traj), (*tmpMol)->current_step);
			}
	}		
	catch(QmolError &e){
		// Restore the cursor
		::SetCursor(LoadCursor(NULL, IDC_ARROW));

		AfxMessageBox(e.message());
		
		// Clean up the progress bar
		CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
		if(pFrame){
			pFrame->CloseProgress();
		}
		
	}

	// Restore the cursor
	::SetCursor(LoadCursor(NULL, IDC_ARROW));

	// Redraw the molecule
	postRedisplay();

	return 0L;
}


void CQmolView::OnOptionsSurfaceColor() 
{
	// Create the surface color/options dialog
	if(m_surfDlg->GetSafeHwnd() == 0){
		molecule *ptrMol = GetActiveMol();

		ASSERT(ptrMol != NULL);

		int r, g, b, a;

		// Bring up the color options on top
		m_surfDlg->SetColorFocus();

		// Set the color options
		if(ptrMol->surf.ColorByAtom()){
			m_surfDlg->SetColorStyle(CSurfColorProperty::COLOR_BY_ATOM);
		}
		else{
			m_surfDlg->SetColorStyle(CSurfColorProperty::COLOR_USER);
		}

		m_surfDlg->SetAlpha(ptrMol->surf.GetTransparency());
		ptrMol->surf.GetSurfColor(r, g, b, a);
		m_surfDlg->SetRGBA(r, g, b, a);

		// Set the rendering options
		m_surfDlg->SetSurfQuality(ptrMol->surf.GetSurfQuality());
		m_surfDlg->SetSurfType(ptrMol->surf.GetSurfType());
		m_surfDlg->SetProbeRadius(ptrMol->surf.GetProbeSphereRadius());
		m_surfDlg->UseSolventAtoms(ptrMol->surf.UseSolventAtoms());

		m_surfDlg->Create(this);
	}
}

void CQmolView::OnUpdateOptionsSurfaceColor(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();
	
	pCmdUI->Enable(ptrMol != NULL);
}

void CQmolView::OnMeasureSurfaceResidue() 
{
	molecule *ptrMol = GetActiveMol();

	ASSERT(ptrMol != NULL);

	BOOL removeSurface = FALSE;

	if(!ptrMol->surf.ValidSurface()){
		try{
			// Set the hourglass cursor - this may take a while
			::SetCursor(LoadCursor(NULL, IDC_WAIT));

			removeSurface = TRUE;
			ptrMol->surf.BuildSurface(&(ptrMol->atoms), 
				&(ptrMol->traj), ptrMol->current_step);
		}
		catch(QmolError &e){
			// Restore the cursor
			::SetCursor(LoadCursor(NULL, IDC_ARROW));

			AfxMessageBox(e.message());
			
			// Clean up the progress bar
			CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
			if(pFrame){
				pFrame->CloseProgress();
			}

		}

		// Restore the cursor
		::SetCursor(LoadCursor(NULL, IDC_ARROW));
	}

	array <float> area(ptrMol->NumAtoms());
	CString area_info;
	CString total_info;
	float total_area = 0.0f;

	ptrMol->surf.GetAtomSurfaceArea(area);

	// Do we need to clean up and remove the surface?
	if(removeSurface){
		ptrMol->surf.InvalidateSurface();
	}

	for(int i = 0;i < ptrMol->secondaryStruc.Length();i++){

		for(int j = 0;j < ptrMol->secondaryStruc[i].sequence.Length();j++){

			CString residueArea;
			float tmpArea = 0.0f;

			for(int k = 0;k < ptrMol->secondaryStruc[i].sequence[j].atom_id.Length();k++){
				tmpArea += area[ptrMol->secondaryStruc[i].sequence[j].atom_id[k]];
			}

			residueArea.Format("%d\t%f\r\n", ptrMol->secondaryStruc[i].sequence[j].resSeq, 
				tmpArea);
		
			area_info += residueArea;

			total_area += tmpArea;
		}
	}

	total_info.Format("###################################\n# Total Surface Area = %f", 
		total_area);

	area_info += total_info;

	CTextOutput textOutDlg("Residue Surface Areas (in Angstrom^2)", area_info);

	textOutDlg.DoModal();
}

void CQmolView::OnUpdateMeasureSurfaceResidue(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol && (ptrMol->secondaryStruc.Length() != 0));
}

void CQmolView::OnMeasureSurfaceAtom() 
{
	molecule *ptrMol = GetActiveMol();

	ASSERT(ptrMol != NULL);

	BOOL removeSurface = FALSE;

	if(!ptrMol->surf.ValidSurface()){
		try{
			// Set the hourglass cursor - this may take a while
			::SetCursor(LoadCursor(NULL, IDC_WAIT));

			removeSurface = TRUE;
			ptrMol->surf.BuildSurface(&(ptrMol->atoms), 
				&(ptrMol->traj), ptrMol->current_step);
		}
		catch(QmolError &e){
			// Restore the cursor
			::SetCursor(LoadCursor(NULL, IDC_ARROW));

			AfxMessageBox(e.message());
			
			// Clean up the progress bar
			CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
			if(pFrame){
				pFrame->CloseProgress();
			}

		}

		// Restore the cursor
		::SetCursor(LoadCursor(NULL, IDC_ARROW));
	}

	array <float> area(ptrMol->NumAtoms());
	CString area_info;
	CString total_info;
	float total_area = 0.0f;

	ptrMol->surf.GetAtomSurfaceArea(area);

	// Do we need to clean up and remove the surface?
	if(removeSurface){
		ptrMol->surf.InvalidateSurface();
	}

	for(int i = 0;i < ptrMol->NumAtoms();i++){
		CString atomArea;

		atomArea.Format("%d\t%f\r\n", ptrMol->atoms[i].id, area[i]);
		
		area_info += atomArea;

		total_area += area[i];
	}

	total_info.Format("###################################\n# Total Surface Area = %f", 
		total_area);

	area_info += total_info;

	CTextOutput textOutDlg("Atomic Surface Areas (in Angstrom^2)", area_info);

	textOutDlg.DoModal();
}

void CQmolView::OnUpdateMeasureSurfaceAtom(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol && (ptrMol->NumAtoms() != 0));
}

void CQmolView::OnOptionsDefault() 
{

	if(AfxMessageBox("About to return all settings to their default values.\r\n"
		"Continue?", MB_YESNO) == IDNO){
		return;
	}

	// Restore all default values. Don't use the values
	// stored in the registry!
	LoadDefaults(FALSE);

	// Re-initialize the openGL subsystem using the
	// newly restored default values.
	InitializeOpenGL();

	// Redraw the screen so that these changes take effect immediately
	postRedisplay();
}

void CQmolView::OnOptionsPurgeReg() 
{
	purgeRegistryEntries = !purgeRegistryEntries;
	
	if(!purgeRegistryEntries)
		return;

	// Make sure the user knows what they are getting into
	if(AfxMessageBox("Removing the QMol registry entries will\r\n"
		"return all settings to their default values.\r\n"
		"The list of most recently opened files will also be \r\n"
		"lost. Are you sure about this?", MB_YESNO) == IDNO){
		purgeRegistryEntries = FALSE;
	}
}

void CQmolView::OnUpdateOptionsPurgeReg(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(purgeRegistryEntries);
}

void CQmolView::UpdateStatusBar(SelectObj &atom)
{
	// Keep the user informed of the window size
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame){
		CQmolDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		// Make sure this is a valid object
#ifdef _DEBUG
		if(!pDoc->IsValidObj(atom)){
			throw QmolError("Invalid object in UpdateStatusBar",
				__FILE__, __LINE__);
		}
#endif // _DEBUG

		CString str;
		CStatusBar* pStatus = &pFrame->m_wndStatusBar;
		molecule *molPtr = atom.GetMol();
		int atomIndex = atom.GetIndex();

		// The atom number
		str.Format("%d", molPtr->atoms[atomIndex].id);
		pStatus->SetPaneText(ATOM_NUM_PANE, str);

		// The atom type
		str.Format("%s", molPtr->atoms[atomIndex].name);
		pStatus->SetPaneText(ATOM_ID_PANE, str);

		// The amino acid
		str.Format("%s%d", molPtr->atoms[atomIndex].resName, 
			molPtr->atoms[atomIndex].resSeq);
		pStatus->SetPaneText(AMINO_ACID_PANE, str);
	}
}

void CQmolView::OnOptionsMolSel() 
{
	if(m_molSelDlg->GetSafeHwnd() == 0){
		m_molSelDlg->Create();
	}
}

LRESULT CQmolView::OnMolSelectDialogMsg(WPARAM wParam, LPARAM lParam)
{
	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that we have a valid dialog
	ASSERT(m_molSelDlg != NULL);

	switch(wParam){
		case IDCANCEL:
			m_molSelDlg->DestroyWindow();
			break;
		case WM_MOL_SELECT_DELETE:
			{
				// Construct a list of selected molecule pointers from
				// the list of selected molecule indicies. This avoids 
				// needing to sort the list of molecule indicies. Note that
				// sorting would be the better solution!
				list<int> selList;
				list<molecule*> molList;

				m_molSelDlg->GetSelectedMol(selList);
				
				ListIterator<int> nextIndex(selList);
				int *indexPtr;
				
				while(indexPtr = nextIndex())
					molList.Push(pDoc->molList[*indexPtr]);
				
				ListIterator<molecule *> nextMol(molList);
				molecule **tmpMol;
				
				while(tmpMol = nextMol()){
					if(*tmpMol == pDoc->activeMol)
						pDoc->activeMol = NULL;
					
					for(int i = 0;i < pDoc->molList.Length();i++){
						if(pDoc->molList[i] == (*tmpMol)){
							// Remove the selected molecules from the molcule
							// list and delete them.
							pDoc->molList.Remove(i);
							delete  (*tmpMol);
						}
						
					}
				}
				
				if(pDoc->activeMol == NULL && !pDoc->molList.IsEmpty()){
					// Promote the first molecule remaining in the list to be the
					// active molecule
					pDoc->activeMol = pDoc->molList[0];
				}
				
				// Update hydrogen bonds as needed
				if(showHBonds){
					pDoc->UpdateHydrogenBonding();
				}

				// Update any measurements that may have involved the now deleted
				// molecule(s)
				pDoc->molMetric.Flush(pDoc->molList);

				// Update trajectory animation
				pDoc->UpdateTraj();

				if(pDoc->activeMol == NULL){
					// No molecules loaded, so stop animating
					animate = FALSE;
				}
				else{
					// The active molecule is always selected
					pDoc->activeMol->isSelected = TRUE;
				}

				if(m_seqDlg->GetSafeHwnd() != 0){
					// Reinitialize the structure dialog if it is 
					// currently visable.
					m_seqDlg->PostMessage(WM_SEQUENCE_RELOAD);
				}

				if(m_atomSelectDlg->GetSafeHwnd() != 0){
					// Reinitialize the atom selection dialog if it is 
					// currently visable.
					m_atomSelectDlg->PostMessage(WM_ATOM_SELECT_UPDATE);
				}

				m_molSelDlg->PostMessage(WM_MOL_SELECT_UPDATE);
				
				// Redraw the screen so that these changes take effect immediately
				postRedisplay();
			}
			break;
		case WM_MOL_SELECT_ACTIVE:
			{
				int molIndex = m_molSelDlg->GetSelectedMol();
				
				// Has the user selected a molecule?
				if(molIndex >= 0){
					int len = pDoc->molList.Length();

					for(int i = 0;i < len;i++){
						if(i == molIndex){
							pDoc->activeMol = pDoc->molList[molIndex];

							// The active molecule is always selected
							pDoc->activeMol->isSelected = TRUE;
						}
						else{
							// Unselect all other molecules
							pDoc->molList[i]->isSelected = FALSE;
						}
					}

					m_molSelDlg->PostMessage(WM_MOL_SELECT_UPDATE);

					if(m_seqDlg->GetSafeHwnd() != 0){
						// Reinitialize the structure dialog if it is 
						// currently visable.
						m_seqDlg->PostMessage(WM_SEQUENCE_RELOAD);
					}
				}
			}
			break;
		case WM_MOL_SELECT_DYNAMIC:
			{
				// Construct a list of selected molecules
				list<int> selList;
				
				m_molSelDlg->GetSelectedMol(selList);
				
				ListIterator<int> next(selList);
				int *ptr;
				
				while(ptr = next()){
					pDoc->molList[*ptr]->ToggleDynamic();
				}
				
				// The active molecule is always selected
				if(pDoc->activeMol)
					pDoc->activeMol->isSelected = TRUE;

				m_molSelDlg->PostMessage(WM_MOL_SELECT_UPDATE);
			}
			break;
		case WM_MOL_SELECT_SELECTION:
			{
				// Construct a list of selected molecules
				list<int> selList;
				m_molSelDlg->GetSelectedMol(selList);

				// Convert molecule indicies into molecule pointers
				ListIterator<int> next(selList);
				int *ptr;
					
				while(ptr = next())
					pDoc->molList[*ptr]->ToggleSelection();

				// The active molecule is always selected
				pDoc->activeMol->isSelected = TRUE;

				m_molSelDlg->PostMessage(WM_MOL_SELECT_UPDATE);
			}
			break;
		case WM_MOL_SELECT_CLONE:
			{
				// Construct a list of selected molecules
				list<int> selList;
				m_molSelDlg->GetSelectedMol(selList);

				// Convert molecule indicies into molecule pointers
				ListIterator<int> next(selList);
				int *ptr;
					
				while(ptr = next()){
					molecule *tmpMol = new molecule;
					
					*tmpMol = *(pDoc->molList[*ptr]);
					pDoc->molList.Push(tmpMol);
				}

				// The active molecule is always selected
				pDoc->activeMol->isSelected = TRUE;

				m_molSelDlg->PostMessage(WM_MOL_SELECT_UPDATE);
			}
			break;
	};

	return 0L;
}

void CQmolView::OnOptionsScaleFit() 
{
	scaleToFit = !scaleToFit;

	CQmolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Get the size of the largest dimension
	maxDim = pDoc->GetMaxDim(scaleToFit);
}

void CQmolView::OnUpdateOptionsScaleFit(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(scaleToFit);
}

LRESULT CQmolView::OnAtomSelectDialogMsg(WPARAM wParam, LPARAM lParam)
{
	// Currently, close is the only message!
	m_atomSelectDlg->DestroyWindow();

	return 0L;
}

void CQmolView::OnMoleculeAlign() 
{
	if(m_atomSelectDlg->GetSafeHwnd() == 0)
		m_atomSelectDlg->Create();
}

void CQmolView::OnUpdateMoleculeAlign(CCmdUI* pCmdUI) 
{
	molecule *ptrMol = GetActiveMol();

	pCmdUI->Enable(ptrMol && !m_atomSelectDlg->GetSafeHwnd());
}

